<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request as Req;
use App\Http\Controllers\Controller;
use Carbon\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use Illuminate\Support\Facades\Log;
use Illuminate\Bus\Batch;
use Illuminate\Support\Facades\Bus;
use GuzzleHttp\HandlerStack;
use Spatie\GuzzleRateLimiterMiddleware\RateLimiterMiddleware;

class OfferApiController extends Controller
{
    public function updateTakealotOffer($data)
    {
        try {
            if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
                $headers = ['Authorization' => 'Key ' . $data['takealot_api_key'], 'x-RateLimit-Limit' => $data['x-RateLimit-Limit'], 'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'], 'Content-Type' => 'application/json'];
                $client   = new Client(['base_uri' => $data['base_url'], 'verify' => false, 'headers' => $headers]);
                $response = $client->patch('offers/offer/ID' . $data['offer_id'], [
                    'body' => json_encode($data['data'])
                ]);

                if ($response->getStatusCode() == 200) {
                    $result = $response->getBody()->getContents();
                    if (!empty($result)) {

                        return response()->json(['status' => 'success', 'message' => 'Offer updated successfully.', 'data' => $result], 200);
                    } else {
                        return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returned data for updation.'], 500);
                    }
                } else {
                    // Logger('update offer response');
                    // Logger($response);
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Api key is required'], 429);
            }
        } catch (\Exception $e) {
            // Logger('update offer catch');
            // Logger($e);
            return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $e->getMessage()], $e->getCode());
        }
    }
    public function fetchSingleOffer($data)
    {
        try {
            if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
                $headers = ['Authorization' => 'Key ' . $data['takealot_api_key'], 'x-RateLimit-Limit' => $data['x-RateLimit-Limit'], 'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'], 'Content-Type' => 'application/json'];
                $client   = new Client(['base_uri' => $data['base_url'], 'verify' => false, 'headers' => $headers]);
                $response = $client->get('offers/offer/' . $data['offer_id']);

                if ($response->getStatusCode() == 200) {
                    $result = $response->getBody()->getContents();
                    if (!empty($result)) {

                        return response()->json(['status' => 'success', 'message' => 'Offer data fetched successfully.', 'data' => $result], 200);
                    } else {
                        return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returning data.'], 500);
                    }
                } else {
                    // Logger('fetch single offer response');
                    // Logger($response);
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Api key is required', 'data' => $e->getMessage()], $e->getCode());
            }
        } catch (\Exception $e) {
            // Logger('fetch single offer catch');
            // Logger($e);
            return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $e->getMessage()], $e->getCode());
        }
    }
    public function offersCount($data)
    {
        try {
            if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
                $headers = ['Authorization' => 'Key ' . $data['takealot_api_key'], 'x-RateLimit-Limit' => $data['x-RateLimit-Limit'], 'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'], 'Content-Type' => 'application/json'];
                $client   = new Client(['base_uri' => $data['base_url'], 'verify' => false, 'headers' => $headers]);
                $response = $client->get('offers/count');
                if ($response->getStatusCode() == 200) {
                    $result = $response->getBody()->getContents();
                    if (!empty($result)) {

                        return response()->json(['status' => 'success', 'message' => 'Offers counted successfully.', 'data' => $result], 200);
                    } else {
                        return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returning data.'], 500);
                    }
                } else {
                    // Logger('offer count response');
                    // Logger($response);
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Api key is required'], 429);
            }
        } catch (\Exception $e) {
            Logger('offers count catch');
            Logger($e);
            return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $e->getMessage()], $e->getCode());
        }
    }
    public function fetchAllOffers($data)
    {

        try {
            if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
                $headers = ['Authorization' => 'Key ' . $data['takealot_api_key'], 'x-RateLimit-Limit' => $data['x-RateLimit-Limit'], 'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'], 'Content-Type' => 'application/json'];
                $client   = new Client(['base_uri' => $data['base_url'], 'verify' => false, 'headers' => $headers]);
                $response = $client->get('offers?page_number=' . $data['page_number'] . '&page_size=' . $data['page_size']);
                if ($response->getStatusCode() == 200) {
                    $result = $response->getBody()->getContents();
                    if (!empty($result)) {

                        return response()->json(['status' => 'success', 'message' => 'Offer data fetched successfully.', 'data' => $result], 200);
                    } else {
                        return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returning data.'], 500);
                    }
                } else {
                    // Logger('all offer response');
                    // Logger($response);
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Api key is required'], 429);
            }
        } catch (\Exception $e) {
            // Logger('all offer catch');
            // Logger($e);
            return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $e->getMessage()], $e->getCode());
        }
    }
}
