<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request as Req;
use App\Http\Controllers\Controller;
use GuzzleHttp\Client;
use Illuminate\Log\Logger;

class OrdersApiController extends Controller
{
    public function fetchAllOrders($data)
    {
        $pageNo = !empty($data) && isset($data['page_number']) ? $data['page_number'] : 1;
        $pageSize = !empty($data) && isset($data['page_size']) ? $data['page_size'] : 100;

        if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
            $headers = [
                'Authorization' => 'Key ' . $data['takealot_api_key'],
                'x-RateLimit-Limit' => $data['x-RateLimit-Limit'],
                'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'],
                'Content-Type' => 'application/json'
            ];

            $client = new Client(['base_uri' => $data['base_url'], 'verify' => true, 'headers' => $headers]);
            
            if (isset($data['tsin'])) {
                $response = $client->get('sales?filters=tsin:' . $data['tsin'] . ';start_date:' . $data['startDate'] . ';end_date:' . $data['endDate'] . '&page_number=' . $pageNo . '&page_size=' . $pageSize);
            } else {
                $response = $client->get('sales?filters=start_date:' . $data['startDate'] . ';end_date:' . $data['endDate'] . '&page_number=' . $pageNo . '&page_size=' . $pageSize);
               
            }         
            
            if ($response->getStatusCode() == 200) {
                $result = $response->getBody()->getContents();
                if (!empty($result)) {
                    return response()->json(['status' => 'success', 'message' => 'Orders data fetched successfully.', 'data' => $result], 200);
                } else {
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returning data.'], 500);
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
            }
        } else {
            return response()->json(['status' => 'error', 'message' => 'Api key is required'], 429);
        }
    }

    public function fetchAllSales($data)
    {
        try {
            if (!empty($data) && isset($data['takealot_api_key']) && !empty($data['takealot_api_key'])) {
                $headers = [
                    'Authorization' => 'Key ' . $data['takealot_api_key'],
                    'x-RateLimit-Limit' => $data['x-RateLimit-Limit'],
                    'x-RateLimit-Remaining' => $data['x-RateLimit-Remaining'],
                    'Content-Type' => 'application/json'
                ];
                $client = new Client(['base_uri' => $data['base_url'], 'verify' => false, 'headers' => $headers]);
                $response = $client->get('sales');
                if ($response->getStatusCode() == 200) {
                    $result = $response->getBody()->getContents();
                    if (!empty($result)) {
                        return response()->json(['status' => 'success', 'message' => 'Orders data fetched successfully.', 'data' => $result], 200);
                    } else {
                        return response()->json(['status' => 'error', 'message' => 'Sorry! Takealot is not returning data.'], 500);
                    }
                } else {
                    return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $response], $response->getStatusCode());
                }
            } else {
                return response()->json(['status' => 'error', 'message' => 'Api key is required'], 429);
            }
        } catch (\Exception $e) {
            Logger($e);
            return response()->json(['status' => 'error', 'message' => 'Sorry! Something went wrong.', 'data' => $e->getMessage()], $e->getCode());
        }
    }
}
