<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use App\Models\User;
use App\Models\UserDetails;
use App\Models\BusinessSettings;
use App\Models\TakealotShops;
use App\Models\SubscriptionPackages;
use App\Models\PackageBuy;
use App\Models\PackageBuyHistory;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use Carbon\Carbon;
class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = RouteServiceProvider::HOME;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
 
    public function __construct()
    {
        $this->middleware('guest');

    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'email'          => ['required', 'string', 'email','min:2', 'max:100', 'unique:users'],
            'password'       => ['required', 'string', 'min:8','max:14', 'confirmed'],
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {       
        // try
        //   {
        //     DB::transaction(function () use ($data) {
            $user = new User();
            $user->username = 'salesup'.rand(pow(10, 8 - 1), pow(10, 8) -1);
            $user->email    = $data['email'];
            $user->password =  Hash::make($data['password']);
            if($user->save())
            {          
                $userDetails = new UserDetails();
                $userDetails->user_id =$user->id ;
                $userDetails->total_shops= 1;
                $userDetails->total_sub_accounts= 1;
                if($userDetails->save())
                {
                    $webhookUid=$user->id*786;
                    $takealotShops = new TakealotShops();
                    $takealotShops->seller_id=$user->id;
                    $takealotShops->takealot_account_name=null;
                    $takealotShops->webhook_uuid=(string) Str::uuid().'-'.$webhookUid;     
                    if($takealotShops->save())
                    {
                            $package=SubscriptionPackages::where([['type','=','monthly'],['plan_type','=','basic'],['package_type','=','free'],['price','=',0]])->first();
                            if(!empty($package) && $package->package_type=='free')
                            {
                                
                                $buyNow=new PackageBuy();
                                $buyNow->subscription_packages_id=$package->id;
                                $buyNow->payment_status='paid';
                                $buyNow->status='active';
                                $buyNow->subscription_status='enabled';
                                $buyNow->seller_id=$user->id;                           
                                if($buyNow->save())
                                {
                                    $pkgBuyHistory=new PackageBuyHistory();
                                    $pkgBuyHistory->seller_id=$user->id;   
                                    $pkgBuyHistory->package_details=null;
                                    $pkgBuyHistory->event='free';
                                    $pkgBuyHistory->price=$package->price;
                                    $pkgBuyHistory->subscription_packages_id=$package->id;
                                    $pkgBuyHistory->save();
                                }
                            }
                    } 
                
                }

            }
            return $user;
    //         });
    //     }
    //     catch (\Exception $e) 
    //     {           
    //         return redirect()->route('register')->with('error','Sorry! Something went wrong.');
    //     }
    //     return redirect()->route('register');
    }

}
