<?php

namespace App\Http\Controllers\Seller;
use App\Http\Controllers\Controller;
use App\Models\TakealotRePricing;
use App\Models\TakealotPlidHistory;
use App\Models\TakealotCompetitionHistory;
use App\Models\TakealotOffers;
use App\Models\PackageBuy;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use App\Jobs\TakealotRepricingJob;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;

use App\Http\Controllers\Seller\OffersController;
use App\Http\Controllers\Api\ScrapingApiController;
use App\Http\Controllers\Api\OfferApiController;

use Illuminate\Bus\Batch;
use Illuminate\Support\Facades\Bus;
use GuzzleHttp\HandlerStack;
use Spatie\GuzzleRateLimiterMiddleware\RateLimiterMiddleware;


use App\Models\TakealotShops;
use App\Models\TsinUpdateHistory;
use App\Jobs\FetchAllOffers;
use App\Jobs\TakealotCreateOfferJob;
use App\Jobs\TakealoltOffersShopByJob;
use App\Jobs\TakealoltRefetchOffersUpdate;
use App\Jobs\TakealoltRefetchOffersShopByJob;
use App\Jobs\TakealotNewCreatedOffers;

class ManualRePricingController extends Controller
{
      public $rateLimit=0;
      public $handlerStack;
      public $updateResponse;
      public function __construct()
      {
        $this->rateLimit=get_setting('takealot_reserved_rate_limit');
        $this->handlerStack = HandlerStack::create();
        $this->handlerStack->push(RateLimiterMiddleware::perMinute($this->rateLimit));
        $this->updateResponse=null;
      }  
      public function repriceVariantParams($id)
      {
        try
        {
          $repriceData =TakealotRePricing::with('offer','offer.takealotOfferDetails')->where([['id','=',$id]])->first();
          $this->instantRepriceTime($repriceData);
          return response()->json($this->updateResponse);
        }
        catch (\Exception $e) 
        {     
          $errorMessage = $e->getMessage();
          return response()->json(['status'=>'error','message' => $errorMessage]);
        } 
      }
      public function instantRepriceTime($rePriceData)
      {
        $this->updateResponse=null;    
        if(!empty($rePriceData))
        {           
            $result = $this->guzzelCurl($rePriceData);
            if(!empty($result))
            {
              if($rePriceData->offer->is_variant==0 && empty($rePriceData->offer->variant_params) && !empty($result) && isset($result->variants) && !empty($result->variants) && isset($result->variants->selectors) && !empty($result->variants->selectors) && count($result->variants->selectors)>0)
              {
                $getVariantParams=$this->variantOfferUrl($rePriceData->offer->title,$rePriceData->offer->offer_url,$result);
                if(!empty($getVariantParams))
                {
                  $rePriceData->offer->is_variant=1;
                  $rePriceData->offer->variant_params=$getVariantParams;
                  if($rePriceData->offer->save())
                  {
                    $this->repriceVariantParams($rePriceData->id);
                  }
                }
                  
              
              }
              if(!empty($result))
              {               
                $this->updateOffer($result,$rePriceData);
              }
          }
          else
          { 
            if(empty($this->updateResponse))  
            {
              $this->updateResponse=['status'=>'error','message'=>'Sorry! something went wrong. While scraping offer information.'];
            }
          }

        }
        else
        { 
          $this->updateResponse=['status'=>'error','message'=>'Sorry! Offer not found in re-pricing list.'];
        }
      }      
	    public function updateOffer($data,$rePricing)
      {            
                  $offerCurrentStatus=$rePricing->winner;
                  $winner=$rePricing->winner;                    
                  if(!empty($data->data_layer->totalPrice) && !empty($data->seller_detail->seller_id))
                  {            
                      if($data->data_layer->totalPrice>=$rePricing->min_price)
                      {               
                        $sellerCount=0;
                        $sellerCount=(!empty($data->other_offers->conditions[0]->items)) ? count($data->other_offers->conditions[0]->items) : 0;
                        //if there is no compititor on takealot against this product than our offer will be reached at max price
                        if( ($sellerCount==0 && empty($data->other_offers) && empty($data->other_offers->conditions[0])) && (!empty($data->seller_detail->seller_id) && trim($data->seller_detail->seller_id)==trim($rePricing->offer->takealotShops->tk_seller_id)))
                        {                          
                          $winner=1;  
                          if($rePricing->status==1)
                          {
                            $this->updateLiveOfferPrice($rePricing,$rePricing->max_price);
                          }
                          // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);
                          $this->updateResponse=['status'=>'success','message'=>'Product is winner and product max price updated because there is no compititor!'];
                        }
                        else
                        {                          
                              $winner=0;

                              //check other sellers are selling or not this product if not selling so we are the winner
                              if($sellerCount==0 && trim($rePricing->offer->takealotShops->tk_seller_id)==trim($data->seller_detail->seller_id))
                              {                            
                                $winner=1;
                                $this->updateResponse=['status'=>'success','message'=>'Product is winner and product max price updated because there is no compititor!'];
                              }
                              
                              //check winner account is our account 
                              if(!empty($rePricing->offer->takealotShops->tk_seller_id) && trim($rePricing->offer->takealotShops->tk_seller_id)==trim($data->seller_detail->seller_id))
                              {  
                                $winner=1;        
                                $this->updateResponse=['status'=>'success','message'=>'Your product is already winner!'];                      
                              }

                              //checking our selling price is greater than min or max price 
                              if($rePricing->offer->selling_price<$rePricing->min_price)
                              {
                                $winner=0;
                                $this->updateResponse=['status'=>'error','message'=>'Product selling price is less than min limit so price is updated !'];      
                              }
                              if($rePricing->offer->selling_price>$rePricing->max_price)
                              {
                                $winner=0;
                                $this->updateResponse=['status'=>'error','message'=>'Product selling price is greater than max price. So price is updated !'];   
                              }
                              //end checking
                              //if our offer is  winner
                              if($winner==1)
                              {
                                
                                  $checkPlid=TakealotPlidHistory::where('plid','=',$rePricing->offer->plid)->latest()->first();
                                  if(!empty($checkPlid))
                                  {
                                    $checkPlid->status='winner';
                                    $checkPlid->save();
                                  }
                                  $checkOffer=TakealotCompetitionHistory::where('tsin','=',$rePricing->offer->tsin_id)->latest()->first();        
                                  if(!empty($checkOffer))
                                  {
                                    $checkOffer->status='winner';
                                    $checkOffer->last_status='winner';
                                    $checkOffer->save();
                                  }
                                  $rePriceData=$this->offerPriceUp($data,$rePricing);
                                  if(!empty($rePriceData))
                                  {
                                    $rePricing= $rePriceData;
                                  }
                                
                                  // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);
                                  $this->updateResponse=['status'=>'success','message'=>"You'r offer is already winner!"];   
                              }
                              //end winner check color                        
                            
                              //if our offer is losser
                              if($winner==0)
                              {
                              
                                $takealotStockTotal=0;
                                $leadTimeStockTotal=0;
                                //stock code started
                                if(!empty($rePricing->offer->leadtime_stock) && isset($rePricing->offer->leadtime_stock))
                                {
                                    $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
                                    $leadTimeStockTotal=0;
                                    if(!empty($leadtime_decode) && count($leadtime_decode)>0)
                                    {
                                        foreach($leadtime_decode as $key=>$item)
                                        {
                                            $qty=(!empty($item->quantity_available) && isset($item->quantity_available) && $item->quantity_available>0) ? $item->quantity_available :0;
                                            $leadTimeStockTotal=$leadTimeStockTotal+$qty;
                                        }
                                      
                                    }
                                
                                }
                                if(!empty($rePricing->offer->stock_at_takealot) && isset($rePricing->offer->stock_at_takealot))
                                {
                                    $takealotStock_decode=json_decode($rePricing->offer->stock_at_takealot);
                                    if(!empty($takealotStock_decode) && count($takealotStock_decode)>0)
                                    {
                                        foreach($takealotStock_decode as $key=>$item)
                                        {
                                            $qty=(!empty($item->quantity_available) && isset($item->quantity_available)) ? $item->quantity_available : 0;
                                            $takealotStockTotal=$takealotStockTotal+$qty;
                                        }
                                    }
                              
                                }
                                //stock code ended
                                    if(($data->data_layer->totalPrice>=$rePricing->min_price && $data->data_layer->totalPrice<=$rePricing->max_price))
                                    {
                                    
                                      if(!empty($rePricing->takealotShopDetails->takealot_api_key))
                                      {

                                        //start our product leadtime/instock and compititor product on leadtime/instock
                                        if(($data->stock_availability->is_leadtime==true && $takealotStockTotal<=0 && $leadTimeStockTotal>0) || ($data->stock_availability->is_leadtime==false && $takealotStockTotal>0))
                                        {                                      
                                          $rePrice=$data->data_layer->totalPrice-1;
                                          if($rePrice<=$rePricing->min_price)
                                          {
                                            $rePrice=$rePricing->min_price;
                                          }
                                          if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price) && $rePricing->status==1 && $rePricing->offer->status=='Buyable')
                                          {
                                            $this->updateLiveOfferPrice($rePricing,$rePrice);
                                          }
                                        }
                                        //end our product leadtime/instock and compititor product on leadtime/instock
                                        if(!empty($rePricing->min_price) && !empty($rePricing->max_price) && $rePricing->status==1  && $rePricing->offer->status=='Buyable' && $takealotStockTotal>0)
                                        {
                                          $updatedAt = Carbon::parse($rePricing->price_updated_at);
                                          $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
                                          //start our product instock and compititor product on leadtime
                                          if($minutesAgo>=8)
                                          {
                                            $this->instockVsLeadtime($data,$rePricing,$takealotStockTotal);
                                          }
                                          //end our product instock and compititor product on leadtime
                                        }  
                                        
                                        if(!empty($rePricing->min_price) && !empty($rePricing->max_price)  && $rePricing->status==1  && $rePricing->offer->status=='Buyable' && $takealotStockTotal<=0 && $leadTimeStockTotal>0)
                                        {      
                                          $updatedAt = Carbon::parse($rePricing->price_updated_at);
                                          $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
                                          //start our product leadtime and compititor product on instock
                                          if($minutesAgo>=8)
                                          {
                                              $this->leadtimeVsInstock($data,$rePricing,$leadTimeStockTotal);
                                          }
                                          //end our product leadtime and compititor product on instock
                                        }
                                    
                                      }
                                    }
                                    else
                                    {
                                      
                                      if($rePricing->status==1  && $rePricing->offer->status=='Buyable')
                                      {
                                          if($data->data_layer->totalPrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $data->data_layer->totalPrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                                          {
                                            if($data->data_layer->totalPrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                                            {
                                              $this->updateLiveOfferPrice($rePricing,$rePricing->max_price);
                                            }
                                            if($data->data_layer->totalPrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                                            {
                                              $this->updateLiveOfferPrice($rePricing,$rePricing->min_price);
                                            }
                                          }
                                          else
                                          {
                                              if($data->data_layer->totalPrice==$rePricing->min_price && (($data->stock_availability->is_leadtime==false  && $takealotStockTotal>0) || ($data->stock_availability->is_leadtime==true && $takealotStockTotal<=0 && $leadTimeStockTotal>0)))
                                              {
                                                $newPrice=$rePricing->min_price;
                                              }
                                              elseif($data->data_layer->totalPrice==$rePricing->min_price)
                                              {
                                                $newPrice=$rePricing->min_price+1;
                                              }
                                              else
                                              {
                                                $newPrice=$rePricing->min_price;
                                              }
                                              $this->updateLiveOfferPrice($rePricing,$newPrice);
                                            
                                          }                                   
                                    
                                      }
                                    }
                                  
                              }            
                              //end losser code
                              // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);
                        }
                      
                        
                        if(!empty($rePricing->offer->takealotOfferDetails))
                        {
                          $rePricing=$this->updateTakealotOfferDetails($rePricing,$data,$sellerCount);
                        }

                        
                        if($winner!=$offerCurrentStatus)
                        {
                        
                          $activityLog=array(
                            'repricing_status'=>array(
                            'seller_id'=>$rePricing->offer->seller_id,
                            'takealot_shops_id'=>$rePricing->offer->takealot_shops_id,
                            'tsin'=>$rePricing->offer->tsin_id,
                            'new_status'=>($winner==1) ? 'Winner' : 'Losser',
                            'old_status'=>($offerCurrentStatus==1) ? 'Winner' : 'Losser',
                            'ip'=>getUserIp(),
                            'updated_through'=>'repricing'
                            )
                          );
                          $updateHistoyByTsin=new OffersController();
                          $updateHistoyByTsin->tsinUpdateHistoryWithObject($activityLog,'Re-Pricing');  
                        }
                      
                    }  
                    $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);
                  }
                  else
                  {
                    $this->updateResponse=['status'=>'error','message'=>'Offer price is not within your min-max range!'];   
                  }  
                
      }
      public function updateRepricingStatus($rePricing,$data,$winner)
      {
         
          $stockStatus=(!empty($data->stock_availability) && !empty($data->stock_availability->status)) ? $data->stock_availability->status : '';
          $rePricing->stock_status=$stockStatus;
          $rePricing->leadtime_status=(!empty($data->stock_availability->is_leadtime) && $data->stock_availability->is_leadtime==true) ? 1 : 0;
          $rePricing->winning_price=$data->data_layer->totalPrice;
          $rePricing->winner_account_name=trim($data->seller_detail->display_name);
          $rePricing->winner_seller_id=trim($data->seller_detail->seller_id);
          $rePricing->winner=$winner;
          $rePricing->price_updated_at=Carbon::now();
          $rePricing->sellers=(!empty($data->other_offers->conditions[0]->items)) ? count($data->other_offers->conditions[0]->items) : 0;
          $rePricing->updated_at=Carbon::now();
          $rePricing->save();
          return $rePricing;
      }
      public function updateTakealotOfferDetails($rePricing,$data,$sellerCount)
      {
        $rePricing->offer->takealotOfferDetails->star_rating=!empty($data->core->star_rating) ? $data->core->star_rating : 0;
        $rePricing->offer->takealotOfferDetails->total_sellers=$sellerCount;
        $rePricing->offer->takealotOfferDetails->save();
        return $rePricing;
      }
      public function instockVsLeadtime($data,$rePricing,$takealotStockTotal)
      {
          
          if($data->stock_availability->is_leadtime==true && $takealotStockTotal>0)
          {            
            $checkPlid=TakealotPlidHistory::where([['plid','=',$rePricing->offer->plid],['stock_type','=','instockVsLeadtime']])->latest()->first();
           
            if(!empty($checkPlid))
            {   
              if($checkPlid->status=='winner')
              {                   
                  $checkPlid->status='losser';
                  $checkPlid->save();                    
                  $getNewPercentage=($data->data_layer->totalPrice*$checkPlid->percentage)/100;
                  $finalPrice=$data->data_layer->totalPrice+(floor($getNewPercentage));
                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {
                    $this->competitonHistory($data,$rePricing,$checkPlid->id,$checkPlid->percentage,$finalPrice);
                    $this->updateLiveOfferPrice($rePricing,$finalPrice);    
                  }

              }
              else
              {
               
                $createPlidHistory=$checkPlid;
                if($checkPlid->percentage>=1)
                { 
                  $subTwonumbers=$data->data_layer->totalPrice - $rePricing->offer->selling_price;
                  $addTwonumbers=($data->data_layer->totalPrice + $rePricing->offer->selling_price)/2;
                  $percentageDifference=($subTwonumbers/$addTwonumbers)*100;
                  $percentageDifference=abs($percentageDifference);
                  if($percentageDifference>=33)
                  {
                    $instockPercentage=32;
                  }
                  else
                  {     
                    $instockPercentage=$percentageDifference-($rePricing->offer->takealotShops->leadtime_percentage);                    
                  }  

                  if($instockPercentage<0)
                  {
                    $instockPercentage=0;
                  }
                 
                  $getNewPercentage=$data->data_layer->totalPrice*$instockPercentage/100;
                  $finalPrice=$data->data_layer->totalPrice+(floor($getNewPercentage));
                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {
                    $finalPrice=$finalPrice;
                  }
                  else
                  {
                    $instockPercentage=$percentageDifference;
                    $finalPrice=$rePricing->min_price;
                  }
               
                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {
                      $createPlidHistory->plid=$rePricing->offer->plid;
                      $createPlidHistory->percentage=$instockPercentage;
                      $createPlidHistory->status='losser';
                      if($createPlidHistory->save())
                      {
                        
                        if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                        {
                          $this->competitonHistory($data,$rePricing,$createPlidHistory->id,$createPlidHistory->percentage,$finalPrice);
                          $this->updateLiveOfferPrice($rePricing,$finalPrice);    
                        }

                      }
                  }
                }                               

              }
              
            }
            else
            {
              $subTwonumbers=$data->data_layer->totalPrice - $rePricing->offer->selling_price;
              $addTwonumbers=($data->data_layer->totalPrice + $rePricing->offer->selling_price)/2;
              $percentageDifference=($subTwonumbers/$addTwonumbers)*100;
              $percentageDifference=abs($percentageDifference); 
              if($percentageDifference>=33)  
              {
                $instockPercentage=32;
              }
              else
              {
                $instockPercentage=$percentageDifference-($rePricing->offer->takealotShops->leadtime_percentage);                
              }

              if($instockPercentage<0)
              {
                $instockPercentage=0;
              }

              $getNewPercentage=$data->data_layer->totalPrice*$instockPercentage/100;
              $finalPrice=$data->data_layer->totalPrice+(floor($getNewPercentage));  
              if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
              {
                $finalPrice=$finalPrice;
              }
              else
              {
                $instockPercentage=$instockPercentage;
                $finalPrice=$rePricing->min_price;
              }

              if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
              {
                    $checkPlid=new TakealotPlidHistory();
                    $checkPlid->plid=$rePricing->offer->plid;
                    $checkPlid->percentage=$instockPercentage;
                    $checkPlid->status='losser';
                    $checkPlid->stock_type='instockVsLeadtime';
                    if($checkPlid->save())
                    {    
                        if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                        {
                          $this->competitonHistory($data,$rePricing,$checkPlid->id,$instockPercentage,$finalPrice);
                          $this->updateLiveOfferPrice($rePricing,$finalPrice);    
                        }
                    }
              }

            }

          }
          
      }
      public function leadtimeVsInstock($data,$rePricing,$leadTimeStockTotal)
      {   
           if($data->stock_availability->is_leadtime==false && $leadTimeStockTotal>0)
          {
            
            $checkPlid=TakealotPlidHistory::where([['plid','=',$rePricing->offer->plid],['stock_type','=','leadtimeVsinstock']])->latest()->first();
            if(!empty($checkPlid))
            {

              if($checkPlid->status=='winner')
              {      

                $checkPlid->status='losser';
                $checkPlid->save();
                $percentageDifference=($data->data_layer->totalPrice*$checkPlid->percentage)/100;
                $finalPrice=$data->data_layer->totalPrice-(floor($percentageDifference));
                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {
                    $this->competitonHistory($data,$rePricing,$checkPlid->id,$checkPlid->percentage,$finalPrice);
                    $this->updateLiveOfferPrice($rePricing,$finalPrice);    
                  }

              }
              else
              {                
                $createPlidHistory=$checkPlid;
                if($createPlidHistory->percentage<=34.5)
                {
                  
                  $newPercentage=$checkPlid->percentage+$rePricing->offer->takealotShops->leadtime_percentage; 
                  if($newPercentage>34.5)
                  {
                    $newPercentage=33.1;
                  }

                  $percentageDifference=($data->data_layer->totalPrice*$newPercentage)/100;
                  $finalPrice=$data->data_layer->totalPrice-(floor($percentageDifference));

                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {
                    $finalPrice=$finalPrice;
                  }
                  else
                  {                    
                    $newPercentage=$checkPlid->percentage;
                    $finalPrice=$rePricing->min_price;
                  }

                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {   
                      $createPlidHistory->plid=$rePricing->offer->plid;
                      $createPlidHistory->percentage=$newPercentage;
                      $createPlidHistory->status='losser';
                      if($createPlidHistory->save())
                      {                     
                          $this->competitonHistory($data,$rePricing,$checkPlid->id,$newPercentage,$finalPrice);                      
                          $this->updateLiveOfferPrice($rePricing,$finalPrice);   
                        
                      }
                  }
                }
                                

              }
              
            }
            else
            {
              
              $percentageDifference=($data->data_layer->totalPrice*10)/100;
              $finalPrice=$data->data_layer->totalPrice-(floor($percentageDifference));            
              if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
              {
                $finalPrice=$finalPrice;
              }
              else
              {               
                $finalPrice=$rePricing->min_price;
              }
             
              
              if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
              {
                $checkPlid=new TakealotPlidHistory();
                $checkPlid->plid=$rePricing->offer->plid;
                $checkPlid->percentage=10;
                $checkPlid->status='losser';
                $checkPlid->stock_type='leadtimeVsinstock';              
                if($checkPlid->save())
                {  
                  if(($finalPrice>=$rePricing->min_price && $finalPrice<=$rePricing->max_price))
                  {                      
                    $this->competitonHistory($data,$rePricing,$checkPlid->id,10,$finalPrice);                    
                    $this->updateLiveOfferPrice($rePricing,$finalPrice); 
                  }
                }
              }


            }
            

          } 
          else
          {            
            $this->updateTakealotOfferSoh($rePricing->offer->offer_id,$rePricing->takealotShopDetails,$rePricing);                             
          }
      }
      public function updateLiveOfferPrice($rePricing,$rePrice)
      {
        
        $result=$this->updateTakealotOffer($rePricing->offer->offer_id,$rePrice,$rePricing->takealotShopDetails,$rePricing);
        if(!empty($result)) 
        {
          $offerUpdated=json_decode($result);    
          if(!empty($offerUpdated->offer))
          {
            $rePricing->offer->selling_price=$offerUpdated->offer->selling_price;
            $rePricing->offer->rrp=$offerUpdated->offer->rrp;
            if($rePricing->offer->save())
            {
              $current_time = Carbon::now();
              $rePricing->winning_price=$rePricing->offer->selling_price;
              $rePricing->price_updated_at=$current_time;
              $rePricing->save();

            }

          }
          
        }
        return $rePricing;
      }
      public function competitonHistory($data,$rePricing,$leadtimeID,$percentage,$finalPrice)
      {

        $checkOffer=TakealotCompetitionHistory::where('tsin','=',$rePricing->offer->tsin_id)->first();        
        if(!empty($checkOffer))
        {
          $checkOffer->status='losser';
          $checkOffer->save();
        }

        $competition=new TakealotCompetitionHistory();
        $competition->plid_history_id=$leadtimeID;
        $competition->tsin=$rePricing->offer->tsin_id;
        $competition->your_price=$finalPrice;
        $competition->competitor_price=$data->data_layer->totalPrice;
        $competition->percentage=$percentage;
        $competition->status='losser';
        $competition->last_status='losser';
        $competition->save();  
        return true;
      }
      //start code --- offer price up if our offer is already winner
      public function offerPriceUp($data,$rePricing)
      {
       
        $takealotStockTotal=0;
        $leadTimeStockTotal=0;
        //stock code started
        if(!empty($rePricing->offer->leadtime_stock))
        {
             $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
             $leadTimeStockTotal=0;
             if(!empty($leadtime_decode) && count($leadtime_decode)>0)
             {
                 foreach($leadtime_decode as $key=>$item)
                 {
                     $qty=(!empty($item->quantity_available) && isset($item->quantity_available) && $item->quantity_available>0) ? $item->quantity_available :0;
                     $leadTimeStockTotal=$leadTimeStockTotal+$qty;
                 }
               
             }
        
        }
        if(!empty($rePricing->offer->stock_at_takealot))
        {
            $takealotStock_decode=json_decode($rePricing->offer->stock_at_takealot);
            if(!empty($takealotStock_decode) && count($takealotStock_decode)>0)
            {
                 foreach($takealotStock_decode as $key=>$item)
                 {
                    $qty=(!empty($item->quantity_available) && isset($item->quantity_available)) ? $item->quantity_available : 0;
                    $takealotStockTotal=$takealotStockTotal+$qty;
                 }
            }
      
        }
        //stock code ended     
          if(!empty($data->other_offers->conditions[0]->items))
          {
              $otherOfferTopPrice=$data->other_offers->conditions[0]->items[0]->price;
              $stock=$data->other_offers->conditions[0]->items[0]->stock_availability->is_in_stock;
              $checkOnTop=[];
              if(isset($data->other_offers->conditions[0]->items) && !empty($data->other_offers->conditions[0]->items))
              {
                foreach($data->other_offers->conditions[0]->items as $key=>$itemOther)
                {
                    if($itemOther->stock_availability->is_in_stock==false)
                    {
                      $checkOnTop['leadtime']=1;                  
                    }
                    else
                    {
                      $checkOnTop['instock'][0]=$key+1;
                      $checkOnTop['instock'][1]=$data->other_offers->conditions[0]->items[$key]->price;
                      break;
                    }

                }
              }
              
      
              //if it is true it means in other offers on top is instock offer if false it means in other offers on top offer is leadtime
              if($stock==true && (isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && $checkOnTop['instock'][0]==1))
              {
                  if($data->stock_availability->is_leadtime==false  && $takealotStockTotal>0)
                  {
                      $rePrice=$checkOnTop['instock'][1]-1;
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                        {
                          $rePrice=$rePricing->max_price;
                        }
                        if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                        {
                          $rePrice=$rePricing->min_price;
                        }
                      }

                      if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                      { 
                         return $this->updateLiveOfferPrice($rePricing,$rePrice);
                      }

                  }

              }
              else
              {
               
                //if other instock is available insode otheroffer list 
                if($stock==false && (isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1])))
                {
                  
                    if($data->stock_availability->is_leadtime==false && $takealotStockTotal>0)
                    {
                        $rePrice=$checkOnTop['instock'][1]-1;
                        if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                        {
                          if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                          {
                            $rePrice=$rePricing->max_price;
                          }
                          if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                          {
                            $rePrice=$rePricing->min_price;
                          }
                        }
                        if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                        {                          
                           return $this->updateLiveOfferPrice($rePricing,$rePrice);
                        }
  
                    }
  
                }
                //if our and compititor offer is on leadtime
                if(($stock==false && $data->stock_availability->is_leadtime==true ) && (isset($checkOnTop['leadtime']) && $checkOnTop['leadtime']==1) && ($takealotStockTotal<=0 && $leadTimeStockTotal>0))
                {
                    $rePrice=$otherOfferTopPrice-1;  
                    if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                    {
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                      {
                        $rePrice=$rePricing->max_price;
                      }
                      if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        $rePrice=$rePricing->min_price;
                      }
                    }

                    if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                    {                     
                      return $this->updateLiveOfferPrice($rePricing,$rePrice);
                    }

                }
                 //our offer is instock and compititor offers on leadtime
                if($stock==false && $takealotStockTotal>0)
                {
                  $priceUpdatedAt = Carbon::parse($rePricing->price_updated_at);
                  $currentDateTime = Carbon::now();
                  $diffInMinutes = $priceUpdatedAt->diffInMinutes($currentDateTime);
                  if($diffInMinutes>=20 && $rePricing->winner==1)
                  { 
                   $checkPlid=TakealotPlidHistory::where([['plid','=',$rePricing->offer->plid],['stock_type','=','instockVsLeadtime']])->latest()->first();
                   if(!empty($checkPlid))
                   {
                    $checkPlid->percentage=($checkPlid->percentage+1>34.1) ? 33.1 : $checkPlid->percentage+1;
                    $getPercenategDifference=($otherOfferTopPrice*$checkPlid->percentage)/100;
                    $rePrice=$otherOfferTopPrice+(floor($getPercenategDifference));

                    if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                    {
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                      {
                        $rePrice=$rePricing->max_price;
                      }
                      if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        $rePrice=$rePricing->min_price;
                      }
                    }
                    if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                    {
                      $checkPlid->save();
                      return $this->updateLiveOfferPrice($rePricing,$rePrice);
                    }

                   }
                   else   
                   {
                      $checkPlid=new TakealotPlidHistory();
                      $checkPlid->plid=$rePricing->offer->plid;
                      $checkPlid->stock_type='instockVsLeadtime';
                      $checkPlid->percentage=10;
                      $checkPlid->status='losser';
                      $getPercenategDifference=($otherOfferTopPrice*$checkPlid->percentage)/100;
                      $rePrice=$otherOfferTopPrice+(floor($getPercenategDifference));
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                        {
                          $rePrice=$rePricing->max_price;
                        }
                        if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                        {
                          $rePrice=$rePricing->min_price;
                        }
                      }
                      if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                      {
                        
                        if((isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1])))
                        {
                          if($rePrice>=$checkOnTop['instock'][1])
                          {
                            $rePrice=$checkOnTop['instock'][1]-1;
                            $firstVal=abs($checkOnTop['instock'][1]-$rePricing->offer->selling_price);
                            $secondVal=($checkOnTop['instock'][1]+$rePricing->offer->selling_price)/2;
                            $finalVal=abs($firstVal)/$secondVal;
                            $checkPlid->percentage=$finalVal/100;
                          }
                          
                        }
                        $checkPlid->save();
                        return $this->updateLiveOfferPrice($rePricing,$rePrice);
                      }
                   }
                  }            
                
                }

                //our offer is on leadtime and compititor offers is instock
                if($stock==false && $takealotStockTotal<=0 && $leadTimeStockTotal>0)
                {
                  $priceUpdatedAt = Carbon::parse($rePricing->price_updated_at);
                  $currentDateTime = Carbon::now();
                  $diffInMinutes = $priceUpdatedAt->diffInMinutes($currentDateTime);
                  if($diffInMinutes>=20 && $rePricing->winner==1)
                  { 
                   $checkPlid=TakealotPlidHistory::where([['plid','=',$rePricing->offer->plid],['stock_type','=','TakealotPlidHistory']])->latest()->first();
                  
                   if(!empty($checkPlid))
                   {
                      $checkPlid->percentage=($checkPlid->percentage+1<=1) ? 1 : $checkPlid->percentage+1;
                      $getPercenategDifference=($otherOfferTopPrice*$checkPlid->percentage)/100;
                      $rePrice=$otherOfferTopPrice-(floor($getPercenategDifference));
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                        {
                          $rePrice=$rePricing->max_price;
                        }
                        if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                        {
                          $rePrice=$rePricing->min_price;
                        }
                      }
                      if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                      {
                        $checkPlid->save();
                        return $this->updateLiveOfferPrice($rePricing,$rePrice);
                      }

                   }
                   else   
                   {
                      $checkPlid=new TakealotPlidHistory();
                      $checkPlid->plid=$rePricing->offer->plid;
                      $checkPlid->stock_type='TakealotPlidHistory';
                      $checkPlid->percentage=10;
                      $checkPlid->status='losser';
                      $getPercenategDifference=($otherOfferTopPrice*$checkPlid->percentage)/100;
                      $rePrice=$otherOfferTopPrice+(floor($getPercenategDifference));
                      if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price || $rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                      {
                        if($rePrice>$rePricing->max_price || $rePricing->offer->selling_price>$rePricing->max_price)
                        {
                          $rePrice=$rePricing->max_price;
                        }
                        if($rePrice<$rePricing->min_price || $rePricing->offer->selling_price<$rePricing->min_price)
                        {
                          $rePrice=$rePricing->min_price;
                        }
                      }
                      if(($rePrice>=$rePricing->min_price && $rePrice<=$rePricing->max_price))
                      {
                        
                        if((isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1])))
                        {
                          if($rePrice>=$checkOnTop['instock'][1])
                          {
                            $rePrice=$checkOnTop['instock'][1]-1;
                            $firstVal=abs($checkOnTop['instock'][1]-$rePricing->offer->selling_price);
                            $secondVal=($checkOnTop['instock'][1]+$rePricing->offer->selling_price)/2;
                            $finalVal=abs($firstVal)/$secondVal;
                            $checkPlid->percentage=$finalVal/100;
                          }
                          
                        }
                        $checkPlid->save();
                        return $this->updateLiveOfferPrice($rePricing,$rePrice);
                      }
                   }            
                  }
                }


              }
  
            
         }

      }
       //end code --- offer price up if our offer is already winner
      public function variantOfferUrl($title,$url,$result)
      {
              // ini_set('max_execution_time', 0);
              $newTitle=null;            
              if(!empty($result) && !empty($result->variants) && !empty($result->variants->selectors) && count($result->variants->selectors)>0)
              {
                    
                    $selectors=null; 
                    foreach($result->variants->selectors as $key=>$item)
                    {                    
                      foreach($item->options as $key=>$variant)
                      {
                        if(!empty($variant->value->value))
                        {                   
                          $selectors[$item->selector_type][]=$variant->value->value;
                        }
                        else
                        {                    
                          $selectors[$item->selector_type][]=$variant->value;
                        }                      
                      }
                    }

                    $combinations=$this->get_combinations($selectors);                  
                    foreach( $combinations as $Pkey=>$comb)
                    {
                    
                      $varTitle=null;
                      $varParams=null;
                      foreach($comb as $key=>$variant)
                      {
                        if(!empty($varTitle))
                        {
                          $varTitle=$varTitle. ' - '.$variant;
                          $varParams=$varParams.'&'.$key.'='.$variant;
                        }
                        else
                        {
                          $varTitle=' - '.$variant;
                          $varParams='?'.$key.'='.$variant;
                        }
                        
                      
                      }
                    
                      $newTitle=null;
                      $newTitle=$result->title.$varTitle;
                    
                      if($newTitle==$title)
                      {
                        
                        $productVariant=$varParams;    
                        return  $productVariant;                 
                      }
                    }  

              }
                        
            return null;
       }

      function get_combinations($arrays) 
      {
        $result = array(array());
        foreach ($arrays as $property => $property_values) {
            $tmp = array();
            foreach ($result as $result_item) {
                foreach ($property_values as $property_key => $property_value) {
                    $tmp[] = $result_item + array($property => $property_value);
                }
            }
            $result = $tmp;
        }
        return $result;

      }   
      public function guzzelCurl($repriceData)
      {
       
          if(!empty($repriceData->offer->plid))
          {          
                $apiCall=apiProxyCall('manual');                
                if(!empty($apiCall) && $apiCall->active_status==1 && $apiCall->reserved_limit>0 && isset($apiCall->api_url))
                {
                      // Your data
                      $data = [
                        "plid" => $repriceData->offer->plid,
                        "is_variant" => $repriceData->offer->is_variant,
                        "variant_params" => $repriceData->offer->variant_params
                      ];
                       
                       if($apiCall->api_url==env('APP_URL')) 
                       {
                            $scrapCont = new ScrapingApiController(); 
                            $result=$scrapCont->scrapData($data);                           
                            if(!empty($result) && $result->getStatusCode()==200)
                            {
                              $jsonContent = $result->getContent();
                              $result = json_decode($jsonContent);
                              if(isset($result->status) && $result->status=='success' && isset($result->data) && !empty($result->data))
                              {
                                return $result->data;
                              }
                              return null;
                            }
                            else
                            {
                              $data=[                                    
                                'type'=>'manual',
                                'status'=>$result->getStatusCode(),
                                'data'=>$result 
                              ];
                              saveError($data); 
                              $this->updateResponse=['status'=>'error','message'=>'Sorry! Something went wrong during refetching'];
                              return null;
                            }
                       }
                       else
                       {                       
                          // Create a Guzzle client
                          $client = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                          $jsonData = json_encode($data);
                          $response = $client->post('/api/scrap/offer', [ 
                            'body' => $jsonData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                          ]); 


                          if(!empty($response) && $response->getStatusCode()==200)
                          {

                              $result=json_decode($response->getBody()->getContents());  
                              if(!empty($result) && $result->status=='success')
                              {          
                                  return $result->data;
                              }
                              else
                              {
                                  $this->updateResponse=['status'=>'error','message'=>'Sorry! Something went wrong during refetching'];
                                  return null;
                              }

                          }
                          else
                          {
                            $data=[                                    
                              'type'=>'manual',
                              'status'=>$response->getStatusCode(),
                              'data'=>$response 
                            ];
                            saveError($data);
                            $this->updateResponse=['status'=>'error','message'=>$response->getStatusCode().'Error . Sorry! Something went wrong during refetching'];
                            return null;
                          }
                       }
                      
                }
                else
                {
                  $this->updateResponse=['status'=>'error','message'=>'Sorry! your reserved limit is exceeded. Please try after a minute.'];
                  return null;
                }
                
           
          }
         
             
      }    
      public function updateTakealotOffer($offer_id,$selling_price,$shopDetails,$rePricing=null)
      { 
               
              $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
              if($ratelimit>0)
              {
                // if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null && $rePricing->offer->stock_at_takealot_total<=0))
                if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null))
               {
                  $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
                  $leadtime_qty=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->quantity_available)) ? $leadtime_decode[0]->quantity_available : 0;
                  $merchant_warehouse_id=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->merchant_warehouse->warehouse_id)) ? $leadtime_decode[0]->merchant_warehouse->warehouse_id : 0;
                 
                  if(!empty($merchant_warehouse_id) && $merchant_warehouse_id>0)
                  {
                     
                      $leadtime=[
                          'merchant_warehouse_id'=>$merchant_warehouse_id,
                          'quantity'=>(int)$leadtime_qty
                      ];
                      $body=[
                          'selling_price' => (int)$selling_price,
                          'leadtime_stock' => [$leadtime],    
                      ];
  
                  }
                  else
                  {
                      $body=['selling_price' => (int)$selling_price];    
                  }                    
                  
              }
              else
              {   
                  $body=['selling_price' => (int)$selling_price];    
              }


                $apiCall=apiProxyCall('manual');                
                if(!empty($apiCall) && $apiCall->reserved_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
                {
                  if($apiCall->api_url==env('APP_URL')) 
                  {
                      $allData=[
                        'takealot_api_key'=>$shopDetails->takealot_api_key,
                        'x-RateLimit-Limit'=>$this->rateLimit,
                        'x-RateLimit-Remaining'=>$ratelimit,
                        'base_url'=>takealotBaseUrl(),
                        'data'=>$body,
                        'offer_id'=>$offer_id
                      ];

                       $offerCont = new OfferApiController(); 
                       $result=$offerCont->updateTakealotOffer($allData);
                      
                        if(!empty($result) && !empty($rePricing))
                        {                            
                            $resultDecoded =json_decode($result->getContent(),true);    
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                              $resultDecoded  = json_decode($resultDecoded['data']); 
                              $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                            }
                            else
                            {
                              $this->updateResponse=['status'=>'error','message'=>'Getting empty data after offer updating on takealot'];
                            }
                         }
                         else
                         {
                            $this->updateResponse=['status'=>'error','message'=>"Offer not updated something went wrong."];
                         }
                  }
                  else
                  {

                    $allData=[
                      'takealot_api_key'=>$shopDetails->takealot_api_key,
                      'x-RateLimit-Limit'=>$this->rateLimit,
                      'x-RateLimit-Remaining'=>$ratelimit,
                      'base_url'=>takealotBaseUrl(),
                      'data'=>$body,
                      'offer_id'=>$offer_id
                    ];
                    $postData=json_encode($allData);
                    $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                    $response = $client->post('/api/update/offer/'.$offer_id, [
                        'body' => $postData,
                        'headers' => [
                          'Content-Type' => 'application/json', // Set the content type to JSON
                      ]
                    ]);
                   
                    if($response->getStatusCode()==200)
                    {
                        $result=$response->getBody()->getContents();
                        if(!empty($result) && !empty($rePricing))
                        { 
                            $resultDecoded  = json_decode($result,true); 
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                              $resultDecoded  = json_decode($resultDecoded['data']); 
                              $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                            }
                            else
                            {
                              $this->updateResponse=['status'=>'error','message'=>'Getting empty data after offer updating on takealot'];
                            }
                         }
                         else
                         {
                            $this->updateResponse=['status'=>'error','message'=>"Offer not updated something went wrong."];
                         }
                    }
                    else
                    {
                      $this->updateResponse=['status'=>'error','message'=>"Offer not updated getting error from takealot seller center. something went wrong."];
                    }
                      
                    
                  }
                }
                else
                {
                  $this->updateResponse=['status'=>'error','message'=>"Price not updated. All IP's limit exceeded"];
                }
             
              }
              else
              {
                $this->updateResponse=['status'=>'error','message'=>"Price not updated. Your shop limit exceeded"];
              }
       
        
      }
      public function updateTakealotOfferSoh($offer_id,$shopDetails,$rePricing)
      {
         
          $ratelimit=shopReservedRateLimitChecker();
          if($ratelimit>0)
          {

              $headers = ['Authorization'=>'Key '.$shopDetails->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit,'Content-Type' => 'application/json'];
              
        
              if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null))
              {
                
                  $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
                  $leadtime_qty=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->quantity_available)) ? $leadtime_decode[0]->quantity_available : 0;
                  
                  $merchant_warehouse_id=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->merchant_warehouse->warehouse_id)) ? $leadtime_decode[0]->merchant_warehouse->warehouse_id : 0;
                
                  if(!empty($merchant_warehouse_id) && $merchant_warehouse_id>0 )
                  {  
                      
                      $leadtime=[
                          'merchant_warehouse_id'=>$merchant_warehouse_id,
                          'quantity'=>(int)$leadtime_qty
                      ];
                      $body=[                        
                          'leadtime_stock' => [$leadtime]                            
                      ];
                      $apiCall=apiProxyCall('manual');           
                      if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
                      {
                        if($apiCall->api_url==env('APP_URL')) 
                        {
                            $allData=[
                              'takealot_api_key'=>$shopDetails->takealot_api_key,
                              'x-RateLimit-Limit'=>$this->rateLimit,
                              'x-RateLimit-Remaining'=>$ratelimit,
                              'base_url'=>takealotBaseUrl(),
                              'data'=>$body,
                              'offer_id'=>$offer_id
                            ];
      
                              $offerCont = new OfferApiController(); 
                              $result=$offerCont->updateTakealotOffer($allData);
                              if(!empty($result) && !empty($rePricing))
                              {                            
                                  $resultDecoded =json_decode($result->getContent(),true);    
                                  if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                  {
                                    $resultDecoded  = json_decode($resultDecoded['data']); 
                                    $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                                  }
                                  else
                                  {
                                    $this->updateResponse=['status'=>'error','message'=>'Getting empty data after offer updating on takealot'];
                                  }
                                }
                                else
                                {
                                  $this->updateResponse=['status'=>'error','message'=>"Offer not updated something went wrong."];
                                }
                        }
                        else
                        {
      
                          $allData=[
                            'takealot_api_key'=>$shopDetails->takealot_api_key,
                            'x-RateLimit-Limit'=>$this->rateLimit,
                            'x-RateLimit-Remaining'=>$ratelimit,
                            'base_url'=>takealotBaseUrl(),
                            'data'=>$body,
                            'offer_id'=>$offer_id
                          ];
                          $postData=json_encode($allData);
                          $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                          $response = $client->post('/api/update/offer/'.$offer_id, [
                              'body' => $postData,
                              'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                          ]);
                          
                          if($response->getStatusCode()==200)
                          {
                              $result=$response->getBody()->getContents();
                              if(!empty($result) && !empty($rePricing))
                              { 
                                  $resultDecoded  = json_decode($result,true); 
                                  if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                  {
                                    $resultDecoded  = json_decode($resultDecoded['data']); 
                                    $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer); 
                                  }
                                  else
                                  {
                                    $this->updateResponse=['status'=>'error','message'=>'Getting empty data after offer updating on takealot'];
                                  }
                                }
                                else
                                {
                                  $this->updateResponse=['status'=>'error','message'=>"Offer not updated something went wrong."];
                                }
                          }
                          else
                          {
                            $this->updateResponse=['status'=>'error','message'=>"Offer not updated getting error from takealot seller center. something went wrong."];
                          }
                            
                          
                        }
                      }
                      else
                      {
                        $this->updateResponse=['status'=>'error','message'=>"Price not updated. All IP's limit exceeded"];
                      }
                      

                  }                   
                  else
                  {
                    $this->updateResponse=['status'=>'error','message'=>"Merchant warehouse id not founded."];
                  }   
              }
              else
              {
                $this->updateResponse=['status'=>'error','message'=>"Product leadtime not found"];
              }
        }            
        else
        {
          $this->updateResponse=['status'=>'error','message'=>"Soh not updated. Your shop limit exceeded"];
        }  
      }
      public function updateRefetchedOffer($result,$takeAlotOffers)
      {  
       
          $takeAlotOffers->sku                      =   $result->sku;
          $takeAlotOffers->barcode                  =   $result->barcode;
          $takeAlotOffers->product_label_number     =   $result->product_label_number;
          $takeAlotOffers->selling_price            =   $result->selling_price;
          $takeAlotOffers->rrp                      =   $result->rrp;
          $takeAlotOffers->leadtime_days            =   $result->leadtime_days;
          $takeAlotOffers->leadtime_stock           =   json_encode($result->leadtime_stock);
          $takeAlotOffers->status                   =   $result->status;
          $takeAlotOffers->title                    =   $result->title;
          $takeAlotOffers->offer_url                =   $result->offer_url; 
          $takeAlotOffers->plid                     =   str_replace("https://www.takealot.com/x/","",$result->offer_url);
          $takeAlotOffers->stock_at_takealot        =   json_encode($result->stock_at_takealot);
          $takeAlotOffers->stock_on_way             =   json_encode($result->stock_on_way);
          $takeAlotOffers->total_stock_on_way       =   $result->total_stock_on_way;
          $takeAlotOffers->stock_cover              =   json_encode($result->stock_cover);
          $takeAlotOffers->total_stock_cover        =   $result->total_stock_cover;
          $takeAlotOffers->sales_units              =   json_encode($result->sales_units);
          $takeAlotOffers->stock_at_takealot_total  =   $result->stock_at_takealot_total;
          $takeAlotOffers->date_created             =   $result->date_created;
          $takeAlotOffers->storage_fee_eligible     =   $result->storage_fee_eligible;
          $takeAlotOffers->discount                 =   $result->discount;
          $takeAlotOffers->discount_shown           =   $result->discount_shown;
          $takeAlotOffers->replen_block_jhb         =   (isset($result->replen_block_jhb)) ? $result->replen_block_jhb : null;
          $takeAlotOffers->replen_block_cpt         =   (isset($result->replen_block_cpt)) ? $result->replen_block_cpt : null;                                    
          $takeAlotOffers->updated_through          =   0;                                    
          $takeAlotOffers->ip                       =   getUserIp();     
          $takeAlotOffers->image_url                =   $result->image_url;                                   
          if($takeAlotOffers->save())
          {
              $this->updateResponse=['status'=>'success','message'=>'Offer re-priced successfully'];
          }
          else
          {
              $this->updateResponse=['status'=>'error','message'=>'Error during offer updating on takealot'];
          }
        
      }
      public function updateSku($shopDetails,$offer_id,$sku)
      {                 
          $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
          if($ratelimit>0)
          { 
            $body=['sku' => $sku];    
           
            $apiCall=apiProxyCall('manual');   
            if(!empty($apiCall) && $apiCall->active_status==1 && $apiCall->reserved_limit>0 && isset($apiCall->api_url))
            {
              if($apiCall->api_url==env('APP_URL')) 
              {  
                  $allData=[
                    'takealot_api_key'=>$shopDetails->takealot_api_key,
                    'x-RateLimit-Limit'=>$this->rateLimit,
                    'x-RateLimit-Remaining'=>$ratelimit,
                    'base_url'=>takealotBaseUrl(),
                    'data'=>$body,
                    'offer_id'=>$offer_id
                  ];
  
                    $offerCont = new OfferApiController(); 
                    $result=$offerCont->updateTakealotOffer($allData);    
                    if(!empty($result) && !empty($shopDetails))
                    {                            
                        $resultDecoded =json_decode($result->getContent(),true);    
                        if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                        {
                          $resultDecoded  = json_decode($resultDecoded['data']); 
                          $this->updateResponse=['status'=>'success','message'=>'Sku updated successfully.'];
                          return response()->json($this->updateResponse);
                        }
                        else
                        {
                          $this->updateResponse=['status'=>'error','message'=>'Sorry! Data not updated.'];
                          return response()->json($this->updateResponse);
                        }
                     }
                     else
                     {
                        $this->updateResponse=['status'=>'error','message'=>'Sorry! takealot not returned updated data.'];
                        return response()->json($this->updateResponse);
                     }
              }
              else
              {
                $this->updateResponse=['status'=>'error','message'=>'Sorry! Something went wrong.'];
                return response()->json($this->updateResponse);
              }
            }
            else
            {
              $this->updateResponse=['status'=>'error','message'=>'Sorry! System default Api rate limit exceeded. Please retry after 1 minute.'];
              return response()->json($this->updateResponse);
            }
         
          }
          else
          {
            $this->updateResponse=['status'=>'error','message'=>'Sorry! Your shop Api rate limit exceeded. Please retry after 1 minute.'];
            return response()->json($this->updateResponse);
          }
      }
}
