<?php

namespace App\Http\Controllers\Seller;
use App\Http\Controllers\Controller;
use App\Models\TakealotShops;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TsinUpdateHistory;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Auth;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\HandlerStack;
use Spatie\GuzzleRateLimiterMiddleware\RateLimiterMiddleware;

use App\Http\Controllers\Api\OfferApiController;

use App\Jobs\FetchAllOffers; 
use App\Jobs\TakealotCreateOfferJob;
use App\Jobs\TakealoltOffersShopByJob;
use App\Jobs\TakealoltRefetchOffersUpdate;
use App\Jobs\TakealoltRefetchOffersShopByJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use App\Jobs\TakealotNewCreatedOffers;
use Illuminate\Bus\Batch;
use Illuminate\Support\Facades\Bus;
class OffersController extends Controller
{
    public $rateLimit=0;
    public $remaingCallByLimit=0;
    public $createOffersBatch;
    public $handlerStack;
    public function __construct()
    {
        $this->rateLimit=get_setting('takealot_ratelimit');
        $this->remaingCallByLimit=$this->rateLimit;
        $this->createOffersBatch=[];
        $this->handlerStack = HandlerStack::create();
        $this->handlerStack->push(RateLimiterMiddleware::perMinute($this->rateLimit));
        
    }
    public function index()
    {      
      return view('seller.offers.index');
    } 
    public function offersLive()
    {      
      return view('seller.offers.offers-live');
    }
    public function tsinHistory()
    {
        return view('seller.tsinHistory.tsinHistory');
    }
    public function takealotAllShopsOffersJob()
    {        
        FetchAllOffers::dispatch();       
    }
    //this is use for all sellers products
    public function fetchAllOffers($shopID=null)
    {
        $shops=null;
        $apiPeriority='low';
        if(!empty($shopID) && $shopID!=null)
        {
            $apiPeriority='high';
            $shops=TakealotShops::with('checkShopSubscription')->where('id','=',$shopID)->orderBy('created_at','desc')->get();
        }
        else
        {
            $apiPeriority='low';
            $shops=TakealotShops::with('checkShopSubscription')->inRandomOrder()->get();
        }

      
        if(!empty($shops))
        {
            $jobs = [];
            foreach($shops->chunk(1) as $chunk)
            {                
                foreach($chunk as $shopItem)
                {                  
                    $job = new TakealoltOffersShopByJob($shopItem);
                    $jobs[] = $job;
                }
            }
            if(!empty($jobs))
            {
                $batch = Bus::batch($jobs)->name('Shop by fetch all offers batch')->onQueue($apiPeriority)->dispatch();
                $jobs=[];
            }
        }
      
    }    
    public function offersShopBy($shop,$offersBatch=null)
    {
        ini_set('xdebug.max_nesting_level', -1);        
        if(!empty($shop->checkShopSubscription) && !empty($shop->takealot_api_key) && ($shop->checkShopSubscription->status=='active'))
        {           
            $this->offersCurl($shop,100,1,$this->rateLimit-1,$offersBatch);           
        } 
        
        if(!empty($this->createOffersBatch))
        {          
            $batch = Bus::batch($this->createOffersBatch)->name($shop->takealot_account_name.' offers in chunks batch')->onQueue('medium')->dispatch();
            $this->createOffersBatch=[];
        }
    }    
    // public function offersCurl($shop,$page_size,$page_number,$remainingCalls,$offersBatch)
    // {
    //    // ini_set('max_execution_time',-1);
    //     ini_set('xdebug.max_nesting_level', -1);
    //     $ipRateLimit=IpRateLimitChecker();
    //     if($ipRateLimit>0)
    //     { 
    //         $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);         
    //         if($ratelimit>0)
    //         {        
    //             $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //             $client  = new Client(['handler' => $this->handlerStack,'base_uri' => takealotBaseUrl(),'verify' => false]);
    //             $request = new Request('GET', 'offers?page_number='.$page_number.'&page_size='.$page_size.'',$headers);
    //             $response = $client->send($request);
    //             $result  = json_decode($response->getBody()->getContents()); 
    //             $promise = $client->sendAsync($request)->then(function ($response) use ($shop,$page_size,$page_number,$remainingCalls,$offersBatch) {
    //             $result  = json_decode($response->getBody()->getContents()); 
            
                
    //                 if(!empty($result) && (isset($result->offers) && !empty($result->offers) && count($result->offers)>0))
    //                 { 
    //                     if(!empty($result->offers))
    //                     {
                            
    //                         $job = new TakealotCreateOfferJob($shop,$result->offers);
    //                         $this->createOffersBatch[]=$job; 
    //                         if(!empty($this->createOffersBatch) && count($this->createOffersBatch)>=5)
    //                         {          
    //                             $batch = Bus::batch($this->createOffersBatch)->name($shop->takealot_account_name.' offers in chunks batch')->onQueue('medium')->dispatch();
    //                             $this->createOffersBatch=[];
    //                         }                
    //                         $this->offersCurl($shop,$page_size,$page_number+1,$remainingCalls-1,null);
    //                     }
    //                 }

    //             });
                
    //             $promise->wait();
    //         }
    //    }
    // }
    public function offersCurl($shop,$page_size,$page_number,$remainingCalls,$offersBatch)
    {
        ini_set('max_execution_time',-1);
        ini_set('xdebug.max_nesting_level', -1);
        $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
        if($ratelimit>0)
        {
             
          $apiCall=apiProxyCall();                
          if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
          {
            if($apiCall->api_url==env('APP_URL')) 
            {
                $allData=[
                    'takealot_api_key'=>$shop->takealot_api_key,
                    'x-RateLimit-Limit'=>$this->rateLimit,
                    'x-RateLimit-Remaining'=>$ratelimit,
                    'base_url'=>takealotBaseUrl(),
                    'page_number'=>$page_number,
                    'page_size'=>$page_size,                
                    ];

                  $offerCont = new OfferApiController(); 
                  $result=$offerCont->fetchAllOffers($allData);                
                  if(!empty($result) && !empty($shop))
                  {                            
                      $resultDecoded =json_decode($result->getContent(),true);    
                      if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                      {
                        $result  = json_decode($resultDecoded['data']); 
                        if(!empty($result) && (isset($result->offers) && !empty($result->offers) && count($result->offers)>0))
                        { 
                            if(!empty($result->offers))
                            {                                
                                $job = new TakealotCreateOfferJob($shop,$result->offers);
                                $this->createOffersBatch[]=$job; 
                                if(!empty($this->createOffersBatch) && count($this->createOffersBatch)>=5)
                                {          
                                    $batch = Bus::batch($this->createOffersBatch)->name($shop->takealot_account_name.' offers in chunks batch')->onQueue('medium')->dispatch();
                                    $this->createOffersBatch=[];
                                }                
                                $this->offersCurl($shop,$page_size,$page_number+1,$remainingCalls-1,null);
                            }
                        }
                      }
                   }
            }
            else
            {

                $allData=[
                'takealot_api_key'=>$shop->takealot_api_key,
                'x-RateLimit-Limit'=>$this->rateLimit,
                'x-RateLimit-Remaining'=>$ratelimit,
                'base_url'=>takealotBaseUrl(),
                'page_number'=>$page_number,
                'page_size'=>$page_size,                
                ];

                $postData=json_encode($allData);
                $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                $response = $client->post('/api/offers', [
                    'body' => $postData,
                    'headers' => [
                        'Content-Type' => 'application/json', // Set the content type to JSON
                    ]
                ]);             
                if($response->getStatusCode()==200)
                {
                        $result=$response->getBody()->getContents();
                        if(!empty($result) && !empty($shop))
                        { 
                            $resultDecoded  = json_decode($result,true); 
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                                $result  = json_decode($resultDecoded['data']); 
                                if(!empty($result) && (isset($result->offers) && !empty($result->offers) && count($result->offers)>0))
                                { 
                                    if(!empty($result->offers))
                                    {                                        
                                        $job = new TakealotCreateOfferJob($shop,$result->offers);
                                        $this->createOffersBatch[]=$job; 
                                        if(!empty($this->createOffersBatch) && count($this->createOffersBatch)>=5)
                                        {          
                                            $batch = Bus::batch($this->createOffersBatch)->name($shop->takealot_account_name.' offers in chunks batch')->onQueue('medium')->dispatch();
                                            $this->createOffersBatch=[];
                                        }                
                                        $this->offersCurl($shop,$page_size,$page_number+1,$remainingCalls-1,null);
                                    }
                                }
                            }
                            
                        }
                }
                
              
            }
          }
          else
          {
            return null;
          }
       
        }
        else
        {
            return null;
        }
       
    }
    public function insertOffer($shop,$chunk)
    {
       
        foreach($chunk as $key=>$item)
        {                  
            $takeAlotOffersCount = TakealotOffers::where([['tsin_id','=',$item->tsin_id],['offer_id','=',$item->offer_id],['seller_id','=',$shop->seller_id],['takealot_shops_id','=',$shop->id]])->count();
            if($takeAlotOffersCount==0 && empty($takeAlotOffersCount))
            {                
                $takeAlotOffers                           =   new TakealotOffers();
                $takeAlotOffers->seller_id                =   $shop->seller_id;
                $takeAlotOffers->takealot_shops_id        =   $shop->id;
                $takeAlotOffers->tsin_id                  =   $item->tsin_id;
                $takeAlotOffers->offer_id                 =   $item->offer_id;
                $takeAlotOffers->plid                     =   str_replace("https://www.takealot.com/x/","",$item->offer_url);
                $takeAlotOffers->sku                      =   $item->sku;
                $takeAlotOffers->barcode                  =   $item->barcode;
                $takeAlotOffers->product_label_number     =   $item->product_label_number;
                $takeAlotOffers->selling_price            =   $item->selling_price;
                $takeAlotOffers->rrp                      =   $item->rrp;
                $takeAlotOffers->leadtime_days            =   $item->leadtime_days;
                $takeAlotOffers->leadtime_stock           =   json_encode($item->leadtime_stock);
                $takeAlotOffers->status                   =   $item->status;
                $takeAlotOffers->title                    =   $item->title;
                $takeAlotOffers->offer_url                =   $item->offer_url;
                $takeAlotOffers->stock_at_takealot        =   json_encode($item->stock_at_takealot);
                $takeAlotOffers->stock_on_way             =   json_encode($item->stock_on_way);
                $takeAlotOffers->total_stock_on_way       =   $item->total_stock_on_way;
                $takeAlotOffers->stock_cover              =   json_encode($item->stock_cover);
                $takeAlotOffers->total_stock_cover        =   $item->total_stock_cover;
                $takeAlotOffers->sales_units              =   json_encode($item->sales_units);
                $takeAlotOffers->stock_at_takealot_total  =   $item->stock_at_takealot_total;
                $takeAlotOffers->date_created             =   $item->date_created;
                $takeAlotOffers->storage_fee_eligible     =   $item->storage_fee_eligible;
                $takeAlotOffers->discount                 =   $item->discount;
                $takeAlotOffers->discount_shown           =   $item->discount_shown;
                $takeAlotOffers->replen_block_jhb         =   (isset($item->replen_block_jhb) && !empty($item->replen_block_jhb)) ? $item->replen_block_jhb : 0;
                $takeAlotOffers->replen_block_cpt         =   (isset($item->replen_block_cpt) && !empty($item->replen_block_cpt)) ? $item->replen_block_cpt : 0;
                $takeAlotOffers->image_url                =   $item->image_url;                                    
                $takeAlotOffers->save();
                $shop->total_offers=$shop->total_offers+1;
                $shop->save();
                
            }
            else
            {
                $takeAlotOffers = TakealotOffers::where([['tsin_id','=',$item->tsin_id],['offer_id','=',$item->offer_id],['seller_id','=',$shop->seller_id],['takealot_shops_id','=',$shop->id]])->first();
                if(!empty($takeAlotOffers))
                {
                    $takeAlotOffers->seller_id                =   $shop->seller_id;
                    $takeAlotOffers->takealot_shops_id        =   $shop->id;
                    $takeAlotOffers->tsin_id                  =   $item->tsin_id;
                    $takeAlotOffers->offer_id                 =   $item->offer_id;
                    $takeAlotOffers->plid                     =   str_replace("https://www.takealot.com/x/","",$item->offer_url);
                    $takeAlotOffers->sku                      =   $item->sku;
                    $takeAlotOffers->barcode                  =   $item->barcode;
                    $takeAlotOffers->product_label_number     =   $item->product_label_number;
                    $takeAlotOffers->selling_price            =   $item->selling_price;
                    $takeAlotOffers->rrp                      =   $item->rrp;
                    $takeAlotOffers->leadtime_days            =   $item->leadtime_days;
                    $takeAlotOffers->leadtime_stock           =   json_encode($item->leadtime_stock);
                    $takeAlotOffers->status                   =   $item->status;
                    $takeAlotOffers->title                    =   $item->title;
                    $takeAlotOffers->offer_url                =   $item->offer_url;
                    $takeAlotOffers->stock_at_takealot        =   json_encode($item->stock_at_takealot);
                    $takeAlotOffers->stock_on_way             =   json_encode($item->stock_on_way);
                    $takeAlotOffers->total_stock_on_way       =   $item->total_stock_on_way;
                    $takeAlotOffers->stock_cover              =   json_encode($item->stock_cover);
                    $takeAlotOffers->total_stock_cover        =   $item->total_stock_cover;
                    $takeAlotOffers->sales_units              =   json_encode($item->sales_units);
                    $takeAlotOffers->stock_at_takealot_total  =   $item->stock_at_takealot_total;
                    $takeAlotOffers->date_created             =   $item->date_created;
                    $takeAlotOffers->storage_fee_eligible     =   $item->storage_fee_eligible;
                    $takeAlotOffers->discount                 =   $item->discount;
                    $takeAlotOffers->discount_shown           =   $item->discount_shown;
                    $takeAlotOffers->replen_block_jhb         =   (isset($item->replen_block_jhb) && !empty($item->replen_block_jhb)) ? $item->replen_block_jhb : 0;
                    $takeAlotOffers->replen_block_cpt         =   (isset($item->replen_block_cpt) && !empty($item->replen_block_cpt)) ? $item->replen_block_cpt : 0;
                    $takeAlotOffers->image_url                =   $item->image_url;                                    
                    $takeAlotOffers->save();
                }
               
            }
             
        }
        
    }
    // public function getOffersCount($shop)
    // {   
    //     $ipRateLimit=IpRateLimitChecker();
    //     if($ipRateLimit>0)
    //     {
    //         $ratelimit=shopRateLimitChecker($shop,$this->rateLimit); 
    //         if($ratelimit>0)
    //         {    
    //             $headers = ['Authorization' => 'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //             $client = new Client(['handler' => $this->handlerStack,'base_uri' =>  takealotBaseUrl(),'verify' => false]);
    //             $request = new Request('GET', 'offers/count',$headers);
    //             $response = $client->send($request);
    //             return json_decode($response->getBody()->getContents());
    //         }
    //     }
    // }
    public function getOffersCount($shop)
    {   
        
            $ratelimit=shopRateLimitChecker($shop,$this->rateLimit); 
            if($ratelimit>0)
            {    
                if(!empty($shop) && isset($shop->takealot_api_key))
                {
                
                    $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
                    if($ratelimit>0)
                    {
            
                        $apiCall=apiProxyCall();           
                        if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
                        {
                            $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit,'Content-Type' => 'application/json'];
        
                            if($apiCall->api_url==env('APP_URL')) 
                            {
                                $allData=[
                                'takealot_api_key'=>$shop->takealot_api_key,
                                'x-RateLimit-Limit'=>$this->rateLimit,
                                'x-RateLimit-Remaining'=>$ratelimit,
                                'base_url'=>takealotBaseUrl(),
                                ];
        
                                $offerCont = new OfferApiController(); 
                                $result=$offerCont->offersCount($allData);
                                if(!empty($result) && !empty($shop))
                                {                            
                                    $resultDecoded =json_decode($result->getContent(),true);    
                                    if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                    {
                                        $resultDecoded  = json_decode($resultDecoded['data']);                                                       
                                        return $resultDecoded;
                                    }
                                    else
                                    {
                                        return null;
                                    }
                                }
                                else
                                {
                                    return null;
                                }
                            }
                            else
                            {
                                    $allData=[
                                    'takealot_api_key'=>$shop->takealot_api_key,
                                    'x-RateLimit-Limit'=>$this->rateLimit,
                                    'x-RateLimit-Remaining'=>$ratelimit,
                                    'base_url'=>takealotBaseUrl(),
                                    ];
                                    $postData=json_encode($allData);
                                    $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                                    $response = $client->post('/api/offers/offer/'.$offer_id, [
                                        'body' => $postData,
                                        'headers' => [
                                        'Content-Type' => 'application/json', // Set the content type to JSON
                                    ]
                                    ]);
                                    
                                    if($response->getStatusCode()==200)
                                    {
                                        $result=$response->getBody()->getContents();
                                        if(!empty($result) && !empty($shop))
                                        { 
                                            $resultDecoded  = json_decode($result,true); 
                                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                            {
                                                $resultDecoded  = json_decode($resultDecoded['data']); 
                                                return $resultDecoded;
                                            } 
                                            else
                                            {
                                                return null;
                                            }
            
                                        }
                                        else
                                        {
                                            return null;
                                        }
                                    }
                                    else
                                    {
                                        return null;
                                    }
                                    
                            
                            }
                        }
                         
                  }
                   
                }
                else 
                {
                    return null;
                }
            }
        
    }
    public function updateTakealotOffer($offer_id,$selling_price,$shopDetails,$rePricing=null)
    { 
            $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
            if($ratelimit>0)
            {
              // if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null && $rePricing->offer->stock_at_takealot_total<=0))
              if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null))
             {
                $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
                $leadtime_qty=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->quantity_available)) ? $leadtime_decode[0]->quantity_available : 0;
                $merchant_warehouse_id=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->merchant_warehouse->warehouse_id)) ? $leadtime_decode[0]->merchant_warehouse->warehouse_id : 0;
               
                if(!empty($merchant_warehouse_id) && $merchant_warehouse_id>0)
                {
                   
                    $leadtime=[
                        'merchant_warehouse_id'=>$merchant_warehouse_id,
                        'quantity'=>(int)$leadtime_qty
                    ];
                    $body=[
                        'selling_price' => (int)$selling_price,
                        'leadtime_stock' => [$leadtime],    
                    ];

                }
                else
                {
                    $body=['selling_price' => (int)$selling_price];    
                }                    
                
            }
            else
            {   
                $body=['selling_price' => (int)$selling_price];    
            }


              $apiCall=apiProxyCall();                
              if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
              {
                if($apiCall->api_url==env('APP_URL')) 
                {
                    $allData=[
                      'takealot_api_key'=>$shopDetails->takealot_api_key,
                      'x-RateLimit-Limit'=>$this->rateLimit,
                      'x-RateLimit-Remaining'=>$ratelimit,
                      'base_url'=>takealotBaseUrl(),
                      'data'=>$body,
                      'offer_id'=>$offer_id
                    ];

                     $offerCont = new OfferApiController(); 
                     $result=$offerCont->updateTakealotOffer($allData);
                    
                      if(!empty($result) && !empty($rePricing))
                      {                            
                          $resultDecoded =json_decode($result->getContent(),true);    
                          if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                          {
                            $resultDecoded  = json_decode($resultDecoded['data']); 
                            $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                          }
                          else
                          {
                            $this->updateResponse=['status'=>'error','message'=>'Getting empty data after offer updating on takealot'];
                          }
                       }
                       else
                       {
                          $this->updateResponse=['status'=>'error','message'=>"Offer not updated something went wrong."];
                       }
                }
                else
                {

                  $allData=[
                    'takealot_api_key'=>$shopDetails->takealot_api_key,
                    'x-RateLimit-Limit'=>$this->rateLimit,
                    'x-RateLimit-Remaining'=>$ratelimit,
                    'base_url'=>takealotBaseUrl(),
                    'data'=>$body,
                    'offer_id'=>$offer_id
                  ];
                  $postData=json_encode($allData);
                  $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                  $response = $client->post('/api/update/offer/'.$offer_id, [
                      'body' => $postData,
                      'headers' => [
                        'Content-Type' => 'application/json', // Set the content type to JSON
                    ]
                  ]);
                 
                    if($response->getStatusCode()==200)
                    {
                            $result=$response->getBody()->getContents();
                            if(!empty($result) && !empty($rePricing))
                            { 
                                $resultDecoded  = json_decode($result,true); 
                                if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                {
                                    $resultDecoded  = json_decode($resultDecoded['data']); 
                                    $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                                }
                                
                            }
                    }
                    
                  
                }
              }
           
            }
     
      
    }
    public function updateTakealotOfferSoh($offer_id,$shopDetails,$rePricing)
    {
       
        $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
        if($ratelimit>0)
        {

            $headers = ['Authorization'=>'Key '.$shopDetails->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit,'Content-Type' => 'application/json'];
            
      
            if(!empty($rePricing) && ($rePricing->offer->leadtime_days!=null))
            {
              
                $leadtime_decode=json_decode($rePricing->offer->leadtime_stock);
                $leadtime_qty=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->quantity_available)) ? $leadtime_decode[0]->quantity_available : 0;
                
                $merchant_warehouse_id=(!empty($leadtime_decode[0]) && !empty($leadtime_decode[0]->merchant_warehouse->warehouse_id)) ? $leadtime_decode[0]->merchant_warehouse->warehouse_id : 0;
              
                if(!empty($merchant_warehouse_id) && $merchant_warehouse_id>0 )
                {  
                    
                    $leadtime=[
                        'merchant_warehouse_id'=>$merchant_warehouse_id,
                        'quantity'=>(int)$leadtime_qty
                    ];
                    $body=[                        
                        'leadtime_stock' => [$leadtime]                            
                    ];
                    $apiCall=apiProxyCall();           
                    if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
                    {
                      if($apiCall->api_url==env('APP_URL')) 
                      {
                          $allData=[
                            'takealot_api_key'=>$shopDetails->takealot_api_key,
                            'x-RateLimit-Limit'=>$this->rateLimit,
                            'x-RateLimit-Remaining'=>$ratelimit,
                            'base_url'=>takealotBaseUrl(),
                            'data'=>$body,
                            'offer_id'=>$offer_id
                          ];
    
                            $offerCont = new OfferApiController(); 
                            $result=$offerCont->updateTakealotOffer($allData);
                            if(!empty($result) && !empty($rePricing))
                            {                            
                                $resultDecoded =json_decode($result->getContent(),true);    
                                if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                {
                                  $resultDecoded  = json_decode($resultDecoded['data']); 
                                  $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                                }
                              }
                      }
                      else
                      {
    
                        $allData=[
                          'takealot_api_key'=>$shopDetails->takealot_api_key,
                          'x-RateLimit-Limit'=>$this->rateLimit,
                          'x-RateLimit-Remaining'=>$ratelimit,
                          'base_url'=>takealotBaseUrl(),
                          'data'=>$body,
                          'offer_id'=>$offer_id
                        ];
                        $postData=json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                        $response = $client->post('/api/update/offer/'.$offer_id, [
                            'body' => $postData,
                            'headers' => [
                              'Content-Type' => 'application/json', // Set the content type to JSON
                          ]
                        ]);
                         
                        if($response->getStatusCode()==200)
                        {
                            $result=$response->getBody()->getContents();
                            if(!empty($result) && !empty($rePricing))
                            { 
                                $resultDecoded  = json_decode($result,true); 
                                if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                {
                                  $resultDecoded  = json_decode($resultDecoded['data']); 
                                  $this->updateRefetchedOffer($resultDecoded->offer,$rePricing->offer);
                                }

                            }
                        }
                          
                        
                      }
                    }
                    

                }    
            }
      } 
    }
   
    public function updateRefetchedOffer($result,$takeAlotOffers,$auto=false)
    {  
       
        $takeAlotOffers->sku                      =   $result->sku;
        $takeAlotOffers->barcode                  =   $result->barcode;
        $takeAlotOffers->product_label_number     =   $result->product_label_number;
        $takeAlotOffers->selling_price            =   $result->selling_price;
        $takeAlotOffers->rrp                      =   $result->rrp;
        $takeAlotOffers->leadtime_days            =   $result->leadtime_days;
        $takeAlotOffers->leadtime_stock           =   json_encode($result->leadtime_stock);
        $takeAlotOffers->status                   =   $result->status;
        $takeAlotOffers->title                    =   $result->title;
        $takeAlotOffers->offer_url                =   $result->offer_url;
        $takeAlotOffers->plid                     =   str_replace("https://www.takealot.com/x/","",$result->offer_url);
        $takeAlotOffers->stock_at_takealot        =   json_encode($result->stock_at_takealot);
        $takeAlotOffers->stock_on_way             =   json_encode($result->stock_on_way);
        $takeAlotOffers->total_stock_on_way       =   $result->total_stock_on_way;
        $takeAlotOffers->stock_cover              =   json_encode($result->stock_cover);
        $takeAlotOffers->total_stock_cover        =   $result->total_stock_cover;
        $takeAlotOffers->sales_units              =   json_encode($result->sales_units);
        $takeAlotOffers->stock_at_takealot_total  =   $result->stock_at_takealot_total;
        $takeAlotOffers->date_created             =   $result->date_created;
        $takeAlotOffers->storage_fee_eligible     =   $result->storage_fee_eligible;
        $takeAlotOffers->discount                 =   $result->discount;
        $takeAlotOffers->discount_shown           =   $result->discount_shown;
        $takeAlotOffers->replen_block_jhb         =   (isset($item->replen_block_jhb) && !empty($item->replen_block_jhb)) ? $item->replen_block_jhb : 0;
        $takeAlotOffers->replen_block_cpt         =   (isset($item->replen_block_cpt) && !empty($item->replen_block_cpt)) ? $item->replen_block_cpt : 0; 

        $takeAlotOffers->updated_through          =   0;                                    
        $takeAlotOffers->ip                       =   getUserIp();     
        $takeAlotOffers->image_url                =   $result->image_url;                                   
        if($takeAlotOffers->save())
        {
            
            return true;
        }
        else
        {
            return false;
        }
       
    }
    public function reFetchAllOffersJob($shopID=null)
    {
        ini_set('max_execution_time',-1);
        $shops=null;
        if(!empty($shopID) && $shopID!=null)
        {
            $shops=TakealotShops::with('checkShopSubscription')->where('id','=',$shopID)->orderBy('created_at','desc')->get();
        }
        else
        {
            $shops=TakealotShops::with('checkShopSubscription')->orderBy('created_at','desc')->get();
        }

       
        if(!empty($shops))
        {
            //refetch            
            $jobs = [];
            foreach($shops->chunk(1) as $chunk)
            {
                foreach($chunk as $shop)
                {                    
                    $job = new TakealoltRefetchOffersShopByJob($shop);
                    $jobs[] = $job;  
                }
            }
            if(!empty($jobs))
            {
                $batch = Bus::batch($jobs)->name('Re Fetch All Offers Job Batch')->onQueue('low')->dispatch();
                $jobs=[];
            }
        }
      
    }
    public function reFetchOffersShopBy($shop)
    {
        if(!empty($shop) && !empty($shop->checkShopSubscription) && !empty($shop->takealot_api_key) && ($shop->checkShopSubscription->status=='active'))
        {
            if(!empty($shop))
            {
                $reftechOffer=TakealotOffers::with('takealotShops')->where('takealot_shops_id','=',$shop->id)->orderBy('offer_id','desc')->get();
                $jobs = [];
                foreach($reftechOffer->chunk(100) as $chunk)
                { 
                    $job = new TakealoltRefetchOffersUpdate($chunk);
                    $jobs[] = $job;                  
                }
                if(!empty($jobs))
                {
                    $batch = Bus::batch($jobs)->name('Re Fetch All Offers Shop By Job Batch')->onQueue('low')->dispatch();
                    $jobs=[];
                }
            }
                
        }  
    }
    public function offersUpdateJob($offers)
    {
        foreach($offers as $offer)
        {
            $this->reFetchOffer($offer); 
        }
    } 
    public function reFetchOffer($reftechOffer,$apiCallType=null)
    {
        if(!empty($reftechOffer))
        {
        
            $ratelimit=shopRateLimitChecker($reftechOffer->takealotShops,$this->rateLimit);
            if($ratelimit>0)
            {
    
                $apiCall=apiProxyCall($apiCallType);       
                if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
                {
                    $headers = ['Authorization'=>'Key '.$reftechOffer->takealotShops->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit,'Content-Type' => 'application/json'];

                    if($apiCall->api_url==env('APP_URL')) 
                    {
                       
                        $allData=[
                        'takealot_api_key'=>$reftechOffer->takealotShops->takealot_api_key,
                        'x-RateLimit-Limit'=>$this->rateLimit,
                        'x-RateLimit-Remaining'=>$ratelimit,
                        'base_url'=>takealotBaseUrl(),
                        'offer_id'=>$reftechOffer->offer_id
                        ];

                        $offerCont = new OfferApiController(); 
                        $result=$offerCont->fetchSingleOffer($allData);
                        if(!empty($result) && !empty($reftechOffer))
                        {                            
                            $resultDecoded =json_decode($result->getContent(),true);    
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                                $resultDecoded  = json_decode($resultDecoded['data']); 
                                $flag=$this->updateRefetchedOffer($resultDecoded,$reftechOffer,true);                
                                return true;
                            }
                            else
                            {
                                return false;
                            }
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                            $allData=[
                            'takealot_api_key'=>$reftechOffer->takealotShops->takealot_api_key,
                            'x-RateLimit-Limit'=>$this->rateLimit,
                            'x-RateLimit-Remaining'=>$ratelimit,
                            'base_url'=>takealotBaseUrl(),
                            'offer_id'=>$reftechOffer->offer_id
                            ];
                            $postData=json_encode($allData);
                            $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                            $response = $client->post('/api/offers/offer/'.$reftechOffer->offer_id, [
                                'body' => $postData,
                                'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                            ]);
                            
                            if($response->getStatusCode()==200)
                            {
                                $result=$response->getBody()->getContents();
                                if(!empty($result) && !empty($rePricing))
                                { 
                                    $resultDecoded  = json_decode($result,true); 
                                    if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                                    {
                                        $resultDecoded  = json_decode($resultDecoded['data']); 
                                        $flag=$this->updateRefetchedOffer($resultDecoded,$reftechOffer,true);  
                                        return true;
                                    } 
                                    else
                                    {
                                        return false;
                                    }
    
                                }
                                else
                                {
                                    return false;
                                }
                            }
                            else
                            {
                                return false;
                            }
                            
                    
                    }
                }
                 
          }
           
        }
        else 
        {
            return false;
        }
        
    }  
    public function updateSku($shopDetails,$offer_id,$sku)
    {                 
        $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
        if($ratelimit>0)
        {
          $body=['sku' => $sku];    
          $apiCall=apiProxyCall();                
          if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
          {
            if($apiCall->api_url==env('APP_URL')) 
            {
                $allData=[
                  'takealot_api_key'=>$shopDetails->takealot_api_key,
                  'x-RateLimit-Limit'=>$this->rateLimit,
                  'x-RateLimit-Remaining'=>$ratelimit,
                  'base_url'=>takealotBaseUrl(),
                  'data'=>$body,
                  'offer_id'=>$offer_id
                ];

                  $offerCont = new OfferApiController(); 
                  $result=$offerCont->updateTakealotOffer($allData);                
                  if(!empty($result) && !empty($shopDetails))
                  {                            
                      $resultDecoded =json_decode($result->getContent(),true);    
                      if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                      {
                        $resultDecoded  = json_decode($resultDecoded['data']); 
                        return true;
                      }
                      else
                      {
                        return false;
                      }
                   }
                   else
                   {
                     return false;
                   }
            }
            else
            {

                $allData=[
                'takealot_api_key'=>$shopDetails->takealot_api_key,
                'x-RateLimit-Limit'=>$this->rateLimit,
                'x-RateLimit-Remaining'=>$ratelimit,
                'base_url'=>takealotBaseUrl(),
                'data'=>$body,
                'offer_id'=>$offer_id
                ];
                $postData=json_encode($allData);
                $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                $response = $client->post('/api/update/offer/'.$offer_id, [
                    'body' => $postData,
                    'headers' => [
                        'Content-Type' => 'application/json', // Set the content type to JSON
                    ]
                ]);             
                if($response->getStatusCode()==200)
                {
                        $result=$response->getBody()->getContents();
                        if(!empty($result) && !empty($rePricing))
                        { 
                            $resultDecoded  = json_decode($result,true); 
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                                $resultDecoded  = json_decode($resultDecoded['data']); 
                                return true;
                            }
                            else
                            {
                                return false;
                            }
                            
                        }
                        else
                        {
                            return false;
                        }
                }
                else
                {
                    return false;
                }
                
              
            }
          }
          else
          {
            return false;
          }
       
        }
        else
        {
            return false;
        }
    }
    public function updateSellingPrice($shopDetails,$offer_id,$selling_price)
    {                           
        $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
        if($ratelimit>0)
        {
          $body=['selling_price' => (int)$selling_price];    
          $apiCall=apiProxyCall();                
          if(!empty($apiCall) && $apiCall->remaining_limit>0 &&  $apiCall->status==1 && isset($apiCall->api_url))
          {
            if($apiCall->api_url==env('APP_URL')) 
            {
                $allData=[
                  'takealot_api_key'=>$shopDetails->takealot_api_key,
                  'x-RateLimit-Limit'=>$this->rateLimit,
                  'x-RateLimit-Remaining'=>$ratelimit,
                  'base_url'=>takealotBaseUrl(),
                  'data'=>$body,
                  'offer_id'=>$offer_id
                ];

                  $offerCont = new OfferApiController(); 
                  $result=$offerCont->updateTakealotOffer($allData);                
                  if(!empty($result) && !empty($shopDetails))
                  {                            
                      $resultDecoded =json_decode($result->getContent(),true);    
                      if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                      {
                        $resultDecoded  = json_decode($resultDecoded['data']); 
                        return true;
                      }
                      else
                      {
                        return false;
                      }
                   }
                   else
                   {
                     return false;
                   }
            }
            else
            {

                $allData=[
                'takealot_api_key'=>$shopDetails->takealot_api_key,
                'x-RateLimit-Limit'=>$this->rateLimit,
                'x-RateLimit-Remaining'=>$ratelimit,
                'base_url'=>takealotBaseUrl(),
                'data'=>$body,
                'offer_id'=>$offer_id
                ];
                $postData=json_encode($allData);
                $client   = new Client(['base_uri' => $apiCall->api_url,'verify' => false]);
                $response = $client->post('/api/update/offer/'.$offer_id, [
                    'body' => $postData,
                    'headers' => [
                        'Content-Type' => 'application/json', // Set the content type to JSON
                    ]
                ]);             
                if($response->getStatusCode()==200)
                {
                        $result=$response->getBody()->getContents();
                        if(!empty($result) && !empty($rePricing))
                        { 
                            $resultDecoded  = json_decode($result,true); 
                            if(!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data']))
                            {
                                $resultDecoded  = json_decode($resultDecoded['data']); 
                                return true;
                            }
                            else
                            {
                                return false;
                            }
                            
                        }
                        else
                        {
                            return false;
                        }
                }
                else
                {
                    return false;
                }
                
              
            }
          }
          else
          {
            return false;
          }
       
        }
        else
        {
            return false;
        }
           
            
        
    }
    public function newCreatedOffer()
    {
        
        $reftechOffer=TakealotOffers::with('rePricing','takealotShops')->where('title','=',null)->orWhere('title','=','')->orderBy('id','asc')->get();
        $jobs = [];
        if(!empty($reftechOffer) && count($reftechOffer)>0)
        {
            $updateOffers=Bus::batch([])->name('New Created Offers Batch')->onQueue('high')->dispatch();
            foreach($reftechOffer->chunk(100) as $chunk)
            {
                $job = new TakealotNewCreatedOffers($chunk);
                $jobs[] = $job;               
            }
            if(!empty($jobs))
            {
                $batch = Bus::batch($jobs)->name('New Created Offers Batch')->onQueue('high')->dispatch();
                $jobs=[];
            }
           
           
        }
    }
    //create history of tsin update    
    public function tsinUpdateHistory($oldData,$newData,$type)
    {
            $data=[
                'old'=>$oldData,
                'new'=>$newData
            ];
            $data=json_encode($data);           
            $tsinHistory=new TsinUpdateHistory();
            $tsinHistory->seller_id=$oldData['seller_id'];
            $tsinHistory->takealot_shops_id=$oldData['takealot_shops_id'];
            $tsinHistory->updated_through=$newData['updated_through'];
            $tsinHistory->tsin=$oldData['tsin'];
            $tsinHistory->data=$data;
            $tsinHistory->ip_address=$newData['ip'];
            $tsinHistory->type=$type;
            if($tsinHistory->save())
            {
                
               return true;
            }
            else
            {
              return false;
            }
        
       
    }
    public function tsinUpdateHistoryWithObject($object,$type)
    {
           
            $data=json_encode($object);           
            $tsinHistory=new TsinUpdateHistory();
            $tsinHistory->seller_id=$object['repricing_status']['seller_id'];
            $tsinHistory->takealot_shops_id=$object['repricing_status']['takealot_shops_id'];
            $tsinHistory->updated_through=$object['repricing_status']['updated_through'];
            $tsinHistory->tsin=$object['repricing_status']['tsin'];
            $tsinHistory->data=$data;
            $tsinHistory->ip_address=$object['repricing_status']['ip'];
            $tsinHistory->type=$type;
            if($tsinHistory->save())
            {                
               return true;
            }
            else
            {
              return false;
            }
        
       
    }

}
