<?php

namespace App\Http\Controllers\Seller;

use App\Http\Controllers\Controller;
use App\Models\TakealotShops;
use App\Models\TakealotOrders;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Auth;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use App\Jobs\TakealotCreateSalesJob;
use App\Jobs\TakealotFatchAllSalesJob;
use App\Jobs\TakealoltSalesShopByJob;
use App\Jobs\TakealotUpdateOrderStatus;
use Illuminate\Bus\Batch;
use Illuminate\Support\Facades\Bus;
use PDF;
use GuzzleHttp\HandlerStack;
use Spatie\GuzzleRateLimiterMiddleware\RateLimiterMiddleware;
use App\Http\Controllers\Api\OrdersApiController;

class OrdersController extends Controller
{
    public $rateLimit = 0;
    public $remaingCallByLimit = 0;
    public $createOrdersBatch;
    public $handlerStack;
    public function __construct()
    {
        $this->rateLimit = get_setting('takealot_ratelimit');
        $this->remaingCallByLimit = $this->rateLimit;
        $this->createOrdersBatch = [];
        $this->handlerStack = HandlerStack::create();
        $this->handlerStack->push(RateLimiterMiddleware::perMinute($this->rateLimit));
    }
    public function index()
    {

        return view('seller.orders.index');
    }
    public function downloadOffersPdf($takealotOffers)
    {
        $pdf = PDF::loadView('components.replenishment-pdf', $takealotOffers)->setOptions(['defaultFont' => 'sans-serif']);
        // $pdf->loadView('pdf.welcome', $data);

        return response()->streamDownload(function () use ($pdf) {
            $pdf->stream();
        }, 'welcome.pdf');
    }
    public function takealotAllShopsSalesJob()
    {

        TakealotFatchAllSalesJob::dispatch();
    }
    public function fetchAllSales($shopID = null)
    {

        ini_set('max_execution_time', -1);
        $shops = null;
        $timePeriod = null;
        if (!empty($shopID) && $shopID != null) {
            //6 months
            $timePeriod = 6;
            $shops = TakealotShops::with('checkShopSubscription')->where('id', '=', $shopID)->orderBy('created_at', 'desc')->get();
        } else {
            //1 day
            $timePeriod = 1;
            $shops = TakealotShops::with('checkShopSubscription')->orderBy('created_at', 'desc')->get();
        }

        if (!empty($shops)) {
            $jobs = [];
            foreach ($shops->chunk(1) as $chunk) {

                foreach ($chunk as $key => $shop) {

                    $job = new TakealoltSalesShopByJob($shop,$timePeriod);
                    $jobs[] = $job;   
                    // $this->salesShopBy($shop, $timePeriod);
                }
            }
            if (!empty($jobs)) {
                $batch = Bus::batch($jobs)->name('Shop by fetch all sales job batch')->onQueue('medium')->dispatch();
                $jobs = [];
            }
        }
    }
    public function salesShopBy($shop, $timePeriod, $salesBatch = null)
    {
        ini_set('max_execution_time', -1);
        ini_set('xdebug.max_nesting_level', -1);

        try {


            if (!empty($shop->checkShopSubscription) && !empty($shop->takealot_api_key) && ($shop->checkShopSubscription->status == 'active')) {

                $totalPages = $this->getSalesCount($shop, $timePeriod);
                if (!empty($totalPages) && isset($totalPages->page_summary->total)) {
                    $this->ordersCurl($shop, 100, 1, $this->remaingCallByLimit, $totalPages->page_summary->total, $timePeriod, $salesBatch);
                }
                if (!empty($this->createOrdersBatch)) {
                    $batch = Bus::batch($this->createOrdersBatch)->name($shop->takealot_account_name . ' Fetch Sales in chunk Job Batch')->onQueue('medium')->dispatch();
                    $this->createOrdersBatch = [];
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function ordersCurl($shop, $page_size, $page_number, $remainingCalls, $totalPages, $timePeriod, $salesBatch)
    {
        ini_set('max_execution_time', -1);
        ini_set('xdebug.max_nesting_level', -1);
        if ($timePeriod == 6) {
            $startDate = Carbon::now()->subDays(180)->toDateString();
        } elseif ($timePeriod == 1) {
            $startDate = Carbon::now()->subDays(1)->toDateString();
        } else {
            $startDate = Carbon::now()->subDays(2)->toDateString();
        }

        try {
            sleep(1);
            $endDate = Carbon::now()->toDateString();
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllOrders($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {

                                    $job = new TakealotCreateSalesJob($shop, $result->sales);
                                    $this->createOrdersBatch[] = $job;
                                    if (!empty($this->createOrdersBatch) && count($this->createOrdersBatch) >= 5) {
                                        $batch = Bus::batch($this->createOrdersBatch)->name($shop->takealot_account_name . ' Fetch Sales in chunk Job Batch')->onQueue('medium')->dispatch();
                                        $this->createOrdersBatch = [];
                                    }
                                    $this->ordersCurl($shop, $page_size, $page_number + 1, $ratelimit, $result->page_summary->total, $timePeriod, null);
                                }
                            }
                        }
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders/filters', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {

                                        $job = new TakealotCreateSalesJob($shop, $result->sales);
                                        $this->createOrdersBatch[] = $job;
                                        if (!empty($this->createOrdersBatch) && count($this->createOrdersBatch) >= 5) {
                                            $batch = Bus::batch($this->createOrdersBatch)->name($shop->takealot_account_name . ' Fetch Sales in chunk Job Batch')->onQueue('medium')->dispatch();
                                            $this->createOrdersBatch = [];
                                        }
                                        $this->ordersCurl($shop, $page_size, $page_number + 1, $ratelimit, $result->page_summary->total, $timePeriod, null);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function createOrder($shop, $result)
    {
        // ini_set('max_execution_time',-1);
        foreach ($result as $key => $item) {
            try {
                $takeAlotOrder = TakealotOrders::where([['order_item_id', '=', $item->order_item_id], ['order_id', '=', $item->order_id], ['seller_id', '=', $shop->seller_id], ['takealot_shops_id', '=', $shop->id]])->first();
                if (empty($takeAlotOrder)) {

                    $dateObject = Carbon::parse($item->order_date);
                    $takeAlotOrder                          =   new TakealotOrders();
                    $takeAlotOrder->seller_id               =   $shop->seller_id;
                    $takeAlotOrder->takealot_shops_id       =   $shop->id;
                    $takeAlotOrder->order_item_id           =   $item->order_item_id;
                    $takeAlotOrder->order_id                =   $item->order_id;
                    $takeAlotOrder->order_date              =   $dateObject->format('Y-m-d h:i:s');
                    $takeAlotOrder->sale_status             =   $item->sale_status;
                    $takeAlotOrder->offer_id                =   $item->offer_id;
                    $takeAlotOrder->tsin                    =   $item->tsin;
                    $takeAlotOrder->sku                     =   $item->sku;
                    $takeAlotOrder->product_title           =   $item->product_title;
                    $takeAlotOrder->takealot_url_mobi       =   $item->takealot_url_mobi;
                    $takeAlotOrder->selling_price           =   $item->selling_price;
                    $takeAlotOrder->quantity                =   $item->quantity;
                    $takeAlotOrder->dc                      =   $item->dc;
                    if (isset($item->warehouse) && !empty($item->warehouse)) {
                        $takeAlotOrder->warehouse           =   (isset($item->warehouse) && !empty($item->warehouse)) ? $item->warehouse : ((isset($takeAlotOrder->warehouse) && !empty($takeAlotOrder->warehouse)) ? $takeAlotOrder->warehouse : '');
                    } else {
                        $takeAlotOrder->warehouse  =   (isset($item->dc) && !empty($item->dc)) ? $item->dc : ((isset($takeAlotOrder->dc) && !empty($takeAlotOrder->dc)) ? $takeAlotOrder->dc : '');
                    }
                    $takeAlotOrder->customer                =   $item->customer;
                    $takeAlotOrder->takealot_url            =   $item->takealot_url;
                    $takeAlotOrder->success_fee             =   $item->success_fee;
                    $takeAlotOrder->fulfillment_fee         =   $item->fulfillment_fee;
                    $takeAlotOrder->courier_collection_fee  =   $item->courier_collection_fee;
                    $takeAlotOrder->auto_ibt_fee            =   $item->auto_ibt_fee;
                    $takeAlotOrder->total_fee               =   $item->total_fee;
                    $takeAlotOrder->save();

                    $shop->total_orders = $shop->total_orders + 1;
                    $shop->save();
                } else {
                    if (!empty($takeAlotOrder)) {
                        $this->updateOrders($takeAlotOrder, $shop, $item);
                    }
                }
            } catch (\Exception $e) {
                Logger($e);
            }
        }
    }
    public function updateOrders($takeAlotOrder, $shop, $item)
    {

        // ini_set('max_execution_time',-1);
        $takeAlotOrder->seller_id               =   $shop->seller_id;
        $takeAlotOrder->takealot_shops_id       =   $shop->id;
        $takeAlotOrder->order_item_id           =   $item->order_item_id;
        $takeAlotOrder->order_id                =   $item->order_id;
        $takeAlotOrder->sale_status             =   $item->sale_status;
        $takeAlotOrder->offer_id                =   $item->offer_id;
        $takeAlotOrder->tsin                    =   $item->tsin;
        $takeAlotOrder->sku                     =   $item->sku;
        $takeAlotOrder->product_title           =   $item->product_title;
        $takeAlotOrder->takealot_url_mobi       =   $item->takealot_url_mobi;
        $takeAlotOrder->selling_price           =   $item->selling_price;
        $takeAlotOrder->quantity                =   $item->quantity;
        $takeAlotOrder->dc                      =   $item->dc;
        if (isset($item->warehouse) && !empty($item->warehouse)) {
            $takeAlotOrder->warehouse           =   (isset($item->warehouse) && !empty($item->warehouse)) ? $item->warehouse : ((isset($takeAlotOrder->warehouse) && !empty($takeAlotOrder->warehouse)) ? $takeAlotOrder->warehouse : '');
        } else {
            $takeAlotOrder->warehouse  =   (isset($item->dc) && !empty($item->dc)) ? $item->dc : ((isset($takeAlotOrder->dc) && !empty($takeAlotOrder->dc)) ? $takeAlotOrder->dc : '');
        }
        $takeAlotOrder->customer                =   $item->customer;
        $takeAlotOrder->takealot_url            =   $item->takealot_url;
        $takeAlotOrder->success_fee             =   $item->success_fee;
        $takeAlotOrder->fulfillment_fee         =   $item->fulfillment_fee;
        $takeAlotOrder->courier_collection_fee  =   $item->courier_collection_fee;
        $takeAlotOrder->auto_ibt_fee            =   $item->auto_ibt_fee;
        $takeAlotOrder->total_fee               =   $item->total_fee;
        $takeAlotOrder->save();

        $shop->total_orders = $shop->total_orders + 1;
        $shop->save();
    }

    public function getSalesCount($shop, $timePeriod = null)
    {
        sleep(1);
        $ipRateLimit = IpRateLimitChecker();
        if ($ipRateLimit > 0) {
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {
                if (!empty($timePeriod)) {
                    if ($timePeriod == 6) {
                        $startDate = Carbon::now()->subDays(180)->toDateString();
                    } elseif ($timePeriod == 1) {
                        $startDate = Carbon::now()->subDays(1)->toDateString();
                    } else {
                        $startDate = Carbon::now()->subDays(2)->toDateString();
                    }

                    $endDate = Carbon::now()->toDateString();

                    $returnedData = $this->allDateWiseSalesCount($shop, $startDate, $endDate);
                    if ($returnedData) {
                        return $returnedData;
                    }

                    return null;
                } else {
                    $returnedData =   $this->allSalesCount($shop);
                    if ($returnedData) {
                        return $returnedData;
                    }

                    return null;
                }
            }
        }
    }
    public function allDateWiseSalesCount($shop, $startDate, $endDate)
    {
        try {
            sleep(1);
            $endDate = Carbon::now()->toDateString();
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                // $apiCall = apiProxyCall('manual');
                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => 1,
                            'page_size' => 100,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];


                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllOrders($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                return  $result;
                            }
                        }
                        return null;
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => 1,
                            'page_size' => 100,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders/filters', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    return  $result;
                                }
                            }
                        }

                        return null;
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function allSalesCount($shop)
    {

        try {
            sleep(1);
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => 1,
                            'page_size' => 100,
                        ];

                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllSales($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                return  $result;
                            }
                        }
                        return null;
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => 1,
                            'page_size' => 100,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    return  $result;
                                }
                            }
                        }

                        return null;
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function updateSingleOrder($shop, $tsin, $page_size = 100, $page_number = 1)
    {



        try {
            ini_set('max_execution_time', -1);
            sleep(1);
            $startDate = Carbon::now()->subDays(2)->toDateString();
            $endDate = Carbon::now()->toDateString();
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                            'tsin' => $tsin,
                        ];

                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllOrders($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                    $shop = $this->orderUpdate($shop, $result->sales);
                                    if (!empty($shop)) {
                                        $this->updateSingleOrder($shop, $tsin, $page_size, $page_number + 1);
                                    }
                                }
                            }
                        }
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                            'tsin' => $tsin,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders/filters', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                        $shop = $this->orderUpdate($shops, $result->sales);
                                        if (!empty($shop)) {
                                            $this->updateSingleOrder($shops, $tsin, $page_size, $page_number + 1);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function orderUpdate($shop, $result)
    {
        ini_set('max_execution_time', -1);
        if (!empty($result) && count($result) > 0) {
            foreach ($result as $key => $item) {
                $takeAlotOrder = TakealotOrders::where([['order_item_id', '=', $item->order_item_id], ['order_id', '=', $item->order_id], ['seller_id', '=', $shop->seller_id], ['takealot_shops_id', '=', $shop->id]])->first();
                if (!empty($takeAlotOrder)) {
                    $dateObject = Carbon::parse($item->order_date);
                    $takeAlotOrder->seller_id               =   $shop->seller_id;
                    $takeAlotOrder->takealot_shops_id       =   $shop->id;
                    $takeAlotOrder->order_item_id           =   $item->order_item_id;
                    $takeAlotOrder->order_id                =   $item->order_id;
                    $takeAlotOrder->order_date              =   $dateObject->format('Y-m-d h:i:s');
                    $takeAlotOrder->sale_status             =   $item->sale_status;
                    $takeAlotOrder->offer_id                =   $item->offer_id;
                    $takeAlotOrder->tsin                    =   $item->tsin;
                    $takeAlotOrder->sku                     =   $item->sku;
                    $takeAlotOrder->product_title           =   $item->product_title;
                    $takeAlotOrder->takealot_url_mobi       =   $item->takealot_url_mobi;
                    $takeAlotOrder->selling_price           =   $item->selling_price;
                    $takeAlotOrder->quantity                =   $item->quantity;
                    $takeAlotOrder->dc                      =   $item->dc;
                    if (isset($item->warehouse) && !empty($item->warehouse)) {
                        $takeAlotOrder->warehouse               =   (isset($item->warehouse) && !empty($item->warehouse)) ? $item->warehouse : ((isset($takeAlotOrder->warehouse) && !empty($takeAlotOrder->warehouse)) ? $takeAlotOrder->warehouse : '');
                    } else {
                        $takeAlotOrder->warehouse  =   (isset($item->dc) && !empty($item->dc)) ? $item->dc : ((isset($takeAlotOrder->dc) && !empty($takeAlotOrder->dc)) ? $takeAlotOrder->dc : '');
                    }
                    $takeAlotOrder->customer                =   $item->customer;
                    $takeAlotOrder->takealot_url            =   $item->takealot_url;
                    $takeAlotOrder->success_fee             =   $item->success_fee;
                    $takeAlotOrder->fulfillment_fee         =   $item->fulfillment_fee;
                    $takeAlotOrder->courier_collection_fee  =   $item->courier_collection_fee;
                    $takeAlotOrder->auto_ibt_fee            =   $item->auto_ibt_fee;
                    $takeAlotOrder->total_fee               =   $item->total_fee;
                    if ($takeAlotOrder->save()) {
                        return true;
                    }
                }
            }
        }
    }
    public function getShopForOrderStatusChange()
    {

        ini_set('max_execution_time', -1);
        $shops = TakealotShops::with('checkShopSubscription')->orderBy('created_at', 'desc')->get();
        if (!empty($shops)) {

            $jobs = [];
            foreach ($shops->chunk(1) as $chunk) {

                foreach ($chunk as $shop) {

                    $job = new TakealotUpdateOrderStatus($shop);
                    $jobs[] = $job;
                    // TakealotUpdateOrderStatus::dispatch($shop);
                    // $this->StatusShopBy($shop);
                }
            }
            if (!empty($jobs)) {
                $batch = Bus::batch($jobs)->name('Get Shop For Order Status Change Batch')->dispatch();
                $jobs = [];
            }
        }
    }
    public function StatusShopBy($shop)
    {
        ini_set('max_execution_time', -1);
        try {
            if (!empty($shop->checkShopSubscription) && !empty($shop->takealot_api_key) && ($shop->checkShopSubscription->status == 'active')) {
                $totalPages = $this->getSalesCount($shop);
                if (!empty($totalPages->page_summary->total)) {
                    $this->updateOrderStatus($shop, 100, 1, $this->remaingCallByLimit);
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }


    public function updateOrderStatus($shop, $page_size, $page_number, $remainingCalls)
    {
        try {
            ini_set('max_execution_time', -1);
            sleep(1);
            $startDate = Carbon::now()->subDays(30)->toDateString();
            $endDate = Carbon::now()->toDateString();
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllOrders($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                    $shopCheck = $this->changeOrderStatus($shop, $result->sales);
                                    if (!empty($shopCheck)) {
                                        $this->updateOrderStatus($shop, $page_size, $page_number + 1, $remainingCalls - 1);
                                    }
                                }
                            }
                        }
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders/filters', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                        $shopCheck = $this->changeOrderStatus($shop, $result->sales);
                                        if (!empty($shopCheck)) {
                                            $this->updateOrderStatus($shop, $page_size, $page_number + 1, $remainingCalls - 1);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function changeOrderStatus($shop, $result)
    {
        ini_set('max_execution_time', -1);
        if (!empty($result) && count($result) > 0) {
            foreach ($result as $key => $item) {
                $takeAlotOrder = TakealotOrders::where([['order_item_id', '=', $item->order_item_id], ['order_id', '=', $item->order_id], ['seller_id', '=', $shop->seller_id], ['takealot_shops_id', '=', $shop->id]])->first();
                if (!empty($takeAlotOrder)) {
                    $dateObject = Carbon::parse($item->order_date);
                    $takeAlotOrder->seller_id               =   $shop->seller_id;
                    $takeAlotOrder->takealot_shops_id       =   $shop->id;
                    $takeAlotOrder->order_item_id           =   $item->order_item_id;
                    $takeAlotOrder->order_id                =   $item->order_id;
                    $takeAlotOrder->order_date              =   $dateObject->format('Y-m-d h:i:s');
                    $takeAlotOrder->sale_status             =   $item->sale_status;
                    $takeAlotOrder->offer_id                =   $item->offer_id;
                    $takeAlotOrder->tsin                    =   $item->tsin;
                    $takeAlotOrder->sku                     =   $item->sku;
                    $takeAlotOrder->product_title           =   $item->product_title;
                    $takeAlotOrder->takealot_url_mobi       =   $item->takealot_url_mobi;
                    $takeAlotOrder->selling_price           =   $item->selling_price;
                    $takeAlotOrder->quantity                =   $item->quantity;
                    $takeAlotOrder->dc                      =   $item->dc;
                    if (isset($item->warehouse) && !empty($item->warehouse)) {
                        $takeAlotOrder->warehouse               =   (isset($item->warehouse) && !empty($item->warehouse)) ? $item->warehouse : ((isset($takeAlotOrder->warehouse) && !empty($takeAlotOrder->warehouse)) ? $takeAlotOrder->warehouse : '');
                    } else {
                        $takeAlotOrder->warehouse  =   (isset($item->dc) && !empty($item->dc)) ? $item->dc : ((isset($takeAlotOrder->dc) && !empty($takeAlotOrder->dc)) ? $takeAlotOrder->dc : '');
                    }
                    $takeAlotOrder->customer                =   $item->customer;
                    $takeAlotOrder->takealot_url            =   $item->takealot_url;
                    $takeAlotOrder->success_fee             =   $item->success_fee;
                    $takeAlotOrder->fulfillment_fee         =   $item->fulfillment_fee;
                    $takeAlotOrder->courier_collection_fee  =   $item->courier_collection_fee;
                    $takeAlotOrder->auto_ibt_fee            =   $item->auto_ibt_fee;
                    $takeAlotOrder->total_fee               =   $item->total_fee;
                    if ($takeAlotOrder->save()) {
                        return true;
                    }
                } else {
                    $this->createOrder($shop, $result);
                }
            }
        }
    }
    //manual fetch code started
    public function manualFetchAllSales($shopID)
    {

        ini_set('max_execution_time', -1);
        $shops = TakealotShops::with('checkShopSubscription')->where('id', '=', $shopID)->orderBy('created_at', 'desc')->first();
        if (!empty($shops)) {
            $this->manualSalesShopBy($shops);
        }
    }
    public function manualSalesShopBy($shop)
    {
        if (!empty($shop->checkShopSubscription) && !empty($shop->takealot_api_key) && ($shop->checkShopSubscription->status == 'active')) {
            $totalPages = $this->getSalesCount($shop);
            if (!empty($totalPages) && !empty($totalPages->page_summary->total)) {
                $this->manualGetAllOrders($shop, 100, 1, $this->remaingCallByLimit, $totalPages->page_summary->total);
            }
        }
    }
    public function manualGetAllOrders($shop, $page_size, $page_number, $remainingCalls, $totalPages)
    {

        ini_set('max_execution_time', -1);

        DB::transaction(function () use ($shop, $page_size, $page_number, $remainingCalls, $totalPages) {


            if (!empty($shop->takealot_api_key) && $totalPages > 0) {

                $this->manualOrdersCurl($shop, $page_size, $page_number, $remainingCalls, $totalPages);
            }
        });
    }

    public function manualOrdersCurl($shop, $page_size, $page_number, $remainingCalls, $totalPages)
    {

        try {
            ini_set('max_execution_time', -1);
            sleep(1);
            $startDate = Carbon::now()->subDays(30)->toDateString();
            $endDate = Carbon::now()->toDateString();
            $ratelimit = shopRateLimitChecker($shop, $this->rateLimit);
            if ($ratelimit > 0) {

                $apiCall = apiProxyCall();
                if (!empty($apiCall) && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
                    if ($apiCall->api_url == env('APP_URL')) {
                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $offerCont = new OrdersApiController();
                        $result = $offerCont->fetchAllOrders($allData);
                        if (!empty($result) && !empty($shop)) {
                            $resultDecoded = json_decode($result->getContent(), true);
                            if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                $result  = json_decode($resultDecoded['data']);
                                if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {

                                    $shopdata = $this->manualOrderJobCreater($shop, $result->sales);
                                    if (!empty($shopdata)) {

                                        $this->manualGetAllOrders($shopdata, $page_size, $page_number + 1, $remainingCalls - 1, $result->page_summary->total);
                                    }
                                }
                            }
                        }
                    } else {

                        $allData = [
                            'takealot_api_key' => $shop->takealot_api_key,
                            'x-RateLimit-Limit' => $this->rateLimit,
                            'x-RateLimit-Remaining' => $ratelimit,
                            'base_url' => takealotBaseUrl(),
                            'page_number' => $page_number,
                            'page_size' => $page_size,
                            'startDate' => $startDate,
                            'endDate' => $endDate,
                        ];

                        $postData = json_encode($allData);
                        $client   = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
                        $response = $client->post('/api/orders/filters', [
                            'body' => $postData,
                            'headers' => [
                                'Content-Type' => 'application/json', // Set the content type to JSON
                            ]
                        ]);
                        if ($response->getStatusCode() == 200) {
                            $result = $response->getBody()->getContents();
                            if (!empty($result) && !empty($shop)) {
                                $resultDecoded  = json_decode($result, true);
                                if (!empty($resultDecoded) && isset($resultDecoded['data']) && !empty($resultDecoded['data'])) {
                                    $result  = json_decode($resultDecoded['data']);
                                    if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                        if (!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total > 0)) {
                                            $shopdata = $this->manualOrderJobCreater($shop, $result->sales);
                                            if (!empty($shopdata)) {

                                                $this->manualGetAllOrders($shopdata, $page_size, $page_number + 1, $remainingCalls - 1, $result->page_summary->total);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            Logger($e);
        }
    }
    public function manualOrderJobCreater($shop, $result)
    {
        ini_set('max_execution_time', -1);
        if (!empty($result) && !empty($result) && !empty($shop)) {
            $this->createOrder($shop, $result);
            $shop = TakealotShops::with('checkShopSubscription')->where('id', '=', $shop->id)->first();
            return $shop;
        }
    }
}

 // public function getSalesCount($shop,$timePeriod=null)
    // {
    //     // ini_set('max_execution_time',-1);
    //     // ini_set('max_execution_time',-1);        
    //     sleep(1);
    //     $ipRateLimit=IpRateLimitChecker();
    //     if($ipRateLimit>0)
    //     {
    //             $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
    //             if($ratelimit>0)
    //             {
    //                 if(!empty($timePeriod))
    //                 {
    //                     if($timePeriod==6)
    //                     {
    //                         $startDate=Carbon::now()->subDays(180)->toDateString();
    //                     }
    //                     elseif($timePeriod==1)
    //                     {
    //                         $startDate=Carbon::now()->subDays(1)->toDateString();
    //                     }
    //                     else
    //                     {
    //                         $startDate=Carbon::now()->subDays(2)->toDateString();
    //                     } 
    //                     $endDate = Carbon::now()->toDateString();  
    //                     // $headers = ['Authorization' => 'Key '.$apiKey];
    //                     $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];

    //                     $client = new Client(['handler' => $this->handlerStack,'base_uri' =>  takealotBaseUrl(),'verify' => false]);
    //                     $request = new Request('GET', 'sales?filters=start_date:'.$startDate.';end_date:'.$endDate.'',$headers);
    //                     $response = $client->send($request);
    //                     return json_decode($response->getBody()->getContents());
    //                 } 
    //                 else
    //                 {
    //                     // $headers = ['Authorization' => 'Key '.$apiKey];
    //                     $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //                     $client = new Client(['handler' => $this->handlerStack,'base_uri' =>  takealotBaseUrl(),'verify' => false]);
    //                     $request = new Request('GET', 'sales',$headers);
    //                     $response = $client->send($request);
    //                     return json_decode($response->getBody()->getContents());
    //                 }

    //             }
    //     }

    // }
   // public function updateSingleOrder($shops,$tsin,$page_size=100,$page_number=1)
    // {    
    //      ini_set('max_execution_time',-1);        
    //      sleep(1);
    //      $ipRateLimit=IpRateLimitChecker();
    //      if($ipRateLimit>0)
    //      {
    //         $ratelimit=shopRateLimitChecker($shops,$this->rateLimit);
    //         if($ratelimit>0)
    //         {   
    //                     //  ini_set('max_execution_time',-1);
    //                     // $remainingCalls=$this->remaingCallByLimit;
    //                     // $ratelimit=shopRateLimitChecker($shops,$this->rateLimit);
    //                     $startDate=Carbon::now()->subDays(2)->toDateString();
    //                     $endDate=Carbon::now()->toDateString();            
    //                     if(!empty($shops))
    //                     {
    //                         $headers = ['Authorization'=>'Key '.$shops->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //                         $client  = new Client(['handler' => $this->handlerStack,'base_uri' => takealotBaseUrl(),'verify' => false]);
    //                         $request = new Request('GET', 'sales?filters=tsin:'.$tsin.';start_date:'.$startDate.';end_date:'.$endDate.'&page_number='.$page_number.'&page_size='.$page_size,$headers);

    //                         $promise = $client->sendAsync($request)->then(function ($response) use ($shops,$tsin,$page_size,$page_number,$remainingCalls) {
    //                         $result  = json_decode($response->getBody()->getContents());
    //                         if(!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total>0))
    //                         {                
    //                             $shop=$this->orderUpdate($shops,$result->sales);    
    //                             if(!empty($shop))
    //                             {
    //                                 $this->updateSingleOrder($shops,$tsin,$page_size,$page_number+1);
    //                             }        

    //                         }


    //                         });

    //                         $promise->wait();      

    //                 }
    //                 else
    //                 {
    //                     return false;
    //                 }
    //         }
    //     }
    // }
    // public function updateOrderStatus($shop,$page_size,$page_number,$remainingCalls)     
    // {        
    //     ini_set('max_execution_time',-1);        
    //     sleep(1);
    //     $ipRateLimit=IpRateLimitChecker();
    //     if($ipRateLimit>0)
    //     {
    //         $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
    //        if($ratelimit>0)
    //        { 
    //             // ini_set('max_execution_time',-1);        
    //             // $remainingCalls=$this->remaingCallByLimit;
    //             // $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
    //             $startDate=Carbon::now()->subDays(30)->toDateString();       
    //             $endDate = Carbon::now()->toDateString();
    //             $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //             $client  = new Client(['handler' => $this->handlerStack,'base_uri' => takealotBaseUrl(),'verify' => false]);
    //             $request = new Request('GET', 'sales?filters=start_date:'.$startDate.';end_date:'.$endDate.'&page_number='.$page_number.'&page_size='.$page_size.'',$headers);
    //             $promise = $client->sendAsync($request)->then(function ($response) use ($shop,$page_size,$page_number,$remainingCalls) {
    //             $result  = json_decode($response->getBody()->getContents());
    //             if(!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total>0))
    //             {             
    //                 $shopCheck=$this->changeOrderStatus($shop,$result->sales);
    //                 if(!empty($shopCheck))     
    //                 {                 
    //                     $this->updateOrderStatus($shop,$page_size,$page_number+1,$remainingCalls-1);                
    //                 }                  
    //             } 
    //             });

    //             $promise->wait();
    //         }
    //     }
    //} 
    // public function manualOrdersCurl($shop,$page_size,$page_number,$remainingCalls,$totalPages)
    // {
    //     ini_set('max_execution_time',-1);        
    //     sleep(1);
    //     $ipRateLimit=IpRateLimitChecker();
    //     if($ipRateLimit>0)
    //     {
    //         $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
    //        if($ratelimit>0)
    //        { 
    //             // ini_set('max_execution_time',-1);
    //             $startDate=Carbon::now()->subDays(3)->toDateString();        
    //             $endDate = Carbon::now()->toDateString();
    //             $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //             $client  = new Client(['handler' => $this->handlerStack,'base_uri' => takealotBaseUrl(),'verify' => false]);
    //             $request = new Request('GET', 'sales?filters=start_date:'.$startDate.';end_date:'.$endDate.'&page_number='.$page_number.'&page_size='.$page_size.'',$headers);
    //             $promise = $client->sendAsync($request)->then(function ($response) use ($shop,$page_size,$page_number,$remainingCalls,$totalPages) {
    //             $result  = json_decode($response->getBody()->getContents());
    //             if(!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total>0))
    //             {

    //                 $shopdata=$this->manualOrderJobCreater($shop,$result->sales);
    //                 if(!empty($shopdata))
    //                 {

    //                     $this->manualGetAllOrders($shopdata,$page_size,$page_number+1,$remainingCalls-1,$result->page_summary->total);

    //                 }

    //             }


    //             });

    //             $promise->wait();

    //         }
    //     }

    // } 


    // public function ordersCurl($shop,$page_size,$page_number,$remainingCalls,$totalPages,$timePeriod,$salesBatch)
    // {
    //     ini_set('max_execution_time',-1);
    //     ini_set('xdebug.max_nesting_level', -1);  
    //     if($timePeriod==6)
    //     {
    //         $startDate=Carbon::now()->subDays(180)->toDateString();
    //     }
    //     elseif($timePeriod==1)
    //     {
    //         $startDate=Carbon::now()->subDays(1)->toDateString();
    //     }
    //     else
    //     {
    //         $startDate=Carbon::now()->subDays(2)->toDateString();
    //     }   

    //     try
    //     {
    //         // ini_set('max_execution_time',-1);        
    //         sleep(1);
    //         $ipRateLimit=IpRateLimitChecker();
    //         if($ipRateLimit>0)
    //         {
    //                 $ratelimit=shopRateLimitChecker($shop,$this->rateLimit);
    //                 if($ratelimit>0)
    //                 {
    //                         $endDate = Carbon::now()->toDateString();
    //                         $headers = ['Authorization'=>'Key '.$shop->takealot_api_key,'x-RateLimit-Limit'=>$this->rateLimit,'x-RateLimit-Remaining'=>$ratelimit];
    //                         $client  = new Client(['handler' => $this->handlerStack,'base_uri' => takealotBaseUrl(),'verify' => false]);
    //                         $request = new Request('GET', 'sales?filters=start_date:'.$startDate.';end_date:'.$endDate.'&page_number='.$page_number.'&page_size='.$page_size.'',$headers);
    //                         $promise = $client->sendAsync($request)->then(function ($response) use ($shop,$page_size,$page_number,$ratelimit,$totalPages,$timePeriod,$salesBatch) {
    //                         $result  = json_decode($response->getBody()->getContents());
    //                         if(!empty($result) && !empty($result->sales) && (!empty($result->page_summary->total) && $result->page_summary->total>0))
    //                         {

    //                             $job = new TakealotCreateSalesJob($shop,$result->sales);
    //                             $this->createOrdersBatch[]=$job;  
    //                             if(!empty($this->createOrdersBatch) && count($this->createOrdersBatch)>=5)
    //                             {          
    //                                 $batch = Bus::batch($this->createOrdersBatch)->name($shop->takealot_account_name.' Fetch Sales in chunk Job Batch')->onQueue('medium')->dispatch();
    //                                 $this->createOrdersBatch=[];
    //                             }  
    //                             $this->ordersCurl($shop,$page_size,$page_number+1,$ratelimit,$result->page_summary->total,$timePeriod,null);


    //                         }


    //                         });

    //                         $promise->wait();
    //                 }
    //             }
    //     }
    //     catch (\Exception $e) 
    //     {

    //     }
    // }