<?php

namespace App\Http\Controllers\Seller;

use App\Http\Controllers\Controller;
use App\Models\TakealotRePricing;
use App\Models\TakealotPlidHistory;
use App\Models\TakealotCompetitionHistory;
use App\Models\TakealotOffers;
use App\Models\PackageBuy;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use App\Jobs\TakealotRepricingJob;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use App\Http\Controllers\Api\ScrapingApiController;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\Seller\OffersController;
use App\Http\Controllers\Seller\SettingsController;
use Illuminate\Bus\Batch;
use Illuminate\Support\Facades\Bus;
use GuzzleHttp\HandlerStack;
use Spatie\GuzzleRateLimiterMiddleware\RateLimiterMiddleware;

class RePricingController extends Controller
{
  public $rateLimit = 0;
  public $handlerStack;
  public function __construct()
  {
    $this->rateLimit = get_setting('takealot_ratelimit');
    $scrapRateLimit = get_setting('takealot_scraplimit');
    $this->handlerStack = HandlerStack::create();
    $this->handlerStack->push(RateLimiterMiddleware::perMinute($scrapRateLimit));
  }
  public function index()
  {

    return view('seller.re-pricing.index');
  }
  public function checkLosserOffers()
  {
    $losserTime = get_setting('losser_rescrap_time');
    $thresholdTimestamp = Carbon::now()->subMinutes($losserTime);
    $rePricing = TakealotRePricing::select('takealot_repricing.*')
      ->with('offer', 'offer.takealotOfferDetails', 'takealotShopDetails')
      ->join('takealot_offers', function ($query) {

        $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
          ->where([['takealot_offers.status', '=', 'Buyable']])
          ->orderBy('takealot_repricing.price_updated_at', 'asc');
      })
      ->where([['takealot_repricing.winner', '=', 0], ['takealot_repricing.status', '=', 1], ['takealot_repricing.min_price', '>', 0], ['takealot_repricing.max_price', '>', 0]])
      ->where('takealot_repricing.price_updated_at', '<', $thresholdTimestamp)
      //  ->whereRaw('takealot_repricing.price_updated_at < ?', [$thresholdTimestamp])
      ->orderBy('takealot_repricing.price_updated_at', 'asc')
      ->chunk(5, function ($chunk, $index) {
        $jobName = 'repricingLosserOffers' . $index;
        TakealotRepricingJob::dispatch($chunk, $jobName)->onQueue('high');
        // $this->rePricingCurl($chunk);
      });

    echo "Start Checking losser offers...";
  }
  public function checkWinner()
  {
    $winnerTime = get_setting('winner_rescrap_time');
    $thresholdTimestamp = Carbon::now()->subMinutes($winnerTime);
    $rePricing = TakealotRePricing::select('takealot_repricing.*')->with('offer', 'offer.takealotOfferDetails', 'takealotShopDetails')
      ->join('takealot_offers', function ($query) {

        $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
          ->where([['takealot_offers.status', '=', 'Buyable']]);
      })
      ->where([['takealot_repricing.winner', '=', 1], ['takealot_repricing.status', '=', 1], ['takealot_repricing.min_price', '>', 0], ['takealot_repricing.max_price', '>', 0]])
      ->where('takealot_repricing.price_updated_at', '<', $thresholdTimestamp)
      ->orderBy('takealot_repricing.price_updated_at', 'asc')
      ->chunk(5, function ($chunk, $index) {
        $jobName = 'repricing' . $index;
        TakealotRepricingJob::dispatch($chunk, $jobName)->onQueue('medium');
        // $this->rePricingCurl($chunk); 
      });

    echo "Start Checking Winner offers...";
  }
  public function checkMoreThanTenMinutes()
  {
    $winnerTime = get_setting('long_check_rescrap_time');
    $thresholdTimestamp = Carbon::now()->subMinutes($winnerTime);
    $rePricing = TakealotRePricing::select('takealot_repricing.*')->with('offer', 'offer.takealotOfferDetails', 'takealotShopDetails')
      ->join('takealot_offers', function ($query) {
        $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
          ->where([['takealot_offers.status', '=', 'Buyable']]);
      })
      ->where([['takealot_repricing.status', '=', 1], ['takealot_repricing.min_price', '>', 0], ['takealot_repricing.max_price', '>', 0]])
      ->where('takealot_repricing.price_updated_at', '<', $thresholdTimestamp)
      ->orderBy('takealot_repricing.price_updated_at', 'asc')
      ->chunk(5, function ($chunk, $index) {
        $jobName = 'checkMoreThanTenMinutes' . $index;
        TakealotRepricingJob::dispatch($chunk, $jobName)->onQueue('high');
        // $this->rePricingCurl($chunk); 
      });

    echo "Start Checking offers over the 10 minutes...";
  }
  public function rePricingCurl($chunk)
  {

    // ini_set('max_execution_time', 0);
    foreach ($chunk as $key => $reprice) {

      $updatedAt = Carbon::parse($reprice->price_updated_at);
      $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
      if ($minutesAgo > 5) {
        // Logger('Tsin ID = '.$reprice->offer->tsin_id.' Ago= '.$minutesAgo);
        //winner rechecked greater after every two 2 mints  
        // $recheckedtAt = Carbon::parse($reprice->updated_at)->timezone('Africa/Johannesburg');
        // $recheckedtAtAgo = $recheckedtAt->diffInMinutes(Carbon::now('Africa/Johannesburg'));

        // if(($minutesAgo>=10 && $recheckedtAtAgo>=2) && $reprice->winner==1)
        if (($minutesAgo >= 7) && $reprice->winner == 1) {
          $this->instantRepricing($reprice);
        }
        // if(($minutesAgo>=2 && $recheckedtAtAgo>=2) && $reprice->winner==0)
        if (($minutesAgo >= 5) && $reprice->winner == 0) {
          $this->instantRepricing($reprice);
        }
      }
    }
  }
  public function instantRepricing($rePriceData, $type = null)
  {
    if (!empty($rePriceData)) {

      $updatedAt = Carbon::parse($rePriceData->price_updated_at);
      $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
      //winner rechecked greater than 2 mints  
      // $recheckedtAt = Carbon::parse($rePriceData->updated_at)->timezone('Africa/Johannesburg');
      // $recheckedtAtAgo = $recheckedtAt->diffInMinutes(Carbon::now('Africa/Johannesburg'));

      if (empty($type) && $type == null && $minutesAgo >= 2) {
        // if(($minutesAgo>=6 && $recheckedtAtAgo>=2) && $rePriceData->winner==1)
        if (($minutesAgo >= 6) && $rePriceData->winner == 1) {
          $this->instantRepriceTime($rePriceData);
        }
        if (($minutesAgo >= 2) && $rePriceData->winner == 0) {
          $this->instantRepriceTime($rePriceData);
        }
      }
      // else
      // {               
      //   $this->instantRepriceTime($rePriceData);   
      // }


    }
  }
  public function instantRepriceTime($rePriceData)
  {
    // ini_set('max_execution_time', -1);                         
    if (!empty($rePriceData)) {
      $result = $this->guzzelCurl($rePriceData);

      if ($rePriceData->offer->is_variant == 0 && empty($rePriceData->offer->variant_params) && !empty($result) && isset($result->variants) && !empty($result->variants) && isset($result->variants->selectors) && !empty($result->variants->selectors) && count($result->variants->selectors) > 0) {
        $getVariantParams = $this->variantOfferUrl($rePriceData->offer->title, $rePriceData->offer->offer_url, $result);
        if (!empty($getVariantParams)) {
          $rePriceData->offer->is_variant = 1;
          $rePriceData->offer->variant_params = $getVariantParams;
          if ($rePriceData->offer->save()) {
            $this->repriceVariantParams($rePriceData->id);
          }
        }
      }
      if (!empty($result)) {
        $this->updateOffer($result, $rePriceData);
      }
    }
  }
  public function repriceVariantParams($id, $type = null)
  {
    $repriceData = TakealotRePricing::with('offer', 'offer.takealotOfferDetails')->where([['id', '=', $id]])->first();
    $this->instantRepricing($repriceData, $type);
  }
  public function updateOffer($data, $rePricing)
  {


    DB::transaction(function () use ($data, $rePricing) {
      try {
        $offerCurrentStatus = $rePricing->winner;
        $winner = $rePricing->winner;
        if (!empty($data->data_layer->totalPrice) && !empty($data->seller_detail->seller_id)) {
          if ($data->data_layer->totalPrice >= $rePricing->min_price) {
            $sellerCount = 0;
            $sellerCount = (!empty($data->other_offers->conditions[0]->items)) ? count($data->other_offers->conditions[0]->items) : 0;
            //if there is no compititor on takealot against this product than our offer will be reached at max price
            if (($sellerCount == 0 && empty($data->other_offers) && empty($data->other_offers->conditions[0])) && (!empty($data->seller_detail->seller_id) && trim($data->seller_detail->seller_id) == trim($rePricing->offer->takealotShops->tk_seller_id))) {
              $winner = 1;
              if ($rePricing->offer->selling_price != $rePricing->max_price && $rePricing->status == 1 && $rePricing->offer->selling_price != $rePricing->max_price) {
                $this->updateLiveOfferPrice($rePricing, $rePricing->max_price);
              }
              // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);

            } else {


              $winner = 0;
              // $rePricing=$this->updateRepricingStatus($rePricing,$data, $winner); 
              //check other sellers are selling or not this product if not selling so we are the winner
              if ($sellerCount == 0 && trim($rePricing->offer->takealotShops->tk_seller_id) == trim($data->seller_detail->seller_id)) {
                $winner = 1;
              }

              //check winner account is our account 
              if (!empty($rePricing->offer->takealotShops->tk_seller_id) && trim($rePricing->offer->takealotShops->tk_seller_id) == trim($data->seller_detail->seller_id)) {
                $winner = 1;
              }

              //checking our selling price is greater than min or max price 
              if ($rePricing->offer->selling_price < $rePricing->min_price) {
                $winner = 0;
              }
              if ($rePricing->offer->selling_price > $rePricing->max_price) {
                $winner = 0;
              }
              //end checking
              //if our offer is  winner
              if ($winner == 1) {

                $checkPlid = TakealotPlidHistory::where('plid', '=', $rePricing->offer->plid)->latest()->first();
                if (!empty($checkPlid)) {
                  $checkPlid->status = 'winner';
                  $checkPlid->save();
                }
                $checkOffer = TakealotCompetitionHistory::where('tsin', '=', $rePricing->offer->tsin_id)->latest()->first();
                if (!empty($checkOffer)) {
                  $checkOffer->status = 'winner';
                  $checkOffer->last_status = 'winner';
                  $checkOffer->save();
                }
                $rePriceData = $this->offerPriceUp($data, $rePricing);
                if (!empty($rePriceData)) {
                  $rePricing = $rePriceData;
                }

                // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);

              }
              //end winner check color                        

              //if our offer is losser
              if ($winner == 0) {

                $takealotStockTotal = 0;
                $leadTimeStockTotal = 0;
                //stock code started
                if (!empty($rePricing->offer->leadtime_stock) && isset($rePricing->offer->leadtime_stock)) {
                  $leadtime_decode = json_decode($rePricing->offer->leadtime_stock);
                  $leadTimeStockTotal = 0;
                  if (!empty($leadtime_decode) && count($leadtime_decode) > 0) {
                    foreach ($leadtime_decode as $key => $item) {
                      $qty = (!empty($item->quantity_available) && isset($item->quantity_available) && $item->quantity_available > 0) ? $item->quantity_available : 0;
                      $leadTimeStockTotal = $leadTimeStockTotal + $qty;
                    }
                  }
                }
                if (!empty($rePricing->offer->stock_at_takealot) && isset($rePricing->offer->stock_at_takealot)) {
                  $takealotStock_decode = json_decode($rePricing->offer->stock_at_takealot);
                  if (!empty($takealotStock_decode) && count($takealotStock_decode) > 0) {
                    foreach ($takealotStock_decode as $key => $item) {
                      $qty = (!empty($item->quantity_available) && isset($item->quantity_available)) ? $item->quantity_available : 0;
                      $takealotStockTotal = $takealotStockTotal + $qty;
                    }
                  }
                }
                //stock code ended
                if (($data->data_layer->totalPrice >= $rePricing->min_price && $data->data_layer->totalPrice <= $rePricing->max_price)) {

                  if (!empty($rePricing->takealotShopDetails->takealot_api_key)) {

                    //start our product leadtime/instock and compititor product on leadtime/instock
                    if (($rePricing->offer->selling_price != $data->data_layer->totalPrice) && ($data->stock_availability->is_leadtime == true && $takealotStockTotal <= 0 && $leadTimeStockTotal > 0) || ($data->stock_availability->is_leadtime == false && $takealotStockTotal > 0)) {
                      $rePrice = $data->data_layer->totalPrice - 1;
                      if (($rePricing->offer->selling_price != $rePrice) && $rePrice <= $rePricing->min_price) {
                        $rePrice = $rePricing->min_price;
                      }
                      if (($rePricing->offer->selling_price != $rePrice) && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price) && $rePricing->status == 1 && $rePricing->offer->status == 'Buyable') {
                        $this->updateLiveOfferPrice($rePricing, $rePrice);
                      }
                    }
                    //end our product leadtime/instock and compititor product on leadtime/instock
                    if (!empty($rePricing->min_price) && !empty($rePricing->max_price) && $rePricing->status == 1  && $rePricing->offer->status == 'Buyable' && $takealotStockTotal > 0) {
                      $updatedAt = Carbon::parse($rePricing->price_updated_at);
                      $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
                      //start our product instock and compititor product on leadtime
                      if ($minutesAgo >= 8) {
                        $this->instockVsLeadtime($data, $rePricing, $takealotStockTotal);
                      }
                      //end our product instock and compititor product on leadtime
                    }

                    if (!empty($rePricing->min_price) && !empty($rePricing->max_price)  && $rePricing->status == 1  && $rePricing->offer->status == 'Buyable' && $takealotStockTotal <= 0 && $leadTimeStockTotal > 0) {
                      $updatedAt = Carbon::parse($rePricing->price_updated_at);
                      $minutesAgo = $updatedAt->diffInMinutes(Carbon::now());
                      //start our product leadtime and compititor product on instock
                      if ($minutesAgo >= 8) {
                        $this->leadtimeVsInstock($data, $rePricing, $leadTimeStockTotal);
                      }
                      //end our product leadtime and compititor product on instock
                    }
                  }
                } else {

                  if (($rePricing->offer->selling_price != $data->data_layer->totalPrice) && $rePricing->status == 1  && $rePricing->offer->status == 'Buyable') {
                    if ($data->data_layer->totalPrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $data->data_layer->totalPrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                      if (($rePricing->offer->selling_price != $rePricing->max_price) && ($data->data_layer->totalPrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price)) {
                        $this->updateLiveOfferPrice($rePricing, $rePricing->max_price);
                      }
                      if (($rePricing->offer->selling_price != $rePricing->min_price) && ($data->data_layer->totalPrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price)) {
                        $this->updateLiveOfferPrice($rePricing, $rePricing->min_price);
                      }
                    } else {
                      if ($data->data_layer->totalPrice == $rePricing->min_price && (($data->stock_availability->is_leadtime == false  && $takealotStockTotal > 0) || ($data->stock_availability->is_leadtime == true && $takealotStockTotal <= 0 && $leadTimeStockTotal > 0))) {
                        $newPrice = $rePricing->min_price;
                      } elseif ($data->data_layer->totalPrice == $rePricing->min_price) {
                        $newPrice = $rePricing->min_price + 1;
                      } else {
                        $newPrice = $rePricing->min_price;
                      }

                      if ($rePricing->offer->selling_price != $newPrice) {
                        $this->updateLiveOfferPrice($rePricing, $newPrice);
                      }
                    }
                  }
                }
              }
              //end losser code
              // $rePricing=$this->updateRepricingStatus($rePricing,$data,$winner);
            }


            if (!empty($rePricing->offer->takealotOfferDetails)) {
              $rePricing = $this->updateTakealotOfferDetails($rePricing, $data, $sellerCount);
            }


            if ($winner != $offerCurrentStatus) {

              $activityLog = array(
                'repricing_status' => array(
                  'seller_id' => $rePricing->offer->seller_id,
                  'takealot_shops_id' => $rePricing->offer->takealot_shops_id,
                  'tsin' => $rePricing->offer->tsin_id,
                  'new_status' => ($winner == 1) ? 'Winner' : 'Losser',
                  'old_status' => ($offerCurrentStatus == 1) ? 'Winner' : 'Losser',
                  'ip' => getUserIp(),
                  'updated_through' => 'repricing'
                )
              );
              $updateHistoyByTsin = new OffersController();
              $updateHistoyByTsin->tsinUpdateHistoryWithObject($activityLog, 'Re-Pricing');
              // $rePricing=$this->updateRepricingStatus($rePricing,$data,$rePricing->winner);
            }
            // $rePricing=$this->updateRepricingStatus($rePricing,$data,$rePricing->winner);
          }
          //  else
          //  {
          //    $rePricing=$this->updateRepricingStatus($rePricing,$data,$rePricing->winner);
          //  }
          $this->updateRepricingStatus($rePricing, $data, $winner);
        }
      } catch (\Exception $e) {
        // return true;
      }
    });
  }
  public function updateRepricingStatus($rePricing, $data, $winner)
  {
    // $loggerdata=$rePricing->offer->tsin_id.'------'.$rePricing->offer->takealot_shops_id;
    // Logger($loggerdata);
    $stockStatus = (!empty($data->stock_availability) && !empty($data->stock_availability->status)) ? $data->stock_availability->status : '';
    $rePricing->stock_status = $stockStatus;
    $rePricing->leadtime_status = (!empty($data->stock_availability->is_leadtime) && $data->stock_availability->is_leadtime == true) ? 1 : 0;
    $rePricing->winning_price = $data->data_layer->totalPrice;
    $rePricing->winner_account_name = trim($data->seller_detail->display_name);
    $rePricing->winner_seller_id = trim($data->seller_detail->seller_id);
    $rePricing->winner = $winner;
    $rePricing->price_updated_at = Carbon::now();
    $rePricing->sellers = (!empty($data->other_offers->conditions[0]->items)) ? count($data->other_offers->conditions[0]->items) : 0;
    $rePricing->save();
    return $rePricing;
  }
  public function updateTakealotOfferDetails($rePricing, $data, $sellerCount)
  {
    $rePricing->offer->takealotOfferDetails->star_rating = !empty($data->core->star_rating) ? $data->core->star_rating : 0;
    $rePricing->offer->takealotOfferDetails->total_sellers = $sellerCount;
    $rePricing->offer->takealotOfferDetails->save();
    return $rePricing;
  }
  public function instockVsLeadtime($data, $rePricing, $takealotStockTotal)
  {

    if ($data->stock_availability->is_leadtime == true && $takealotStockTotal > 0) {

      $checkPlid = TakealotPlidHistory::where([['plid', '=', $rePricing->offer->plid], ['stock_type', '=', 'instockVsLeadtime']])->latest()->first();

      if (!empty($checkPlid)) {
        if ($checkPlid->status == 'winner') {
          $checkPlid->status = 'losser';
          $checkPlid->save();
          $getNewPercentage = ($data->data_layer->totalPrice * $checkPlid->percentage) / 100;
          $finalPrice = $data->data_layer->totalPrice + (floor($getNewPercentage));
          if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
            if ($rePricing->offer->selling_price != $finalPrice) {
              $this->competitonHistory($data, $rePricing, $checkPlid->id, $checkPlid->percentage, $finalPrice);
              $this->updateLiveOfferPrice($rePricing, $finalPrice);
            }
          }
        } else {

          $createPlidHistory = $checkPlid;
          if ($checkPlid->percentage >= 1) {
            $subTwonumbers = $data->data_layer->totalPrice - $rePricing->offer->selling_price;
            $addTwonumbers = ($data->data_layer->totalPrice + $rePricing->offer->selling_price) / 2;
            $percentageDifference = ($subTwonumbers / $addTwonumbers) * 100;
            $percentageDifference = abs($percentageDifference);
            if ($percentageDifference >= 33) {
              $instockPercentage = 32;
            } else {
              $instockPercentage = $percentageDifference - ($rePricing->offer->takealotShops->leadtime_percentage);
            }

            if ($instockPercentage < 0) {
              $instockPercentage = 0;
            }

            $getNewPercentage = $data->data_layer->totalPrice * $instockPercentage / 100;
            $finalPrice = $data->data_layer->totalPrice + (floor($getNewPercentage));
            if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $finalPrice = $finalPrice;
            } else {
              $instockPercentage = $percentageDifference;
              $finalPrice = $rePricing->min_price;
            }

            if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $createPlidHistory->plid = $rePricing->offer->plid;
              $createPlidHistory->percentage = $instockPercentage;
              $createPlidHistory->status = 'losser';
              if ($createPlidHistory->save()) {

                if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
                  $this->competitonHistory($data, $rePricing, $createPlidHistory->id, $createPlidHistory->percentage, $finalPrice);
                  $this->updateLiveOfferPrice($rePricing, $finalPrice);
                }
              }
            }
          }
        }
      } else {
        $subTwonumbers = $data->data_layer->totalPrice - $rePricing->offer->selling_price;
        $addTwonumbers = ($data->data_layer->totalPrice + $rePricing->offer->selling_price) / 2;
        $percentageDifference = ($subTwonumbers / $addTwonumbers) * 100;
        $percentageDifference = abs($percentageDifference);
        if ($percentageDifference >= 33) {
          $instockPercentage = 32;
        } else {
          $instockPercentage = $percentageDifference - ($rePricing->offer->takealotShops->leadtime_percentage);
        }

        if ($instockPercentage < 0) {
          $instockPercentage = 0;
        }

        $getNewPercentage = $data->data_layer->totalPrice * $instockPercentage / 100;
        $finalPrice = $data->data_layer->totalPrice + (floor($getNewPercentage));
        if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
          $finalPrice = $finalPrice;
        } else {
          $instockPercentage = $instockPercentage;
          $finalPrice = $rePricing->min_price;
        }

        if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
          $checkPlid = new TakealotPlidHistory();
          $checkPlid->plid = $rePricing->offer->plid;
          $checkPlid->percentage = $instockPercentage;
          $checkPlid->status = 'losser';
          $checkPlid->stock_type = 'instockVsLeadtime';
          if ($checkPlid->save()) {
            if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $this->competitonHistory($data, $rePricing, $checkPlid->id, $instockPercentage, $finalPrice);
              $this->updateLiveOfferPrice($rePricing, $finalPrice);
            }
          }
        }
      }
    }
  }
  public function leadtimeVsInstock($data, $rePricing, $leadTimeStockTotal)
  {
    if ($data->stock_availability->is_leadtime == false && $leadTimeStockTotal > 0) {

      $checkPlid = TakealotPlidHistory::where([['plid', '=', $rePricing->offer->plid], ['stock_type', '=', 'leadtimeVsinstock']])->latest()->first();
      if (!empty($checkPlid)) {

        if ($checkPlid->status == 'winner') {

          $checkPlid->status = 'losser';
          $checkPlid->save();
          $percentageDifference = ($data->data_layer->totalPrice * $checkPlid->percentage) / 100;
          $finalPrice = $data->data_layer->totalPrice - (floor($percentageDifference));
          if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
            $this->competitonHistory($data, $rePricing, $checkPlid->id, $checkPlid->percentage, $finalPrice);
            $this->updateLiveOfferPrice($rePricing, $finalPrice);
          }
        } else {
          $createPlidHistory = $checkPlid;
          if ($createPlidHistory->percentage <= 34.5) {

            $newPercentage = $checkPlid->percentage + $rePricing->offer->takealotShops->leadtime_percentage;
            if ($newPercentage > 34.5) {
              $newPercentage = 33.1;
            }

            $percentageDifference = ($data->data_layer->totalPrice * $newPercentage) / 100;
            $finalPrice = $data->data_layer->totalPrice - (floor($percentageDifference));

            if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $finalPrice = $finalPrice;
            } else {
              $newPercentage = $checkPlid->percentage;
              $finalPrice = $rePricing->min_price;
            }

            if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $createPlidHistory->plid = $rePricing->offer->plid;
              $createPlidHistory->percentage = $newPercentage;
              $createPlidHistory->status = 'losser';
              if ($createPlidHistory->save()) {
                $this->competitonHistory($data, $rePricing, $checkPlid->id, $newPercentage, $finalPrice);
                $this->updateLiveOfferPrice($rePricing, $finalPrice);
              }
            }
          }
        }
      } else {

        $percentageDifference = ($data->data_layer->totalPrice * 10) / 100;
        $finalPrice = $data->data_layer->totalPrice - (floor($percentageDifference));
        if (($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
          $finalPrice = $finalPrice;
        } else {
          $finalPrice = $rePricing->min_price;
        }


        if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
          $checkPlid = new TakealotPlidHistory();
          $checkPlid->plid = $rePricing->offer->plid;
          $checkPlid->percentage = 10;
          $checkPlid->status = 'losser';
          $checkPlid->stock_type = 'leadtimeVsinstock';
          if ($checkPlid->save()) {
            if ($rePricing->offer->selling_price != $finalPrice && ($finalPrice >= $rePricing->min_price && $finalPrice <= $rePricing->max_price)) {
              $this->competitonHistory($data, $rePricing, $checkPlid->id, 10, $finalPrice);
              $this->updateLiveOfferPrice($rePricing, $finalPrice);
            }
          }
        }
      }
    } else {
      $updateOfferSoh = new OffersController();
      $updateOfferSoh->updateTakealotOfferSoh($rePricing->offer->offer_id, $rePricing->takealotShopDetails, $rePricing);
    }
  }
  public function updateLiveOfferPrice($rePricing, $rePrice)
  {


    $offersPriceUpdate = new OffersController();
    $result = $offersPriceUpdate->updateTakealotOffer($rePricing->offer->offer_id, $rePrice, $rePricing->takealotShopDetails, $rePricing);

    if (!empty($result)) {
      $offerUpdated = json_decode($result);
      if (!empty($offerUpdated->offer)) {
        $rePricing->offer->selling_price = $offerUpdated->offer->selling_price;
        $rePricing->offer->rrp = $offerUpdated->offer->rrp;
        if ($rePricing->offer->save()) {
          $current_time = Carbon::now();
          $rePricing->winning_price = $rePricing->offer->selling_price;
          $rePricing->price_updated_at = $current_time;
          $rePricing->save();
        }
      }
    }
    return $rePricing;
  }

  public function competitonHistory($data, $rePricing, $leadtimeID, $percentage, $finalPrice)
  {

    $checkOffer = TakealotCompetitionHistory::where('tsin', '=', $rePricing->offer->tsin_id)->first();
    if (!empty($checkOffer)) {
      $checkOffer->status = 'losser';
      $checkOffer->save();
    }

    $competition = new TakealotCompetitionHistory();
    $competition->plid_history_id = $leadtimeID;
    $competition->tsin = $rePricing->offer->tsin_id;
    $competition->your_price = $finalPrice;
    $competition->competitor_price = $data->data_layer->totalPrice;
    $competition->percentage = $percentage;
    $competition->status = 'losser';
    $competition->last_status = 'losser';
    $competition->save();
    return true;
  }
  //start code --- offer price up if our offer is already winner
  public function offerPriceUp($data, $rePricing)
  {

    $takealotStockTotal = 0;
    $leadTimeStockTotal = 0;
    //stock code started
    if (!empty($rePricing->offer->leadtime_stock)) {
      $leadtime_decode = json_decode($rePricing->offer->leadtime_stock);
      $leadTimeStockTotal = 0;
      if (!empty($leadtime_decode) && count($leadtime_decode) > 0) {
        foreach ($leadtime_decode as $key => $item) {
          $qty = (!empty($item->quantity_available) && isset($item->quantity_available) && $item->quantity_available > 0) ? $item->quantity_available : 0;
          $leadTimeStockTotal = $leadTimeStockTotal + $qty;
        }
      }
    }
    if (!empty($rePricing->offer->stock_at_takealot)) {
      $takealotStock_decode = json_decode($rePricing->offer->stock_at_takealot);
      if (!empty($takealotStock_decode) && count($takealotStock_decode) > 0) {
        foreach ($takealotStock_decode as $key => $item) {
          $qty = (!empty($item->quantity_available) && isset($item->quantity_available)) ? $item->quantity_available : 0;
          $takealotStockTotal = $takealotStockTotal + $qty;
        }
      }
    }
    //stock code ended     
    if (!empty($data->other_offers->conditions[0]->items)) {
      $otherOfferTopPrice = $data->other_offers->conditions[0]->items[0]->price;
      $stock = false;
      if (isset($data->other_offers->conditions[0]->items[0]->stock_availability->is_in_stock) ||  $data->other_offers->conditions[0]->items[0]->stock_availability->status == "In stock") {
        $stock = true;
      }
      $checkOnTop = [];
      if (isset($data->other_offers->conditions[0]->items) && !empty($data->other_offers->conditions[0]->items)) {
        foreach ($data->other_offers->conditions[0]->items as $key => $itemOther) {
          if ($itemOther->stock_availability->is_in_stock == false) {
            $checkOnTop['leadtime'] = 1;
          }
          // else{
          if ($itemOther->stock_availability->is_in_stock == true) {
            $checkOnTop['instock'][0] = $key + 1;
            $checkOnTop['instock'][1] = $data->other_offers->conditions[0]->items[$key]->price;
            break;
          }
        }
      }


      //if it is true it means in other offers on top is instock offer if false it means in other offers on top offer is leadtime
      if ($stock == true && (isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && $checkOnTop['instock'][0] == 1) && $takealotStockTotal > 0) {
        //need to remove given below condition
        // if ($data->stock_availability->is_leadtime == false  && $takealotStockTotal > 0) {
        $rePrice = $checkOnTop['instock'][1] - 1;
        if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
          if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
            $rePrice = $rePricing->max_price;
          }
          if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
            $rePrice = $rePricing->min_price;
          }
        }

        if ($rePricing->offer->selling_price != $rePrice && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {
          return $this->updateLiveOfferPrice($rePricing, $rePrice);
        } else {
          return null;
        }
        // }
      } else {

        //if other instock is available insode otheroffer list 
        if ($stock == false && (isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1]))) {

          if ($data->stock_availability->is_leadtime == false && $takealotStockTotal > 0) {
            $rePrice = $checkOnTop['instock'][1] - 1;
            if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
              if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
                $rePrice = $rePricing->max_price;
              }
              if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                $rePrice = $rePricing->min_price;
              }
            }
            if ($rePricing->offer->selling_price != $rePrice && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {
              return $this->updateLiveOfferPrice($rePricing, $rePrice);
            } else {
              return null;
            }
          }
        }
        //if our and compititor offer is on leadtime
        if (($stock == false && $data->stock_availability->is_leadtime == true) && (isset($checkOnTop['leadtime']) && $checkOnTop['leadtime'] == 1) && ($takealotStockTotal <= 0 && $leadTimeStockTotal > 0)) {
          $rePrice = $otherOfferTopPrice - 1;
          if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
            if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
              $rePrice = $rePricing->max_price;
            }
            if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
              $rePrice = $rePricing->min_price;
            }
          }

          if ($rePricing->offer->selling_price != $rePrice && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {
            return $this->updateLiveOfferPrice($rePricing, $rePrice);
          } else {
            return null;
          }
        }
        //our offer is instock and compititor offers on leadtime
        if ($stock == false && $takealotStockTotal > 0) {
          $priceUpdatedAt = Carbon::parse($rePricing->price_updated_at);
          $currentDateTime = Carbon::now();
          $diffInMinutes = $priceUpdatedAt->diffInMinutes($currentDateTime);
          if ($diffInMinutes >= 20 && $rePricing->winner == 1) {
            $checkPlid = TakealotPlidHistory::where([['plid', '=', $rePricing->offer->plid], ['stock_type', '=', 'instockVsLeadtime']])->latest()->first();
            if (!empty($checkPlid)) {
              $checkPlid->percentage = ($checkPlid->percentage + 1 > 34.1) ? 33.1 : $checkPlid->percentage + 1;
              $getPercenategDifference = ($otherOfferTopPrice * $checkPlid->percentage) / 100;
              $rePrice = $otherOfferTopPrice + (floor($getPercenategDifference));

              if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
                  $rePrice = $rePricing->max_price;
                }
                if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                  $rePrice = $rePricing->min_price;
                }
              }
              if (($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {

                if ($rePricing->offer->selling_price != $rePrice) {
                  $checkPlid->save();
                  return $this->updateLiveOfferPrice($rePricing, $rePrice);
                } else {
                  return null;
                }
              }
            } else {
              $checkPlid = new TakealotPlidHistory();
              $checkPlid->plid = $rePricing->offer->plid;
              $checkPlid->stock_type = 'instockVsLeadtime';
              $checkPlid->percentage = 10;
              $checkPlid->status = 'losser';
              $getPercenategDifference = ($otherOfferTopPrice * $checkPlid->percentage) / 100;
              $rePrice = $otherOfferTopPrice + (floor($getPercenategDifference));
              if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
                  $rePrice = $rePricing->max_price;
                }
                if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                  $rePrice = $rePricing->min_price;
                }
              }
              if (($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {

                if ((isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1]))) {
                  if ($rePrice >= $checkOnTop['instock'][1]) {
                    $rePrice = $checkOnTop['instock'][1] - 1;
                    $firstVal = abs($checkOnTop['instock'][1] - $rePricing->offer->selling_price);
                    $secondVal = ($checkOnTop['instock'][1] + $rePricing->offer->selling_price) / 2;
                    $finalVal = abs($firstVal) / $secondVal;
                    $checkPlid->percentage = $finalVal / 100;
                  }
                }


                if ($rePricing->offer->selling_price != $rePrice) {
                  $checkPlid->save();
                  return $this->updateLiveOfferPrice($rePricing, $rePrice);
                } else {
                  return null;
                }
              }
            }
          }
        }

        //our offer is on leadtime and compititor offers is instock
        if ($stock == false && $takealotStockTotal <= 0 && $leadTimeStockTotal > 0) {
          $priceUpdatedAt = Carbon::parse($rePricing->price_updated_at);
          $currentDateTime = Carbon::now();
          $diffInMinutes = $priceUpdatedAt->diffInMinutes($currentDateTime);
          if ($diffInMinutes >= 20 && $rePricing->winner == 1) {
            $checkPlid = TakealotPlidHistory::where([['plid', '=', $rePricing->offer->plid], ['stock_type', '=', 'leadtimeVsinstock']])->latest()->first();

            if (!empty($checkPlid)) {
              $checkPlid->percentage = ($checkPlid->percentage + 1 <= 1) ? 1 : $checkPlid->percentage + 1;
              $getPercenategDifference = ($otherOfferTopPrice * $checkPlid->percentage) / 100;
              $rePrice = $otherOfferTopPrice - (floor($getPercenategDifference));
              if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
                  $rePrice = $rePricing->max_price;
                }
                if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                  $rePrice = $rePricing->min_price;
                }
              }
              if ($rePricing->offer->selling_price != $rePrice && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {
                $checkPlid->save();
                return $this->updateLiveOfferPrice($rePricing, $rePrice);
              } else {
                return null;
              }
            } else {
              $checkPlid = new TakealotPlidHistory();
              $checkPlid->plid = $rePricing->offer->plid;
              $checkPlid->stock_type = 'leadtimeVsinstock';
              $checkPlid->percentage = 10;
              $checkPlid->status = 'losser';
              $getPercenategDifference = ($otherOfferTopPrice * $checkPlid->percentage) / 100;
              $rePrice = $otherOfferTopPrice + (floor($getPercenategDifference));
              if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price || $rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                if ($rePrice > $rePricing->max_price || $rePricing->offer->selling_price > $rePricing->max_price) {
                  $rePrice = $rePricing->max_price;
                }
                if ($rePrice < $rePricing->min_price || $rePricing->offer->selling_price < $rePricing->min_price) {
                  $rePrice = $rePricing->min_price;
                }
              }
              if ($rePricing->offer->selling_price != $rePrice && ($rePrice >= $rePricing->min_price && $rePrice <= $rePricing->max_price)) {

                if ((isset($checkOnTop['instock']) && isset($checkOnTop['instock'][0]) && !empty($checkOnTop['instock'][1]))) {
                  if ($rePrice >= $checkOnTop['instock'][1]) {
                    $rePrice = $checkOnTop['instock'][1] - 1;
                    $firstVal = abs($checkOnTop['instock'][1] - $rePricing->offer->selling_price);
                    $secondVal = ($checkOnTop['instock'][1] + $rePricing->offer->selling_price) / 2;
                    $finalVal = abs($firstVal) / $secondVal;
                    $checkPlid->percentage = $finalVal / 100;
                  }
                }

                if ($rePricing->offer->selling_price != $rePrice) {
                  $checkPlid->save();
                  return $this->updateLiveOfferPrice($rePricing, $rePrice);
                } else {
                  return null;
                }
              }
            }
          }
        }
      }
    }
  }
  //end code --- offer price up if our offer is already winner
  public function variantOfferUrl($title, $url, $result)
  {
    // ini_set('max_execution_time', 0);
    $newTitle = null;
    if (!empty($result) && !empty($result->variants) && !empty($result->variants->selectors) && count($result->variants->selectors) > 0) {

      $selectors = null;
      foreach ($result->variants->selectors as $key => $item) {
        foreach ($item->options as $key => $variant) {
          if (!empty($variant->value->value)) {
            $selectors[$item->selector_type][] = $variant->value->value;
          } else {
            $selectors[$item->selector_type][] = $variant->value;
          }
        }
      }

      $combinations = $this->get_combinations($selectors);
      foreach ($combinations as $Pkey => $comb) {

        $varTitle = null;
        $varParams = null;
        foreach ($comb as $key => $variant) {
          if (!empty($varTitle)) {
            $varTitle = $varTitle . ' - ' . $variant;
            $varParams = $varParams . '&' . $key . '=' . $variant;
          } else {
            $varTitle = ' - ' . $variant;
            $varParams = '?' . $key . '=' . $variant;
          }
        }

        $newTitle = null;
        $newTitle = $result->title . $varTitle;

        if ($newTitle == $title) {

          $productVariant = $varParams;
          return  $productVariant;
        }
      }
    }

    return null;
  }

  function get_combinations($arrays)
  {
    $result = array(array());
    foreach ($arrays as $property => $property_values) {
      $tmp = array();
      foreach ($result as $result_item) {
        foreach ($property_values as $property_key => $property_value) {
          $tmp[] = $result_item + array($property => $property_value);
        }
      }
      $result = $tmp;
    }
    return $result;
  }
  public function guzzelCurl($repriceData)
  {
    try {
      if (!empty($repriceData->offer->plid)) {
        $apiCall = apiProxyCall();
        if (!empty($apiCall) && $apiCall->active_status == 1 && $apiCall->remaining_limit > 0 &&  $apiCall->status == 1 && isset($apiCall->api_url)) {
          // Your data
          $data = [
            "plid" => $repriceData->offer->plid,
            "is_variant" => $repriceData->offer->is_variant,
            "variant_params" => $repriceData->offer->variant_params
          ];

          if ($apiCall->api_url == env('APP_URL')) {

            $scrapCont = new ScrapingApiController();
            $result = $scrapCont->scrapData($data);
            if (!empty($result) && $result->getStatusCode() == 200) {
              $jsonContent = $result->getContent();
              $result = json_decode($jsonContent);
              if (isset($result->status) && $result->status == 'success' && isset($result->data) && !empty($result->data)) {
                return $result->data;
              }

              //this code will run if any error occured
              if (isset($repriceData->offer->id)) {
                $jobName = 'repricingerrorJob' . $repriceData->offer->id;
                TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
              }

              return null;
            } else {

              //this code will run if any error occured
              if (isset($repriceData->offer->id)) {
                $jobName = 'repricingerrorJob' . $repriceData->offer->id;
                TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
              }


              if ($result->getStatusCode() == 429) {
                $data = [
                  'url' => $apiCall->api_url,
                  'status' => $result->getStatusCode(),
                  'data' => $result
                ];

                $settingsController = new SettingsController();
                $settingsController->resetIprateLimitOnError($data);
                saveError($data);
              } else {
                $data = [
                  'url' => $apiCall->api_url,
                  'status' => $result->getStatusCode(),
                  'data' => $result
                ];
                saveError($data);
              }

              return null;
            }
          } else {
            // Create a Guzzle client
            $client = new Client(['base_uri' => $apiCall->api_url, 'verify' => false]);
            $jsonData = json_encode($data);
            $response = $client->post('/api/scrap/offer', [
              'body' => $jsonData,
              'headers' => [
                'Content-Type' => 'application/json', // Set the content type to JSON
              ]
            ]);


            if (!empty($response) && $response->getStatusCode() == 200) {
              $result = json_decode($response->getBody()->getContents());
              if (!empty($result) && $result->status == 'success') {

                return $result->data;
              } else {
                //this code will run if any error occured
                if (isset($repriceData->offer->id)) {
                  $jobName = 'repricingerrorJob' . $repriceData->offer->id;
                  TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
                }
              }
            } else {

              saveError($data);
              //this code will run if any error occured
              if (isset($repriceData->offer->id)) {
                $jobName = 'repricingerrorJob' . $repriceData->offer->id;
                TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
              }

              if ($response->getStatusCode() == 429) {
                $data = [
                  'url' => $apiCall->api_url,
                  'status' => $response->getStatusCode(),
                  'data' => $response
                ];

                $settingsController = new SettingsController();
                $settingsController->resetIprateLimitOnError($data);
                saveError($data);
              } else {
                $data = [
                  'url' => $apiCall->api_url,
                  'status' => $response->getStatusCode(),
                  'data' => $response
                ];
                saveError($data);
              }
            }
          }
        } else {
          //this code will run if any error occured
          if (isset($repriceData->offer->id)) {
            $jobName = 'repricingerrorJob' . $repriceData->offer->id;
            TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
          }
          return null;
        }
      }
    } catch (\Exception $e) {
      //this code will run if any error occured
      if (isset($repriceData->offer->id)) {
        $jobName = 'repricingerrorJob' . $repriceData->offer->id;
        TakealotInstantRepricingJob::dispatch($repriceData->offer->id, $jobName)->onQueue('high');
      }
      $data = [
        'catch' => 'auto repricing catch error',
        'status' => $e->getCode(),
        'data' => $e->getMessage()
      ];
      saveError($data);
      return null;
    }
  }
  public function disableFromRePricing()
  {
    //  ini_set('max_execution_time', 0);
    $rePricing = TakealotRePricing::select('takealot_repricing.*')
      ->join('takealot_offers', function ($query) {

        $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
          ->where([['takealot_offers.status', '!=', 'Buyable']]);
      })->get();

    foreach ($rePricing->chunk(10) as $chunk) {
      foreach ($chunk as $key => $item) {
        $item->status = 0;
        $item->save();
      }
    }
  }
  public function removeExtraOffersFromRePricing()
  {
    $packages = PackageBuy::with('packageDetails')->get();
    if (!empty($packages)) {
      foreach ($packages as $item) {
        if ($item->seller_id != 1) {
          $repricing = TakealotRePricing::where([['seller_id', '=', $item->seller_id]]);
          $alreadyIn = $repricing->count();
          if (!empty($item->packageDetails) && isset($item->packageDetails->repricable_offers) && $alreadyIn > $item->packageDetails->repricable_offers) {
            $totalRemove = $alreadyIn - $item->packageDetails->repricable_offers;
            $getAll = $repricing->limit($totalRemove)->orderBy('status', 'asc')->get();
            foreach ($getAll as $offer) {
              $offer->delete();
            }
          }
        }
      }
    }
  }
}




// public function guzzelCurl($repriceData)
      // {
      //   // try{
      //        sleep(1);
      //       $ipRateLimit=IpRateLimitChecker();
      //       if($ipRateLimit>0)
      //       {
      //         // $rotateProxy=rotateIpProxy();
      //         // $proxy=null;
      //         // if(!empty($rotateProxy) && isset($rotateProxy['status']) && $rotateProxy['status']=='success')
      //         // {
      //         //     $response=json_decode($rotateProxy['data'], true);
      //         //     if(isset($response['curl']))
      //         //     {
      //         //         $proxy=$response['curl'];
      //         //     }
                  
      //         // }
      //         if(!empty($repriceData->offer->plid))
      //         {          
      //           // ini_set('max_execution_time', 0);  
      //           $url="https://api.takealot.com/rest/v-1-10-0/product-details/";
      //           $variantUrl=null;
      //           $singleUrl=null;
      //           if($repriceData->offer->is_variant==1 && !empty($repriceData->offer->variant_params))
      //           {
      //             $variantUrl=$repriceData->offer->plid.$repriceData->offer->variant_params.'&platform=desktop&display_credit=true';
                
      //           }
      //           else
      //           {
      //             $singleUrl=$repriceData->offer->plid.'?platform=desktop&display_credit=true';
      //           }
      //           $headers = [
      //             'Content-Type'=>'application/json',
      //             'accept-encoding'=>'gzip, deflate, br',
      //             'cache-control'=>'max-age=0',
      //             'origin'=>'https://www.takealot.com',
      //             'referer'=>'https://www.takealot.com/',
      //             'user-agent' => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.104 Safari/537.36',
      //             'accept' => 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
      //             // 'proxy' =>  $proxy,
      //           ];
      //           // $client  = new Client(['handler' => $this->handlerStack,'base_uri' =>$url,'verify' => false]);
      //           $client  = new Client(['handler' => $this->handlerStack,'base_uri' =>$url,'verify' => true]);
      //           if(!empty($variantUrl))
      //           {
      //             $request = new Request('GET',$variantUrl,$headers);
      //           }
      //           else
      //           {
      //             $request = new Request('GET',$singleUrl,$headers);

      //           }
              
      //           $response = $client->send($request);
      //           if($response->getStatusCode()==200)
      //           {
      //             $result=json_decode($response->getBody()->getContents());                      
      //             if(!empty($result))
      //             {          
      //               return $result;
      //             }
      //           }
      //           else
      //           {
      //             return null;
      //           }
                
      //         }
      //       }
              
      //         return null;
      //   // }
      //   // catch (\Exception $e) 
      //   // {
      //   //   Logger($e);
      //   // } 
      // }