<?php

namespace App\Http\Controllers\Seller;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\TakealotOffers;
use App\Models\Replenishment;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use PDF;
use Auth;

class ReplenishmentController extends Controller
{
    public function SendStockPdf(Request $request)
    {
        $sortBy = $request->has('sort') ? $request->input('sort') : 'asc';
        ini_set('max_execution_time', 0);
        //start check active shop
        if (empty(session('active_shop_id'))) {
            $activeShop = Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
        }
        $active_shop_id = session('active_shop_id');
        //end active shop id
        $data = [];
        $data['offers'] = $this->getReplenishmentData($active_shop_id);
        $data['offers'] = $this->saveReplenishment($data, $active_shop_id, $sortBy);
        $pdf = PDF::loadView('livewire.seller.replenishment.send-stock-pdf', $data);
        $pdf->setOptions(['isRemoteEnabled' => true]);
        $pdf->getDomPDF()->setProtocol($_SERVER['DOCUMENT_ROOT']);
        return $pdf->stream();
    }
    public function saveReplenishment($data, $active_shop_id, $sortBy)
    {
        Replenishment::where([['seller_id', '=', Auth::user()->id], ['shop_id', '=', $active_shop_id]])->delete();
        foreach ($data['offers'] as $key => $item) {

            $remainingTotalDaysCover = 0;
            $deadCost = 0;
            $getPricesku = explode('-', $item->sku);
            $profitCheck = !empty($getPricesku) && !empty($getPricesku[0]) && is_numeric(trim($getPricesku[0])) ? $item->selling_price - (int) trim($getPricesku[0]) : 0;

            if (!empty($item->takealotOrders) && isset($item->takealotOrders)) {

                $sevenDays = \Carbon\Carbon::now()->subDays(7)->toDateString();
                $seven = $item->takealotOrders->where('order_date', '>=', $sevenDays)->sum('quantity');
                $fifteenDays = \Carbon\Carbon::now()->subDays(15)->toDateString();
                $fifteen = $item->takealotOrders->where('order_date', '>=', $fifteenDays)->sum('quantity');
                $thirtyDays = \Carbon\Carbon::now()->subDays(30)->toDateString();
                $thirty = $item->takealotOrders->where('order_date', '>=', $thirtyDays)->sum('quantity');
                $thirtyDayRemainingPieces = $thirty / 30;
                $ninetyDays = \Carbon\Carbon::now()->subDays(90)->toDateString();
                $ninety = $item->takealotOrders->where('order_date', '>=', $ninetyDays)->sum('quantity');
                $totalReturned = $item->takealotOrders->where('sale_status', '=', 'Returned')->sum('quantity');

                $thirtydaysCost = is_numeric($thirty) && is_numeric($getPricesku[0]) ? $thirty * $getPricesku[0] : $thirty;
                $thirtydaysCostReturned = $thirty - $totalReturned;
                $deadCost = !empty($thirtydaysCost) && !empty($thirtydaysCostReturned) ? $thirtydaysCost / $thirtydaysCostReturned : 0;

                //Stock Day cover takealot start
                $salesUnitCpt = $item->takealotOrders->where('dc', '=', 'CPT')->sum('quantity');
                $salesUnitJhb = $item->takealotOrders->where('dc', '=', 'JHB')->sum('quantity');

                $sevenDays = \Carbon\Carbon::today()->subDays(7);
                $totalSales = $item->takealotOrders->where('order_date', '>=', $sevenDays)->sum('quantity');
                // //Stock Day cover takealot end

                $stockAtTakealot = json_decode($item->stock_at_takealot);
                $stockAtTakealotCptName = !empty($stockAtTakealot[0]->warehouse->name) ? $stockAtTakealot[0]->warehouse->name : null;
                $stockAtTakealotCptQty = !empty($stockAtTakealot[0]->quantity_available) ? $stockAtTakealot[0]->quantity_available : 0;
                $stockAtTakealotJhbName = !empty($stockAtTakealot[1]->warehouse->name) ? $stockAtTakealot[1]->warehouse->name : null;
                $stockAtTakealotJhbQty = !empty($stockAtTakealot[1]->quantity_available) ? $stockAtTakealot[1]->quantity_available : 0;
                $totalStockOnDc = $stockAtTakealotCptQty + $stockAtTakealotJhbQty;

                $stockOnWay = json_decode($item->stock_on_way);
                $stockOnWayCptName = !empty($stockOnWay[0]->warehouse->name) ? $stockOnWay[0]->warehouse->name : null;
                $stockOnWayCptQty = !empty($stockOnWay[0]->quantity_available) ? $stockOnWay[0]->quantity_available : 0;

                $stockOnWayJhbName = !empty($stockOnWay[1]->warehouse->name) ? $stockOnWay[1]->warehouse->name : null;
                $stockOnWayJhbQty = !empty($stockOnWay[1]->quantity_available) ? $stockOnWay[1]->quantity_available : 0;
                $stockOnWayTotal = $stockOnWayCptQty + $stockOnWayJhbQty;

                $percentageJhb = !empty($salesUnitCpt && $salesUnitCpt > 0) && (!empty($salesUnitJhb) && $salesUnitJhb > 0) ? ($salesUnitJhb / ($salesUnitJhb + $salesUnitCpt)) * 100 : (!empty($salesUnitJhb) && $salesUnitJhb > 0 ? 100 : 0);
                $percentageCpt = !empty($salesUnitCpt) && $salesUnitCpt > 0 && (!empty($salesUnitJhb) && $salesUnitJhb > 0) ? ($salesUnitCpt / ($salesUnitCpt + $salesUnitJhb)) * 100 : (!empty($salesUnitCpt) && $salesUnitCpt > 0 ? 100 : 0);

                $percentageJhb = round($percentageJhb); //53%
                $percentageCpt = round($percentageCpt); //47%
                $totalSendUnit = $totalSales * 2; //492

                $sendJhb = $totalSendUnit * ($percentageJhb / 100) - ($stockOnWayJhbQty + $stockAtTakealotJhbQty);
                $sendCpt = $totalSendUnit * ($percentageCpt / 100) - ($stockOnWayCptQty + $stockAtTakealotCptQty);
                $sendNewTotal = $sendJhb + $sendCpt;
                // dd($totalSendUnit);
                if ($sendNewTotal > 0) {
                    $stock = [
                        'jhb' => round($sendJhb),
                        'cpt' => round($sendCpt),
                        'total' => round($sendJhb) + round($sendCpt),
                        'seven' => $seven,
                        'fifteen' => $fifteen,
                        'thirty' => $thirty,
                        'ninety' => $ninety,
                    ];
                    $rep = new Replenishment();
                    $rep->seller_id = Auth::user()->id;
                    $rep->shop_id = $active_shop_id;
                    $rep->takealot_offers_id = $item->id;
                    $rep->send_stock = json_encode($stock);
                    $rep->profit = round($profitCheck);
                    $rep->save();
                }
            }
        }
        $result = Replenishment::with('offer')->where([['seller_id', '=', Auth::user()->id], ['shop_id', '=', $active_shop_id]])->orderBy('profit', $sortBy)->get();
        return $result;
        //fetch all record and return it
    }
    public function getReplenishmentData($active_shop_id)
    {

        $activeShopID = $active_shop_id;
        $activeSellerID = Auth::user()->id;
        $thirtyDays = Carbon::now()->subDays(30)->toDateString();
        $takealotOffers = TakealotOffers::distinct('takealot_offers.id')->select('takealot_offers.id', 'takealot_offers.offer_id', 'takealot_offers.other_status', 'takealot_offers.seller_id', 'takealot_offers.leadtime_stock', 'takealot_offers.stock_at_takealot_total', 'takealot_offers.leadtime_days', 'takealot_offers.takealot_shops_id', 'takealot_offers.stock_cover', 'takealot_offers.stock_on_way', 'takealot_offers.offer_url', 'takealot_offers.tsin_id', 'takealot_offers.sku', 'takealot_offers.stock_at_takealot', 'takealot_offers.status', 'takealot_offers.title', 'takealot_offers.image_url', 'takealot_offers.selling_price', 'takealot_offers.total_stock_on_way', 'takealot_offers.date_created', 'takealot_offers.sales_units', DB::raw('(SELECT SUM(quantity) FROM takealot_orders WHERE offer_id = takealot_offers.offer_id AND takealot_orders.sale_status NOT IN (\'Cancelled\', \'Refunded\', \'Cancelled by Customer\', \'Cancelled - Late\', \'Cancelled by Takealot - DC Stock Inquiry\', \'Cancelled - Inbound Exception (Short Supplied)\')  AND takealot_orders.order_date >= DATE_SUB(NOW(), INTERVAL 30 DAY)) as thirty_day_sales_quantity'))->with('takealotShops', 'takealotOrders', 'rePricing', 'takealotOfferDetails')

            ->leftJoin('takealot_orders', function ($join) use ($thirtyDays, $activeShopID, $activeSellerID) {
                $join->on('takealot_offers.offer_id', '=', 'takealot_orders.offer_id')
                    ->where('takealot_orders.order_date', '>=', $thirtyDays)
                    ->where([['takealot_orders.seller_id', '=', $activeSellerID], ['takealot_orders.takealot_shops_id', '=', $activeShopID]]);
            });
        $takealotOffers = $takealotOffers->where([['takealot_offers.takealot_shops_id', '=', $activeShopID], ['takealot_offers.seller_id', '=', $activeSellerID]]);
        $takealotOffers = $takealotOffers->groupBy('takealot_offers.id')->orderBy('thirty_day_sales_quantity', 'desc')->get();

        return $takealotOffers;
    }
}
