<?php

namespace App\Http\Controllers\Seller;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\UserDetails;
use App\Models\PackageBuy;
use App\Models\SubscriptionPackages;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use App\Models\Countries;
use App\Jobs\PackageExpiryMailJob;
use App\Models\TakealotShops;
use App\Models\BusinessSettings;
use App\Models\ApiProxyCall;
use Illuminate\Support\Facades\Queue;
use Illuminate\Support\Facades\Log;

class SettingsController extends Controller
{

    public function takealotApi()
    {      
      return view('seller.configurations.settings.takealot-api');
    }
    public function resetIpApiLimit()
    {      
      
      //update api call limit 
      $this->checkCronJobs();
      $this->resetApiUrlsLimits();

      //update ip limit
      $remainingRateLimitUpdate = BusinessSettings::where('type','=','remaining_ip_rate_limit')->first();                   
      $totalIpRateLimit = BusinessSettings::where('type','=','ip_rate_limit')->first();                   
      if(!empty($remainingRateLimitUpdate))
      {
          $remainingRateLimitUpdate->value=(!empty($totalIpRateLimit) && (int)$totalIpRateLimit->value>0) ?  ((int)$totalIpRateLimit->value) : 0;
          $remainingRateLimitUpdate->save();
      }

      //update all shops limit
      $apiRateLimitUpdate=get_setting('takealot_ratelimit');
      TakealotShops::updateApiRateLimits($apiRateLimitUpdate);


      //reserved limits update

      //update take alot reserved
      $remainingReservedRateLimitUpdate = BusinessSettings::where('type','=','re_tk_reserved_rate_limit')->first();                   
      $totalRSVTkRateLimit         = BusinessSettings::where('type','=','takealot_reserved_rate_limit')->first();                  
      if(!empty($remainingReservedRateLimitUpdate))
      {
          $rmtktotal=(int)$totalRSVTkRateLimit->value;
          $remainingReservedRateLimitUpdate->value=($rmtktotal>0) ?  $rmtktotal : 0;
          $remainingReservedRateLimitUpdate->save();
      }

      //update ip reserved limit
      $remainingIpReservedRateLimitUpdate = BusinessSettings::where('type','=','re_ip_reserved_rate_limit')->first();                   
      $totalRSVIPTkRateLimit         = BusinessSettings::where('type','=','ip_reserved_rate_limit')->first();                  
      if(!empty($remainingIpReservedRateLimitUpdate))
      {
          $rmTotalnew=(int)$totalRSVIPTkRateLimit->value;
          $remainingIpReservedRateLimitUpdate->value=($rmTotalnew>0) ? $rmTotalnew : 0;
          $remainingIpReservedRateLimitUpdate->save();
      }

    }
    public function checkCronJobs()
    {
      $countQueue = DB::table('jobs')->count();
      if(!empty($countQueue) &&  $countQueue>0 && $countQueue>1000)
      {
           $defaultMaxRuning=get_setting('default_max_jobs_run_time_stop');
            set_setting('long_check_rescrap_time',$defaultMaxRuning);
            set_setting('losser_rescrap_time',$defaultMaxRuning);
            set_setting('winner_rescrap_time',$defaultMaxRuning);
      }
      elseif(!empty($countQueue) &&  $countQueue>0 && $countQueue>500 && $countQueue<1000)
      {
           $defaultMaxRuning=15;
            set_setting('long_check_rescrap_time',$defaultMaxRuning);
            set_setting('losser_rescrap_time',$defaultMaxRuning);
            set_setting('winner_rescrap_time',$defaultMaxRuning);
      }
      elseif(!empty($countQueue) &&  $countQueue>0 && $countQueue>300 && $countQueue<500)
      {
           $defaultMaxRuning=10;
            set_setting('long_check_rescrap_time',$defaultMaxRuning);
            set_setting('losser_rescrap_time',$defaultMaxRuning);
            set_setting('winner_rescrap_time',$defaultMaxRuning);
      }
      elseif(!empty($countQueue) &&  $countQueue>0 && $countQueue>200  && $countQueue<300)
      {
           $defaultMaxRuning=8;
            set_setting('long_check_rescrap_time',$defaultMaxRuning);
            set_setting('losser_rescrap_time',$defaultMaxRuning);
            set_setting('winner_rescrap_time',$defaultMaxRuning);
      }
      else
      {
         $defaultMaxReCheck=get_setting('default_long_check_rescrap_time');
         $defaultWinner=get_setting('default_winner_rescrap_time');
         $defaultLosser=get_setting('default_losser_rescrap_time');
         set_setting('long_check_rescrap_time',$defaultMaxReCheck);
         set_setting('losser_rescrap_time',$defaultLosser);
         set_setting('winner_rescrap_time',$defaultWinner);
      }
    }
    public function resetApiUrlsLimits()
    {      
     
      $lessCheckTime = Carbon::now()->subMinutes(1);   
      $greaterCheckTime = Carbon::now()->subMinutes(5);   
      $apiIpLimit=ApiProxyCall::where([['active_status','=',0],['time_status_changed','>', $greaterCheckTime],['time_status_changed','<', $lessCheckTime]])->get();
     
      if(!empty($apiIpLimit) && $apiIpLimit->count()>0)
      {
        
        foreach($apiIpLimit as $apiItem)
        {
          
          $apiItem->remaining_limit=$apiItem->total_limit;
          $apiItem->reserved_limit=$apiItem->total_reserved_limit;
          $apiItem->status=1;
          $apiItem->active_status=1;
          $allItem->reset_time=Carbon::now();
          $apiItem->save();
        }
      }
      else
      {
        $resetTime = Carbon::now()->subMinutes(1);     
        $ip_rate_limit=get_setting('ip_rate_limit');
        $reserved_rate_limit=get_setting('ip_reserved_rate_limit'); 
     
        $overAllLimit=ApiProxyCall::where('active_status','=',1)->where('reset_time','<', $resetTime)->get();
       
        if(!empty($overAllLimit))
        {
          foreach($overAllLimit as $allItem)
          {
            $allItem->remaining_limit = $ip_rate_limit;
            $allItem->reserved_limit  = $reserved_rate_limit;
            $allItem->total_limit     = $ip_rate_limit;
            $allItem->total_reserved_limit=$reserved_rate_limit;
            $allItem->status=1;
            $allItem->active_status=1;
            $allItem->reset_time=Carbon::now();
            $allItem->save();
          }
        }
        
      }
      
    }
    public function resetIprateLimitOnError($data)
    {      
      if($data['url']!=env('APP_URL'))
      {
          $apiItem=ApiProxyCall::where('api_url','=', $data['url'])->first();
          if(!empty($apiItem))
          {
            
              if($apiItem->remaining_limit>0 && $apiItem->total_limit>0 && $apiItem->total_limit>$apiItem->remaining_limit)
              {
                $errorOnLimit=$apiItem->total_limit-$apiItem->remaining_limit;
                $apiItem->total_limit=$errorOnLimit-1;
                $apiItem->remaining_limit=0;
                $apiItem->reserved_limit=0;
              }
              else
              {
                $ip_rate_limit=get_setting('ip_rate_limit');
                $reserved_rate_limit=get_setting('ip_reserved_rate_limit');
                $apiItem->remaining_limit=0;
                $apiItem->reserved_limit=0;
                $apiItem->total_limit=$ip_rate_limit;
                $apiItem->total_reserved_limit=$reserved_rate_limit;
              }

              $apiItem->status=0;
              $apiItem->active_status=0;
              $apiItem->time_status_changed=Carbon::now();
              $apiItem->save();    

            
          }
      }
    }    
    public function checkPackageExpire()
    {
      $currentDate = Carbon::now()->toDateString();
      $packageBuy=PackageBuy::with('packageDetails','seller')->whereDate('renew_date','<',$currentDate)->get();   
     
      if(!empty($packageBuy))
      {
          foreach($packageBuy as $item)
          {
              if($item->seller_id!=1)
              {
                  $renewDate = Carbon::parse($item->renew_date);
                  $newExpiryDate = $renewDate->addDays(4)->toDateString();
                  if($currentDate>$newExpiryDate)
                  {
                    if($item->packageDetails->plan_type!='basic' && $item->packageDetails->package_type!='free') 
                    {
                      $basic=SubscriptionPackages::where([['plan_type','=','basic'],['package_type','=','free'],['status','=','active']])->first();
                      if(!empty($basic))
                      {              
                          $todayDate = Carbon::now();
                          $futureDate = $todayDate->addDays(30);
                          $item->subscription_packages_id=$basic->id;
                          $item->renew_date=$futureDate;
                          $item->status='active';
                          if($item->save())
                          {
                              $pkgBuyHistory=new PackageBuyHistory();
                              $pkgBuyHistory->seller_id=$item->seller_id;
                              $pkgBuyHistory->package_details=null;
                              $pkgBuyHistory->event='free';
                              $pkgBuyHistory->price=$basic->price;
                              $pkgBuyHistory->subscription_packages_id=$basic->id;
                              $pkgBuyHistory->save();
                          }

                      }
                      else
                      {
                          $item->status='inactive';
                          $item->subscription_status='disabled';
                          $item->save();
                      }
                    }
                    else
                    {
                        if(!empty($item))
                        {
                          $todayDate = Carbon::now();
                          $futureDate = $todayDate->addDays(30);
                          $item->renew_date=$futureDate;
                          $item->status='active';
                          $item->save();
                        }
                         
                    }
                     

                  }
                  else
                  { 
                   
                    if($item->packageDetails->plan_type!='basic' && $item->packageDetails->package_type!='free') 
                    {
                      if(!empty($item->customer_email))
                      {
                        PackageExpiryMailJob::dispatch($newExpiryDate,$item->customer_email);                          
                      }
                      else
                      {
                         if(isset($item->seller->email))
                         {
                           PackageExpiryMailJob::dispatch($newExpiryDate,$item->seller->email);                          
                         }
                        
                      }
                     
                    }
                  }
              }
          }
      }
    }    

}
