<?php

namespace App\Http\Livewire\Admin\RePricing;
use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotCompetitionHistory;
use Livewire\WithPagination;
use Auth;
use App\Http\Controllers\Seller\RePricingController;
use App\Http\Controllers\Seller\OffersController;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
class Index extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $plidHistory=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];
    public $buybox=null;
    public $rePricingStatus=null;
    public $stockFilter=null;
    
    // protected $listeners = ['importAllBuyable','removeInActive','getPlidHistory'];
    protected $listeners = ['getPlidHistory','reCheckWinner','updatetakealotOffer'];

    public function mount()
    {
        
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         $this->active_shop_id=null;
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    }
    public function removeInActive()
    {
        if(TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['status','=',0]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Removed All In-Active Offers','move'=>0]);
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Something went wrong','move'=>0]);
        }

    }   
    
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function updatedStatus()
    {
        $this->resetPage();
    }
    public function updatedBuybox()
    {
        $this->resetPage();
    }
    public function updatedPriceSort()
    {
        $this->resetPage();
    }
    public function updatedRePricingStatus()
    {
        $this->resetPage();
    }
    public function updatedStockFilter()
    {
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingStatus=null;
         $this->stockFilter=null;
         $this->resetPage();

    }
    public function getPlidHistory($tsin)
    {
        $this->plidHistory=TakealotCompetitionHistory::where([['tsin','=',decrypt($tsin)]])->orderBy('created_at', 'desc')->limit(20)->get();
       
    }    
    public function closeModel()
    {
        $this->plidHistory=[];
    }
    
    public function importAllBuyable()
    {   
        $allBuyable=TakealotOffers::doesntHave('rePricing')->where([['status','=','Buyable'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->get();  
        if(!empty($allBuyable))
        {
            foreach($allBuyable as $buyable)
            {
                $checkAlready=TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['takealot_offers_id','=',$buyable->id]])->first();
                if(empty($checkAlready))
                {
                    $rePricing= new TakealotRePricing();
                    $rePricing->seller_id=$this->sellerID;
                    $rePricing->takealot_shops_id=$this->active_shop_id;
                    $rePricing->takealot_offers_id= $buyable->id;
                    $rePricing->min_price=0;
                    $rePricing->max_price=0;
                    if($rePricing->save())
                    {
                       // $repricing=new RePricingController();
                        //$repricing->repriceVariantParams($rePricing->id);
                       // TakealotInstantRepricingJob::dispatch($rePricing->id)->onQueue('high');
                    }
         
                }
            }
        }
        
    }
    
    public function render()  
    {
       
        $takealotRePricing = $this->readyToLoad ? TakealotRePricing::with('offer','offer.takealotPlidHistory','offer.takealotCompetitionHistory','takealotOfferDetails')
        ->when($this->buybox,function($query, $buybox){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->where('takealot_repricing.winner', $buybox);
           })
        ->when($this->rePricingStatus,function($query, $rePricingStatus){
            $rePricingStatus=($rePricingStatus=='enabled') ? 1 : 0;
            $query->where('takealot_repricing.status', $rePricingStatus);
           })
        ->when($this->search,function($query, $search){
            $search=trim($search);
            $query->whereHas('offer',fn($query) => $query->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%"));
           })
        ->when($this->status,function($query, $status){
            $query->whereHas('offer',fn($query) => $query->whereIn('takealot_offers.status',$status));
           })
        ->when($this->stockFilter,function($query, $stockFilter){

            if($stockFilter=='leadtime')
            {
                $query->whereHas('offer',fn($query) => $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) > 0"));
            }
            if($stockFilter=='instock')
            {
                $query->whereHas('offer',fn($query) => $query->where('takealot_offers.stock_at_takealot_total','>',0));
            }
            if($stockFilter=='outofstock')
            {
                $query->whereHas('offer',fn($query) => $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) <= 0")->where('takealot_offers.stock_at_takealot_total','<=',0));
            }

        
         })         
        ->when($this->active_shop_id,function($query, $active_shop_id){
            $query->where('takealot_shops_id','=',$active_shop_id);
           }) : [];
       
        if(!empty($takealotRePricing))
        {
        if(!empty($this->priceSort))
        {
           if($this->priceSort=='min')
           {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.min_price', $this->orderBy)->paginate($this->entries);
           }
           else if($this->priceSort=='max')
           {

            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.max_price', 'asc')->paginate($this->entries);

           }
           else
           {

             $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.id', 'desc')->paginate($this->entries);

           }

        }
        else
        {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.id', $this->orderBy)->paginate($this->entries);
        }
        }
        if(!empty($this->plidHistory))
        {
            
            $this->dispatchBrowserEvent('load-modal',['modelID'=> 'exampleModalScrollable']);
        }
        else
        {
            $this->dispatchBrowserEvent('close-modal',['modelID'=> 'exampleModalScrollable']);
        }
        // dd($takealotRePricing);
        return view('livewire.admin.re-pricing.index',['takealotRePricing'=>$takealotRePricing,'plidHistory'=>$this->plidHistory])->extends('admin.layouts.app');;
    }
}


