<?php

namespace App\Http\Livewire\Admin\Seller;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use Livewire\WithPagination;
use Auth;
use App\Jobs\TakealotInstantRepricingJob;
use App\Http\Controllers\Seller\RePricingController;
class Offers extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';
    
    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];
    public $buybox=null;
    public $rePricingFilter=null;
    public $stockFilter=null;
    public function mount()
    {
         
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         $this->active_shop_id=null;
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
    public function updatedStockFilter()
    {
        $this->resetPage();
    }
   
    public function removeFromRePricing($id)
    {
        if($rePricing=TakealotRePricing::where([['takealot_offers_id','=',decrypt($id)]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Successfully Removed From Re-Pricing List','move'=>0]);
        }
       
    }
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingFilter=null;
         $this->stockFilter=null;
         $this->resetPage();

    }
    
    public function render()
    {
            
        $takealotOffers =TakealotOffers::with('rePricing','takealotOfferDetails')      
        ->when($this->buybox,function($query, $buybox){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->whereHas('rePricing',fn($query) => $query->where('takealot_repricing.winner', $buybox)
           );
           })
        ->when($this->rePricingFilter,function($query, $rePricingFilter){   
              if($rePricingFilter=='yes')
              {

                $query->whereHas('rePricing',fn($query) => $query->where('takealot_repricing.winner','=',1)->orWhere('takealot_repricing.winner','=',0));

              }
              else
              {
                $query->doesntHave('rePricing');
              }
     
           })       
            ->when($this->active_shop_id,function($query, $active_shop_id){
           
                $query->where([['takealot_offers.takealot_shops_id','=',$active_shop_id]]);

            })
            ->when($this->search,function($query, $search){
            $search=trim($search);
            $query->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%");

            })
        ->when($this->status,function($query, $status){

             $query->whereIn('takealot_offers.status',$status);

            })
            ->when($this->stockFilter,function($query, $stockFilter){

             if($stockFilter=='leadtime')
            {
                $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) > 0");
            }
            if($stockFilter=='instock')
            {
                $query->where('takealot_offers.stock_at_takealot_total','>',0);
            }
            if($stockFilter=='outofstock')
            {
                $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) <= 0")->where('takealot_offers.stock_at_takealot_total','<=',0);
            }

            })->where('takealot_offers.title','!=',null);

            if(!empty($this->priceSort))
            {
                $this->orderBy=($this->priceSort=='lowest') ? 'asc' : 'desc';
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.selling_price', $this->orderBy)->paginate($this->entries);
            }
            else
            {
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.date_created', $this->orderBy)->paginate($this->entries);
            }
       
        $this->dispatchBrowserEvent('reloadSelect2');
        return view('livewire.admin.seller.offers',['takealotOffers'=>$takealotOffers])->extends('admin.layouts.app');
    }
}

