<?php

namespace App\Http\Livewire\Admin\Seller;

use Livewire\Component;
use App\Models\TakealotOrders;
use Livewire\WithPagination;
use App\Http\Controllers\Seller\OrdersController;
use Auth;
class Sales extends Component
{
    
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];

    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 100;
         $this->orderBy  = 'desc';
         $this->active_shop_id=null;
        //end active shop id
    }
    public function fetchLatestOrders()
    {
        $order=new OrdersController();
        if($order->manualFetchAllSales($this->active_shop_id))
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Latest orders successfully fetched','move'=>0]);
        }
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];

    }  
    public function render()
    {
        if($this->readyToLoad)
        {
            $orders = TakealotOrders::with('offer')
            ->when($this->search,function($query, $search){
                $search=trim($search);
                $query->where('takealot_orders.product_title', 'LIKE', "%{$search}%")->orWhere('takealot_orders.sku', 'LIKE', "%{$search}%")->orWhere('takealot_orders.tsin', 'LIKE', "%{$search}%")->orWhere('takealot_orders.order_item_id', 'LIKE', "%{$search}%")->orWhere('takealot_orders.order_id', 'LIKE', "%{$search}%");
    
            })
            ->when($this->status,function($query, $status){
    
                $query->whereIn('takealot_orders.sale_status',$status);
    
            })
            ->when($this->active_shop_id,function($query, $active_shop_id){
    
                $query->where([['takealot_shops_id','=',$active_shop_id]]);
    
            });
            
            if(!empty($this->priceSort))
                {
                    $this->orderBy=($this->priceSort=='lowest') ? 'asc' : 'desc';
                    $orders=$orders->orderBy('takealot_orders.selling_price', $this->orderBy)->paginate($this->entries);
                }
                else
                {
                    $orders=$orders->orderBy('takealot_orders.order_date', $this->orderBy)->paginate($this->entries);
                }
        }        
        else
        {
            $orders=[];
        }
            
        return view('livewire.admin.seller.sales',['orders'=>$orders])->extends('admin.layouts.app');
    }
}

