<?php

namespace App\Http\Livewire\Front\Blog;

use App\Models\BlogCategories;
use App\Models\BlogComment;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Validator;
use Livewire\Component;

class SingleBlog extends Component
{
    public $slug;
    public function __construct()
    {
        $this->slug = Route::current()->parameter('slug');
    }
    public function NextRecord($id)
    {
        $iD         = Crypt::decrypt($id);
        $blog       = \App\Models\Blog::where('id', '>', $iD)->orderBy('id')->first();
        if (!empty($blog))
        {
            $this->slug = $blog->slug;
            $this->dispatchBrowserEvent('urlChange', ['url'=>route('post-single',$blog->slug)]);
        }
        else
        {
            $this->slug = $this->slug;
            $this->dispatchBrowserEvent('alert',[ 'type' => 'error', 'message' => "Next Record Not Found" ]);
        }
    }
    public function PreviousRecord($id)
    {
        $iD         = Crypt::decrypt($id);
        $blog       = \App\Models\Blog::where('id', '<', $iD)->orderBy('id','desc')->first();
        if (!empty($blog))
        {
            $this->slug = $blog->slug;
            $this->dispatchBrowserEvent('urlChange', ['url'=>route('post-single',$blog->slug)]);
        }
        else
        {
            $this->slug = $this->slug;
            $this->dispatchBrowserEvent('alert',[ 'type' => 'error', 'message' => "Previous Record Not Found" ]);
        }
    }
    public function create_comment($input)
    {
        Validator::make($input, [
                            'blog'         => 'required',
                            'user'         => 'required',
                            'comment'      => 'required',
                        ])->validate();
        $blog_comment          = new BlogComment();
        $blog_comment->user_id = Crypt::decrypt($input['user']);
        $blog_comment->blog_id = Crypt::decrypt($input['blog']);
        $blog_comment->comment = $input['comment'];
        $blog_comment->save();
        $this->dispatchBrowserEvent('alert',[ 'type' => 'success', 'message' => "Comment Created Successfully" ]);
    }
    public function render()
    {
        $user = '';
        if (Auth::check())
        {
            $user    = User::select('id','email')->where('id',Auth::user()->id)->first();
        }
        $blog        = \App\Models\Blog::where('slug',$this->slug)->first();
        $categories  = BlogCategories::orderBy('id', 'desc')->take(5)->get();
        $latest_blog = \App\Models\Blog::orderBy('id', 'desc')->take(6)->get();
        return view('livewire.front.blog.single-blog',compact('blog','user','latest_blog','categories'));
    }
}
