<?php

namespace App\Http\Livewire\Seller\Components\Dashboard;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotOrders;
use Illuminate\Support\Facades\DB;
use Livewire\WithPagination;
use Illuminate\Support\Facades\Validator;
use Carbon\Carbon;
use Auth;

class TopSellingOffers extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $filterByGraph; 
    public $readyToLoad    = false;
    public $readyToLoadRepricing    = false;
    public $readyToLoadTopSelling   = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $entries=30;
    public $search=null;  
    public function mount()
    {    
         $this->sellerID         = Auth::user()->id;        
         $this->entries  = 30;
         $this->filterByGraph=encrypt(30);
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    }
    public function updatedSearch()
    {
        $this->clearFilter(true);
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
       
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->orderSort='asc';
         $this->entries=30;

    }
    public function getTakealotOffersQueryProperty()
    {
        $thirtyDays=Carbon::now()->subDays(30)->toDateString();
        $takealotOrders=TakealotOffers::select('takealot_offers.offer_url','takealot_offers.stock_at_takealot','takealot_offers.leadtime_stock','takealot_offers.title','takealot_offers.sku','takealot_offers.tsin_id','takealot_offers.selling_price','takealot_offers.status',DB::raw('SUM(takealot_orders.quantity) as total_quantity'))
        ->join('takealot_orders', 'takealot_orders.offer_id', '=', 'takealot_offers.offer_id')
        ->when($this->search, function ($query,$search)
            {
               if(!empty($search))
               {
                $query->where('takealot_offers.title', 'LIKE', "%{$search}%")
                ->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")
                ->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%");
               }
            })        
        ->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID],['takealot_orders.order_date', '>=', $thirtyDays]])
        ->groupBy('takealot_offers.id')
        ->orderBy('total_quantity',$this->orderBy);
        return $takealotOrders;
    }
    public function getTakealotOffersProperty()
    {       
         return $this->TakealotOffersQuery->paginate($this->entries);
    }
    public function render()
    {
         $topSellingOffers=$this->readyToLoad ? $this->TakealotOffers : []; 
        //  $todayOffers=$this->TakealotOffers; 
        return view('livewire.seller.components.dashboard.top-selling-offers',compact('topSellingOffers'));
    }
}
