<?php

namespace App\Http\Livewire\Seller\Components\Graphs;

use Livewire\Component;
use App\Models\TakealotOrders;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Auth;
class SalesGraph extends Component
{
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $filterBy=null;
    public $graphData=null;
    
    public function mount($filterData)
    {  
         $this->sellerID         = Auth::user()->id;        
         $this->entries  = 30;
         $this->graphData  = null;
         $this->filterBy  = decrypt($filterData);
         
         $this->orderBy  = 'desc';
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
    }
    public function getTakealotSoldUnitsProperty()
    {   
        $daysData=null;
        $dateCondition='>=';        
        $data = [];
        if(empty($this->filterBy))
        {
            $daysData=30;
        }
        if((!empty($this->filterBy) && isset($this->filterBy)) && ($this->filterBy=="today" || $this->filterBy=="yesterday" || $this->filterBy==7 || $this->filterBy==15 || $this->filterBy==30 || $this->filterBy==90 || $this->filterBy==365))
        {
           
            if($this->filterBy=="today")
            {
                $daysData = Carbon::today()->toDateString();
                $dateCondition='=';
            }
            elseif($this->filterBy=="yesterday")
            {
                $daysData = Carbon::yesterday()->toDateString();
                $dateCondition='=';
            }
            else
            {
                $daysData = Carbon::now()->subDays($this->filterBy)->toDateString();
                $dateCondition='>=';
            }

            
        }
       
        $rows = TakealotOrders::selectRaw("UNIX_TIMESTAMP(DATE(order_date))*1000 as orderDateMs,DATE(order_date) as orderDate ,sum(quantity) as totalQuantity")
            ->where([
                ['takealot_shops_id', '=', $this->active_shop_id],
                ['seller_id', '=', $this->sellerID]
            ]);
            if(!empty($daysData) && isset($daysData))
            {
                $rows=$rows->whereDate('order_date',$dateCondition,$daysData); 
            }
            if((!empty($this->filterBy)) && $this->filterBy=='lm')
            {
                $rows=$rows->whereBetween(DB::raw('DATE(order_date)'), [Carbon::now()->subMonth()->startOfMonth()->toDateString(), Carbon::now()->subMonth()->endOfMonth()->toDateString()]); 
            }
            if((!empty($this->filterBy)) && $this->filterBy=='ly')
            {
                $rows=$rows->whereBetween(DB::raw('DATE(order_date)'), [Carbon::now()->subYear()->startOfYear()->toDateString(), Carbon::now()->subYear()->endOfYear()->toDateString()]); 
            }
            if((!empty($this->filterBy)) && $this->filterBy=='wtd')
            {
                $rows=$rows->whereBetween(DB::raw('DATE(order_date)'), [Carbon::now()->startOfWeek(),Carbon::today()]); 
            }
            if((!empty($this->filterBy)) && $this->filterBy=='mtd')
            {
                $rows=$rows->whereBetween(DB::raw('DATE(order_date)'), [Carbon::now()->startOfMonth(),Carbon::today()]); 
            }
            if((!empty($this->filterBy)) && $this->filterBy=='ytd')
            {
                $rows=$rows->whereBetween(DB::raw('DATE(order_date)'), [Carbon::now()->startOfYear(),Carbon::today()]); 
            }
            $rows=$rows->groupBy('orderDate')->orderBy('order_date','asc')->get(); 
           
            foreach ($rows as $row) {
               
                $data[] = [
                     $row->orderDateMs,
                     (int)$row->totalQuantity
                ];
            }
        // $json = json_encode($data);
        return $data;
       
    }
    public function graphDraw($data)
    {
        return $graphData=[
            'chart' => [
                'type' => 'column',
                'alignTicks'=> true,
                'zoomType' => 'x'
            ],
            'title' => [
                'text' => ''
            ],            
            'xAxis' => [
                'type' => 'datetime',
                // 'tickInterval' => 24 * 3600 * 1000
    
            ],
            'yAxis' => [
                'title' => [
                    'text' => ''
                ]
            ],           
            'series' => [
                [
                    'name' => 'Units',
                    'data' => $data,
                    'dataGrouping' => [    
                        'units' => [        
                            ['day', [1]],
                            ['week', [1]],
                            ['month', [1, 2, 3, 4, 6]]
                        ]
                    ],                   
                    'color' => '#7367F0', // Change the color of the chart                   
                ]
            ]
        ];
    }
    public function render()
    {
        $this->graphData=$this->TakealotSoldUnits;
        return view('livewire.seller.components.graphs.sales-graph');
    }
}
