<?php

namespace App\Http\Livewire\Seller\Configurations\Settings;

use Livewire\Component;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\HandlerStack;
use Illuminate\Support\Facades\Validator;
use App\Models\TakealotShops;
use App\Models\Subscriptions;
use App\Models\SubscriptionsRenew;
use App\Jobs\FetchAllOffers;
use App\Jobs\TakealotFatchAllSalesJob;
use Illuminate\Support\Str;
use Carbon\Carbon;
use Auth;
class NewAccountConfig extends Component
{
    public $sellerID=null;
    public $active_shop_id=null;
    public $selectedTab=null;
    public $xlimit=1000;
    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->selectedTab='api';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
        $checkUUID=Auth::user()->AllTakealotShops->where('id','=',$this->active_shop_id)->first();
     
        if(!empty($checkUUID) && empty($checkUUID->webhook_uuid))
        {
            $checkUUID->webhook_uuid= (string) Str::uuid();
            $checkUUID->save();
        }

        $this->checkConfig();
    }
    public function updateTakealotApi($input)
    {
        $lastUpdated=null;
        Validator::make($input, [          
            'takealot_api_key'   => 'required|min:3|max:1000',            
        ])->validate();
         $result=$this->sellerDetailsApi($input['takealot_api_key']);
         if(!empty(currentShop($this->active_shop_id)) && !empty($result) && isset($result->seller_id) && isset($result->display_name) && isset($result->slug))
         {
                $shopData=TakealotShops::where([['slug','=',$result->slug],['tk_seller_id','=',$result->seller_id]])->first();
                if(empty($shopData) && !isset($shopData->id))
                {
                    $shopData=TakealotShops::where('id','=',$this->active_shop_id)->first();                   
                }

                //checking
                if((!empty($shopData) && !empty($result) && $shopData->id==$this->active_shop_id) && (($result->seller_id==$shopData->tk_seller_id) || ((empty($shopData->tk_seller_id) && empty($shopData->slug)) || $shopData->tk_seller_id==0)))
                {
                    
                    if(isset($result->seller_id) && isset($result->display_name))
                    {
                        $this->updateConfigApiKey($input['takealot_api_key'],$result,$shopData);
                        
                        $this->selectedTab='webhook';
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Something Went Wrong','move'=>0]);
                    }
                }
                else
                {
                    $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! This Takealot account is already registered with another email','move'=>0]);
                }
            
             
         }
        
     
    }
    public function updateConfigApiKey($takealot_api_key,$result,$shopData)
    {
        $shopData->takealot_api_key= trim($takealot_api_key);
        $shopData->tk_seller_id= $result->seller_id;
        $shopData->takealot_account_name= $result->display_name;
        $shopData->slug= $result->slug;     
        $shopData->status= 'active';     
        $shopData->seller_details= json_encode($result);
        if($shopData->save())
        {
            $numberOfDays=(!empty(get_setting('trial_days')) ? get_setting('trial_days') : 15);                              
            FetchAllOffers::dispatch($shopData->id)->onQueue('high')->beforeCommit();
            TakealotFatchAllSalesJob::dispatch($shopData->id)->onQueue('low');   
                   
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Api key updated successfully.Your offers and sales will be updated soon on Salesup','move'=>0]);    
        }
        else
        {
             $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Something Went Wrong','move'=>0]);
        }
    }
    public function sellerDetailsApi($apiKey)
    {
      
        try
        {
            $headers = ['Authorization'=>'Key '.$apiKey,'x-RateLimit-Limit'=>1000,'x-RateLimit-Remaining'=>$this->xlimit];
            $client  = new Client(['base_uri' => takealotBaseUrl(),'verify' => false]);
            $request = new Request('GET', 'seller',$headers);
            $response = $client->send($request);
            $result=json_decode($response->getBody()->getContents());
            return  $result;
        }
        catch (\Exception $e) 
        {
            $response = $e->getResponse();
            $statusCode = $response->getStatusCode();
            $errorMessage = $response->getBody()->getContents();
            $errorMessage=json_decode($errorMessage);
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>$statusCode.' '.$errorMessage->message,'move'=>0]);

         }
         return  null;
    }
    public function updateTakealotApiWebHook($input)
    {
        Validator::make($input, [
            'webhook_secret_key' => 'required|min:3|max:2000',             
        ])->validate();
       
        $takealotApi=TakealotShops::where([['id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->first();       
        $takealotApi->takealot_webhook_secreet = $input['webhook_secret_key'];
        if(!empty($takealotApi))
        {
            if($takealotApi->save())
            {
                $this->checkConfig();
                $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Webhook Updated Successfully','move'=>0]);
                
            }
            else
            {
                this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Something Went Wrong','move'=>0]);
            }
        }
        else
        {
            this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Something Went Wrong','move'=>0]);
        }
        
        
    }
    function checkConfig()
    {
        $checkShop=TakealotShops::where('id','=',$this->active_shop_id)->first();
        if(!empty($checkShop))
        {
           if(!empty($checkShop->tk_seller_id) && !empty($checkShop->takealot_account_name) && !empty($checkShop->takealot_account_name) && !empty($checkShop->takealot_api_key) && !empty($checkShop->takealot_webhook_secreet))
           {
              return redirect()->route('seller.dashboard');
           }
        }
    }
    public function render()
    { 
        $this->checkConfig();
        $apiData=  TakealotShops::where([['id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->first();
        if(!empty($apiData->takealot_api_key))
        {
            $this->selectedTab='webhook';
        }
        return view('livewire.seller.configurations.settings.new-account-config',['apiData'=>$apiData])->extends('seller.layouts.app');
    }
}
