<?php

namespace App\Http\Livewire\Seller\Dashboard;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotOrders;
use Illuminate\Support\Facades\DB;
use Livewire\WithPagination;
use Illuminate\Support\Facades\Validator;
use Carbon\Carbon;
use Auth;
class Dashboard extends Component
{
    public $filterByGraph; 
    public $readyToLoadRepricing    = false;
    public $readyToLoadTopSelling   = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $entries=30;
    public $search=null;    
       
    public function mount()
    {    
         $this->sellerID         = Auth::user()->id;        
         $this->entries  = 30;
         $this->filterByGraph=encrypt(30);
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    }
    public function readyToLoadRepricing()
    {
        $this->readyToLoadRepricing = true;
    } 
    public function readyToLoadTopSelling()
    {
        $this->readyToLoadTopSelling = true;
    } 
    public function getTakealotTodayTotalOrdersProperty()
    {       
        $oneDay=Carbon::now()->toDateString();
        $takealotOrders=TakealotOrders::where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->whereDate('order_date','=',$oneDay)->sum('quantity');
         return $takealotOrders;
    }
    
    public function render()
    {
        //Total Buyable offers
        $offersCount=TakealotOffers::where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID],['status','=','Buyable']])->count();
        //Total in repricing
        $repricingCount=TakealotRePricing::where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->count();
        //Active in repricing
        $repricingActive=TakealotRePricing::where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID],['min_price','>',0],['max_price','>',0],['status','=',1]])->count();
        //In active in repricing
        $repricingInActive=TakealotRePricing::where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->where('status','=',0)->count();
        //Buyable winner         
        $repricingWining=TakealotRePricing::with('offer')
        ->join('takealot_offers', function($query)
        {
           
            $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
            ->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID],['takealot_offers.status','=','Buyable']]);
            
        })->where([['takealot_repricing.takealot_shops_id','=',$this->active_shop_id],['takealot_repricing.seller_id','=',$this->sellerID],['takealot_repricing.min_price','>',0],['takealot_repricing.max_price','>',0],['takealot_repricing.status','=',1],['takealot_repricing.winner','=',1]])->count();
       
        //Buyable losser      
        $repricingLossing=TakealotRePricing::with('offer')
        ->join('takealot_offers', function($query)
        {
            $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
            ->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID],['takealot_offers.status','=','Buyable']]);
            
        })->where([['takealot_repricing.takealot_shops_id','=',$this->active_shop_id],['takealot_repricing.seller_id','=',$this->sellerID],['takealot_repricing.min_price','>',0],['takealot_repricing.max_price','>',0],['takealot_repricing.status','=',1],['takealot_repricing.winner','=',0]])->count();
       //30 days sale
        $thirtyDays=Carbon::now()->subDays(30)->toDateString();
        $thirtyDaysTotal = DB::table('takealot_orders') 
       ->whereNotIn('sale_status', ['Cancelled by Customer','Cancelled by Takealot - DC Stock Inquiry','Cancelled - Inbound Exception (Short Supplied)','Cancelled - Late','Returned'])
       ->where(function($query) use ($thirtyDays) {
            $query->whereDate('order_date','>=',$thirtyDays)
            ->where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]]);
        })
        ->sum('selling_price');
        //15 days sale
        $fifteenDays=Carbon::now()->subDays(15)->toDateString();
        $fifteenDaysTotal = DB::table('takealot_orders') 
        ->whereNotIn('sale_status', ['Cancelled by Customer','Cancelled by Takealot - DC Stock Inquiry','Cancelled - Inbound Exception (Short Supplied)','Cancelled - Late','Returned'])
        ->where(function($query) use ($fifteenDays) {
             $query->whereDate('order_date','>=',$fifteenDays)
             ->where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]]);
         })
         ->sum('selling_price');
        //7 days sale
        $sevenDays=Carbon::now()->subDays(7)->toDateString();
        $sevenDaysTotal = DB::table('takealot_orders') 
        ->whereNotIn('sale_status', ['Cancelled by Customer','Cancelled by Takealot - DC Stock Inquiry','Cancelled - Inbound Exception (Short Supplied)','Cancelled - Late','Returned'])
        ->where(function($query) use ($sevenDays) {
             $query->whereDate('order_date','>=',$sevenDays)
             ->where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]]);
         })
         ->sum('selling_price');
        //Sold in one month
      
        $monthsoldTotal = DB::table('takealot_orders') 
        ->whereNotIn('sale_status', ['Cancelled by Customer','Cancelled by Takealot - DC Stock Inquiry','Cancelled - Inbound Exception (Short Supplied)','Cancelled - Late','Returned'])
        ->where(function($query) use ($thirtyDays) {
             $query->whereDate('order_date','>=',$thirtyDays)
             ->where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]]);
         })
         ->sum('quantity');
        //Returned in one month
        $monthsoldTotalReturned = DB::table('takealot_orders')
            ->whereDate('order_date','>=',$thirtyDays)->where([['sale_status','=','Returned'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])
            ->sum('quantity');
        //Cancelled in one month
        $monthsoldTotalCancelled = DB::table('takealot_orders')
                ->whereDate('order_date','>=',$thirtyDays)->where([['sale_status','=','Cancelled by Customer'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])
                ->sum('quantity');
        //Cancelled Late in one month
        $monthsoldTotalCancelledLate = DB::table('takealot_orders')
                ->whereDate('order_date','>=',$thirtyDays)->where([['sale_status','=','Cancelled - Late'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])
                ->sum('quantity');                
        
        //Total today Orders        
        $totalTodayOrders=$this->TakealotTodayTotalOrders;        
        return view('livewire.seller.dashboard.dashboard',['offersCount'=>$offersCount,'repricingCount'=>$repricingCount,'repricingActive'=>$repricingActive,'repricingInActive'=>$repricingInActive,'repricingWining'=>$repricingWining,'repricingLossing'=>$repricingLossing,'thirtyDaysTotal'=>$thirtyDaysTotal,'fifteenDaysTotal'=>$fifteenDaysTotal,'sevenDaysTotal'=>$sevenDaysTotal,'monthsoldTotal'=>$monthsoldTotal,'monthsoldTotalReturned'=>$monthsoldTotalReturned,'monthsoldTotalCancelled'=>$monthsoldTotalCancelled,'monthsoldTotalCancelledLate'=>$monthsoldTotalCancelledLate,'totalTodayOrders'=>$totalTodayOrders])->extends('seller.layouts.app');
    }
}
