<?php

namespace App\Http\Livewire\Seller\Inventory;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotOrders;
use App\Models\TakealotRePricing;
use Livewire\WithPagination;
use Auth;
class Index extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];    
    public $stockFilter=null;
    public $buybox=null;
    public $rePricingFilter=null;

    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
    public function totalSold($tsin,$sku)
    {
        $totalSold=TakealotOrders::where([['tsin','=',$tsin],['sku','=',$sku]])->sum('quantity');
        return $totalSold;
    }
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingFilter=null;
         $this->stockFilter=null;
    }
    public function render()
    {
        $takealotOffers =TakealotOffers::with('rePricing','takealotOfferDetails')      
        ->when($this->buybox,function($query, $buybox){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->whereHas('rePricing',fn($query) => $query->where('takealot_repricing.winner', $buybox)
           );
           })
        ->when($this->rePricingFilter,function($query, $rePricingFilter){   
              if($rePricingFilter=='yes')
              {

                $query->whereHas('rePricing',fn($query) => $query->where('takealot_repricing.winner','=',1)->orWhere('takealot_repricing.winner','=',0));

              }
              else
              {
                $query->doesntHave('rePricing');
              }
     
           })
        ->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID]])
        ->when($this->search,function($query, $search){
            $search=trim($search);
            $query->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%");

            })
        ->when($this->status,function($query, $status){

             $query->whereIn('takealot_offers.status',$status);

            }) 
        ->when($this->stockFilter,function($query, $stockFilter){

                if($stockFilter=='leadtime')
               {
                   $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) > 0");
               }
               if($stockFilter=='instock')
               {
                   $query->where('takealot_offers.stock_at_takealot_total','>',0);
               }
               if($stockFilter=='outofstock')
               {
                   $query->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) <= 0")->where('takealot_offers.stock_at_takealot_total','<=',0);
               }
   
               })
            ;

            if(!empty($this->priceSort))
            {
                $this->orderBy=($this->priceSort=='lowest') ? 'asc' : 'desc';
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.selling_price', $this->orderBy)->paginate($this->entries);
            }
            else
            {
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.date_created', $this->orderBy)->paginate($this->entries);
            }
        return view('livewire.seller.inventory.index',['takealotOffers'=>$takealotOffers]);
    }
}
