<?php
namespace App\Http\Livewire\Seller\Live;
use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotCompetitionHistory;
use App\Models\TakealotShops;
// use Livewire\WithPagination;
use Auth;
use App\Jobs\TakealotInstantRepricingJob;
use App\Http\Controllers\Seller\OffersController;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Pool;
use GuzzleHttp\Promise;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\HandlerStack;
class Offers extends Component
{
    // use WithPagination;
    // protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';
     
    public $page_no;    
    public $page_size;    
    public $search=null;
    public $total_results;
    public $takealotLive;
    protected $listeners = ['updatetakealotOffer','repricedata','pricingdata'];

    public $rateLimit=0;
    public $sortKey;
    public $sortDir;
    
    public function __construct()
    {
        $this->rateLimit=get_setting('takealot_ratelimit');        
    }

    public function mount()
    {
         $this->sellerID      = Auth::user()->id;
         $this->total_results = 0;
         $this->sortKey       = 'offer_id';
         $this->sortDir       = 'desc';
         $this->takealotLive  = [];
 
         

         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id

        $this->rePricingFilter='no';
        $this->shop=TakealotShops::where([['id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->first();
        $this->page_no=1;    
        $this->page_size=100;    
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
   
    // public function rateLimit($shop,$defaultRateLimit)
    // {
    //     if($shop->api_rate_limit<=0)
    //     {
            
            
    //         $shop->api_rate_limit=1000;
    //         $shop->save();
    //         sleep(20);
    //         $ratelimit=$shop->api_rate_limit;
    //         return $ratelimit;
    //     }
    //     else
    //     {
    //         $shop->api_rate_limit=$shop->api_rate_limit-1;
    //         $shop->save();
    //         $ratelimit=$shop->api_rate_limit;
    //         return $ratelimit;
    //     }
    // }
    public function offersCurl()
    {
            
            $shopDetails  = $this->shop;
            $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
            $fullUrl='offers?page_number='.$this->page_no.'&page_size='.$this->page_size.'&sort_key='.$this->sortKey.'&sort_dir='.$this->sortDir;
            $headers = ['Authorization'=>'Key '.$shopDetails->takealot_api_key,'x-RateLimit-Limit'=>1000,'x-RateLimit-Remaining'=>$ratelimit];
            $client  = new Client(['base_uri' => takealotBaseUrl(),'verify' => false]);
            $request = new Request('GET',$fullUrl ,$headers);
            // $response = $client->send($request);
            // $result  = json_decode($response->getBody()->getContents()); 
         
            
            $pageSize     = $this->page_size;
            $pageNumber  = $this->page_no;

            $promise = $client->sendAsync($request)->then(function ($response) use ($shopDetails,$pageSize,$pageNumber) {
                $result  = json_decode($response->getBody()->getContents()); 
                
                if(!empty($result) && (isset($result->offers) && !empty($result->offers) && count($result->offers)>0))
                { 
                    if(!empty($result->offers))
                    {
                        $this->takealotLive=$result->offers;
                        $this->total_results=ceil($result->total_results/100);
                    }
                }

            });
        
            $promise->wait();
            
    }
    public function searchOffer()
    {
        $searchBy=$this->search;
        $searchOffer = TakealotOffers::where([
            ['takealot_offers.takealot_shops_id', '=', $this->active_shop_id],
            ['takealot_offers.seller_id', '=', $this->sellerID]
        ])->where(function($query) use ($searchBy) {
            $query->orWhere('takealot_offers.title', 'LIKE', "%{$searchBy}%")
            ->orWhere('takealot_offers.sku', 'LIKE', "%{$searchBy}%")
            ->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$searchBy}%");
        })->first();

        if(!empty($searchOffer))
        {
            $shopDetails  = $this->shop;
            $ratelimit=shopRateLimitChecker($shopDetails,$this->rateLimit);
            $headers = ['Authorization'=>'Key '.$shopDetails->takealot_api_key,'x-RateLimit-Limit'=>1000,'x-RateLimit-Remaining'=>$ratelimit];
            $client  = new Client(['base_uri' => takealotBaseUrl(),'verify' => false]);
            $request = new Request('GET', 'offers/offer/ID'.$searchOffer->offer_id,$headers);
            $response = $client->send($request);
            $result  = json_decode($response->getBody()->getContents()); 
            if(!empty($result))
            {
                $this->page_no=1;
                $this->total_results=1;
                $this->takealotLive=[
                    0=>$result
                ];
            }
            else
            {
                $this->page_no=1;
                $this->total_results=0;
                $this->takealotLive=[];
            }
        }
        
    }
    public function nextPage()
    {
        if($this->page_no>0 && empty($this->search))
        {
            $this->page_no=$this->page_no+1;
        }
    }
    public function previousPage()
    {
        if($this->page_no>1 && empty($this->search))
        {
            $this->page_no=$this->page_no-1;
        }
    }
    public function render()
    {
        $this->takealotLive=[];

        if(!empty($this->search) && isset($this->search))
        {
            $this->searchOffer();
        }
        else
        {
            $this->offersCurl();
        }
        
        return view('livewire.seller.live.offers');
    }
}


