<?php

namespace App\Http\Livewire\Seller\Offers;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotCompetitionHistory;
use Livewire\WithPagination;
use Auth;
use App\Jobs\TakealotInstantRepricingJob;
use App\Http\Controllers\Seller\OffersController;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
use App\Http\Controllers\Seller\ManualRePricingController;

class Index extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';
    
    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];
    public $buybox=null;
    public $rePricingFilter=null;
    public $stockFilter=null;
    public $editSelling=null;
    public $editofferCost=null;
    public $editTakealotCommission=null;
    public $editOtherCost=null;
    public $editTotalCost=null;
    public $rePricingDetails=null;
    public $pricesDetails=null;
    public $pricingdataID=null;
    protected $listeners = ['updatetakealotOffer','repricedata','pricingdata'];
    
    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         } 
         $this->active_shop_id=session('active_shop_id');
        //end active shop id

        $this->rePricingFilter='no';
    }
    public function repricedata($repriceID)
    {      
       $this->rePricingDetails=TakealotOffers::with('rePricing')->where([['id','=',decrypt($repriceID)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
       $this->dispatchBrowserEvent('popupnew',['id'=>'updateRow']);
    }
    public function pricingdata($repriceID)
    {      
       $this->pricingdataID=$repriceID;
       $this->pricesDetails=TakealotOffers::with('rePricing')->where([['id','=',decrypt($repriceID)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
       $this->dispatchBrowserEvent('popupnew',['id'=>'updatePricesRow']);
      
    }
    public function closeModel()
    {
        $this->rePricingDetails=[];
        $this->pricesDetails=[];        
        $this->pricingdataID=null;
        $this->resetVariables();
    }
        
    public function resetVariables()
    {
        $this->editMin=null;
        $this->editMax=null;
        $this->editSelling=null;
        $this->editofferCost=null;
        $this->editTakealotCommission=null;
        $this->editOtherCost=null;
        $this->editTotalCost=null;
        $this->OfferStatus=null;        
        $this->pricingdataID=null;
    }
    public function offerCostUpdate($input)
    {
        Validator::make($input, [
            'offer_cost'  => 'required|min:1|max:99999999',             
            'updateFor'  => 'required',             
        ])->validate();
        //update min price

        if($input['offer_cost']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->offer_cost=$input['offer_cost'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->offer_cost=$updatePrice->offer_cost;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Product cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function sellingPriceUpdate($input)
    {
          
        Validator::make($input, [
            'selling_price'  => 'required|min:1|max:99999999',  
            'updateFor'  => 'required',              
        ])->validate();
        if($input['selling_price']>0)
        {
          
            $updatePrice=TakealotOffers::with('takealotShops')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->selling_price=$input['selling_price'];
                if($updatePrice->save())
                {
                    $this->pricesDetails->selling_price=$updatePrice->selling_price;
                    $updateOfferPrice=new OffersController();
                    $result=$updateOfferPrice->updateSellingPrice($updatePrice->takealotShops,$updatePrice->offer_id,$updatePrice->selling_price);
                    if($result==true)
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Selling price updated successfully','move'=>0]);
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong'.$result,'move'=>0]);
                    }
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function takealotCommission($input)
    {
        Validator::make($input, [
            'takealot_commission'  => 'required|min:1|max:99999999', 
            'updateFor'  => 'required',               
        ])->validate();
        //update min price

        if($input['takealot_commission']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->takealot_commission=$input['takealot_commission'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->takealot_commission=$updatePrice->takealot_commission;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Takealot commission updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function otherCost($input)
    {
        Validator::make($input, [
            'other_costs'  => 'required|min:1|max:99999999',     
            'updateFor'  => 'required',           
        ])->validate();
        //update min price

        if($input['other_costs']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->other_costs=$input['other_costs'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->other_costs=$updatePrice->other_costs;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Other cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
    public function updatedStockFilter()
    {
        $this->resetPage();
    }
    public function addToRePricing($id)
    {
            $checkAlready=TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['takealot_offers_id','=',decrypt($id)]])->first();
            if(empty($checkAlready))
            {
                $rePricing= new TakealotRePricing();
                $rePricing->seller_id=$this->sellerID;
                $rePricing->takealot_shops_id=$this->active_shop_id;
                $rePricing->takealot_offers_id=decrypt($id);
                $rePricing->min_price=0;
                $rePricing->max_price=0;
                if($rePricing->save())
                {
                    // $repricing=new RePricingController();
                    // $repricing->repriceVariantParams($rePricing->id,'menaul');
                    // $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Successfully Added In Re-Pricing List','move'=>0]);
                    $repricing=new ManualRePricingController();        
                    $response=$repricing->repriceVariantParams($rePricing->id);        
                    if(!empty($response))
                    {
                        $responseData = json_decode($response->getContent(), true);
                        // Access specific data from the array
                        if(!empty($responseData) && $responseData['status'] && $responseData['message'])
                        {
                            $status = $responseData['status'];
                            $message = $responseData['message'];
                            $this->dispatchBrowserEvent('notify', ['error' => $status,'msg'=> $message,'move'=>0]);               
            
                        }
                        else
                        {
                            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                        }
                        
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                    }
                }
     
            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Already Exist in Re-Pricing','move'=>0]);

            }
           
         

    }
    public function removeFromRePricing($id)
    {
        if(TakealotRePricing::where([['takealot_offers_id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Successfully Removed From Re-Pricing List','move'=>0]);
        }
       
    }
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingFilter=null;
         $this->stockFilter=null;
         $this->resetPage();

    }
    public function addAllBuyableToRePricing()
    {
        $allBuyable=TakealotOffers::doesntHave('rePricing')->where([['status','=','Buyable'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->get();  
        if(!empty($allBuyable))
        {
            foreach($allBuyable as $buyable)
            {
                $checkAlready=TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['takealot_offers_id','=',$buyable->id]])->first();
                if(empty($checkAlready))
                {
                    $rePricing= new TakealotRePricing();
                    $rePricing->seller_id=$this->sellerID;
                    $rePricing->takealot_shops_id=$this->active_shop_id;
                    $rePricing->takealot_offers_id= $buyable->id;
                    $rePricing->min_price=0;
                    $rePricing->max_price=0;
                    if($rePricing->save())
                    {
                       // $repricing=new RePricingController();
                        //$repricing->repriceVariantParams($rePricing->id);
                       // TakealotInstantRepricingJob::dispatch($rePricing->id)->onQueue('high');
                    }
         
                }
            }
        }
    }
    public function removeAllInActiveFromReprcing()
    {
        if(TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['status','=',0]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Removed All In-Active Offers','move'=>0]);
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Something went wrong','move'=>0]);
        }
    }
    public function updateOfferDetails($input)
    {
        Validator::make($input, [
            'minPrice'  => 'required|min:1|max:9999999',             
            'maxPrice'  => 'required|min:1|max:9999999',             
            'my_soh'    => 'required|min:1|max:9999999',             
            'updateFor' => 'required',             
        ])->validate();
        
        if(empty($input['my_soh']))
        {
            $input['my_soh']=0;
        }
        if(($input['minPrice']>0 && $input['maxPrice']>0) && $input['minPrice']<$input['maxPrice'])
        {
            $updatePrice=TakealotRePricing::with('offer','takealotShopDetails')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                // $oldData=repricingMap($updatePrice);
                $oldOfferData=offerMap($updatePrice->offer);
                $updatePrice->min_price=$input['minPrice'];
                $updatePrice->max_price=$input['maxPrice'];
                $updatePrice->ip=getUserIp();
                $updatePrice->updated_through=1;
                $updatePrice->status=1;
                if($updatePrice->save())
                {
                   
                    $newData=repricingMap($updatePrice,1);                    
                    if($updatePrice->status==1)
                    {                  
                       
                            $leadtime_decode=json_decode($updatePrice->offer->leadtime_stock);
                          
                            if(!empty($leadtime_decode[0]) && ($input['my_soh']>$leadtime_decode[0]->quantity_available || $input['my_soh']<$leadtime_decode[0]->quantity_available))
                            {
                                
                                $leadtime_decode[0]->quantity_available=$input['my_soh'];
                                $leadtime_encode=json_encode($leadtime_decode);
                                $updatePrice->offer->leadtime_stock=$leadtime_encode;
                                $updatePrice->offer->ip=getUserIp();
                                if($updatePrice->offer->save())
                                { 
                                    $newOfferData=offerMap($updatePrice->offer,1);
                                }
                                
                               
                            }
                        
                        

                        if($updatePrice->offer->selling_price<$updatePrice->min_price)
                        {
                            //start update new price
                              $updatePrice->offer->selling_price=$updatePrice->min_price;                              
                              if($updatePrice->offer->save())
                              {

                                // $updateOfferPrice=new OffersController();
                                $updateOfferPrice=new ManualRePricingController();
                                $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->min_price,$updatePrice->takealotShopDetails,$updatePrice); 
                             
                              }
                            //end update new price 
                        }
                        else
                        {
                            //start update new price
                            $updatePrice->offer->selling_price= $updatePrice->offer->selling_price;                              
                            if($updatePrice->offer->save())
                            {

                            //   $updateOfferPrice=new OffersController();
                              $updateOfferPrice=new ManualRePricingController();
                              $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->offer->selling_price,$updatePrice->takealotShopDetails,$updatePrice); 
                           
                            }
                          //end update new price 
                        }
                       
                        

                        // $repricing=new RePricingController();
                        // $repricing->repriceVariantParams(decrypt($input['updateFor']),'manual');
                        // $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Price Updated Successfully','move'=>0]);
                        
                        $repricing=new ManualRePricingController();        
                        $response=$repricing->repriceVariantParams(decrypt($input['updateFor']));        
                        if(!empty($response))
                        {
                            $responseData = json_decode($response->getContent(), true);
                            // Access specific data from the array
                            if(!empty($responseData) && $responseData['status'] && $responseData['message'])
                            {
                                $status = $responseData['status'];
                                $message = $responseData['message'];
                                $this->dispatchBrowserEvent('notify', ['error' => $status,'msg'=> $message,'move'=>0]);               
                
                            }
                            else
                            {
                                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                            }
                            
                        }
                        else
                        {
                            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                        }



                    }  
                    else
                    {

                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                    }
                }
                else
                {
                    $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                }
                
            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

            }
            
        }
        else       
        {
            
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Min and max price must be greater than 0','move'=>0]);

        }
         
        
    }
    public function render()
    {
        $activeShopID=$this->active_shop_id;
        $activeSellerID=$this->sellerID;

            $takealotOffers = TakealotOffers::leftJoin('takealot_repricing', 'takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
            ->select('takealot_offers.*','takealot_repricing.id as re_id','takealot_repricing.takealot_offers_id as re_takealot_offers_id','takealot_repricing.min_price as re_min_price','takealot_repricing.max_price as re_max_price','takealot_repricing.status as re_status','takealot_repricing.stock_status as re_stock_status','takealot_repricing.leadtime_status as re_leadtime_status','takealot_repricing.winner as re_winner')
            ->when($this->buybox,function($query, $buybox) use ($activeShopID,$activeSellerID){
                $buybox=($buybox=='winner') ? 1 : 0;
                $query->whereHas('rePricing',fn($query) => $query->where([['takealot_repricing.takealot_shops_id','=',$activeShopID],['takealot_repricing.seller_id','=',$activeSellerID]])->where('takealot_repricing.winner', $buybox)->where('takealot_repricing.status','=',1)
               );
               })
               ->when($this->rePricingFilter,function($query, $rePricingFilter) use ($activeShopID,$activeSellerID){   
                if($rePricingFilter=='yes')
                {
                  $query->whereHas('rePricing',fn($query) => $query->where([['takealot_repricing.takealot_shops_id','=',$activeShopID],['takealot_repricing.seller_id','=',$activeSellerID]])->where('takealot_repricing.winner','=',1)->orWhere('takealot_repricing.winner','=',0));            //   }
                }
                  else
                {
                  $query->doesntHave('rePricing');
                }
       
             })
            ->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID]])
            ->when($this->search,function($query, $search) use ($activeShopID,$activeSellerID){
                $search=trim($search);
                $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%");
    
                })
            ->when($this->status,function($query, $status) use ($activeShopID,$activeSellerID){
    
                 $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereIn('takealot_offers.status',$status);
    
                })
                ->when($this->stockFilter,function($query, $stockFilter) use ($activeShopID,$activeSellerID){
    
                 if($stockFilter=='leadtime')
                {
                    $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) > 0");
                }
                if($stockFilter=='instock')
                {
                    $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->where('takealot_offers.stock_at_takealot_total','>',0);
                }
                if($stockFilter=='outofstock')
                {
                    $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) <= 0")->where('takealot_offers.stock_at_takealot_total','<=',0);
                }
    
                })->where([['takealot_offers.takealot_shops_id','=', $this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID],['takealot_offers.title','!=',null]]);   
        
      
        
            if(!empty($this->priceSort))
            {
                $this->orderBy=($this->priceSort=='lowest') ? 'asc' : 'desc';
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.selling_price', $this->orderBy)->paginate($this->entries);
            }
            else
            {
                $takealotOffers=$takealotOffers->orderBy('takealot_offers.date_created', $this->orderBy)->paginate($this->entries);
            }
            if(!empty($this->pricesDetails) && !empty($this->pricingdataID))
            {
             $this->pricingdata($this->pricingdataID);
            }
        $this->dispatchBrowserEvent('reloadSelect2');
        return view('livewire.seller.offers.index',['takealotOffers'=>$takealotOffers]);
    }
}
