<?php

namespace App\Http\Livewire\Seller\Profile;

use Livewire\Component;
use App\Models\User;
use App\Models\UserDetails;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Intervention\Image\Facades\Image;
use Livewire\WithFileUploads;
use App\Models\Upload;
use Response;
use Storage;
class UserInfo extends Component
{
    use WithFileUploads;
    public $image,$imageError;
    public function updatedImage()
    {
        $type = array(
            "jpg"=>"image",
            "jpeg"=>"image",
            "png"=>"image",
            // "svg"=>"image",
            // "webp"=>"image",
            // "gif"=>"image",
        );
        $upload = new Upload;
        $extension = strtolower($this->image->getClientOriginalExtension());
        if(isset($type[$extension])){
            $upload->file_original_name = null;
            $arr = explode('.', $this->image->getClientOriginalName());
            for($i=0; $i < count($arr)-1; $i++){
                if($i == 0){
                    $upload->file_original_name .= $arr[$i];
                }
                else{
                    $upload->file_original_name .= ".".$arr[$i];
                }
            }
            $path = $this->image->store('uploads/all', 'local');
            $size = $this->image->getSize();
            if($type[$extension] == 'image'){
                try {
                    $img = Image::make($this->image->getRealPath())->encode();
                    $height = $img->height();
                    $width = $img->width();
                    if($width > $height && $width > 1500){
                        $img->resize(1500, null, function ($constraint) {
                            $constraint->aspectRatio();
                        });
                    }elseif ($width > 800) {
                        $img->resize(800, null, function ($constraint) {
                            $constraint->aspectRatio();
                        });
                    }
                    $img->save(base_path('public/').$path);
                    clearstatcache();
                    $size = $img->filesize();

                    if (env('FILESYSTEM_DRIVER') == 's3') {
                        Storage::disk('s3')->put($path, file_get_contents(base_path('public/').$path));
                        unlink(base_path('public/').$path);
                    }
                } catch (\Exception $e) {
                }
            }

            $upload->extension = $extension;
            $upload->file_name = $path;
            $upload->user_id = Auth::user()->id;
            $upload->type = $type[$upload->extension];
            $upload->file_size = $size;
            $upload->save();
            $userDetail= UserDetails::where('user_id',Auth::id())->first();
            if (!empty($userDetail)) {
                $userDetail->profile_image = $upload->id;
                if ($userDetail->save()) {
                    $this->successMessage = false;
                    $this->dispatchBrowserEvent('swal:success',[ 'type'=>'success','msg'=>'Profile Image Updated successfully' ]);
                } else {
                    $this->dispatchBrowserEvent('swal:success',[ 'type'=>'error','msg'=>'Something Went Wrong Please try again' ]);
                }
            }else{
          
                $this->dispatchBrowserEvent('swal:success',[ 'type'=>'error','msg'=>'Image Not Updated Something Went Wrong Please try again' ]);

            }

        }
    }
    public function delete_image($img_id)
    {
        $userDetail   = UserDetailss::where('user_id',Auth::user()->id)->first();
        $userDetail->profile_image = 'null';
        $userDetail->save();
        $upload = Upload::Where('id', decrypt($img_id))->delete();
        $this->dispatchBrowserEvent('swal:success',[ 'type'=>'success','msg'=>'Image deleted successfully' ]);
    }
    public function profile_edit($input){
       
        Validator::make($input, [
            'first_name'     => 'required|min:2|max:15',
            'last_name'      => 'required|min:2|max:15',
            'phone'         => 'required|min:7|max:20',
        ])->validate();
        $user =  UserDetails::where('user_id','=',Auth::user()->id)->first();

        //if new user and not completed profile after completion redirect to dashboard     
        $checknew=false;
        if(empty($user->phone))
        {
            $checknew=true;
        }
        
        $user->first_name        = $input['first_name'];
        $user->last_name         = $input['last_name'];
        $user->phone             = $input['phone'];
        if ($user->save()){
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Profile Updated Successfully']);
        }else{
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Something went wrong try again']);
        }

        //if new user and not completed profile after completion redirect to dashboard       
        if($checknew==true)
        {
            return redirect()->route('seller.dashboard');
        }
    }
    public function resetImage()
    {
        $this->image = null;
    }
    public function render()
    {
        return view('livewire.seller.profile.user-info');
    }
}
