<?php

namespace App\Http\Livewire\Seller\RePricing;
use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotCompetitionHistory;
use Livewire\WithPagination;
use Auth;
use App\Http\Controllers\Seller\RePricingController;
use App\Http\Controllers\Seller\OffersController;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
use App\Http\Controllers\Seller\ManualRePricingController;

class Index extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $plidHistory=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];
    public $buybox=null;
    public $byPricingFilter=null;
    public $rePricingStatus=null;
    public $stockFilter=null;
    public $rePricingDetails=null;
    public $OfferStatus=null;
    // protected $listeners = ['importAllBuyable','removeInActive','getPlidHistory'];
    protected $listeners = ['getPlidHistory','reCheckWinner','updatetakealotOffer','repricedata'];
   
    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         }
         $this->active_shop_id=session('active_shop_id');
        //end active shop id

        $this->rePricingStatus='enabled';
    }
    public function repricedata($offerid)
    {      
       $this->rePricingDetails=TakealotRePricing::with('offer')->where([['id','=',decrypt($offerid)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
       $this->dispatchBrowserEvent('popupnew');
    }
    public function closeModel()
    {
        $this->rePricingDetails=[];
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    }
    public function removeInActive()
    {
        if(TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['status','=',0]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Removed All In-Active Offers','move'=>0]);
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Something went wrong','move'=>0]);
        }

    }   
    
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function updatedStatus()
    {
        $this->resetPage();
    }
    public function updatedBuybox()
    {
        $this->resetPage();
    }
    public function updatedPriceSort()
    {
        $this->resetPage();
    }
    public function updatedRePricingStatus()
    {
        $this->resetPage();
    }
    public function updatedStockFilter()
    {
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingStatus=null;
         $this->stockFilter=null;
         $this->byPricingFilter=null;
         $this->resetPage();

    }
    public function getPlidHistory($tsin)
    {
        $this->plidHistory=TakealotCompetitionHistory::where([['tsin','=',decrypt($tsin)]])->orderBy('created_at', 'desc')->limit(20)->get();
       
    } 
    
    public function importAllBuyable()
    {   
        $allBuyable=TakealotOffers::doesntHave('rePricing')->where([['status','=','Buyable'],['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]])->get();  
        if(!empty($allBuyable))
        {
            foreach($allBuyable as $buyable)
            {
                $checkAlready=TakealotRePricing::where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['takealot_offers_id','=',$buyable->id]])->first();
                if(empty($checkAlready))
                {
                    $rePricing= new TakealotRePricing();
                    $rePricing->seller_id=$this->sellerID;
                    $rePricing->takealot_shops_id=$this->active_shop_id;
                    $rePricing->takealot_offers_id= $buyable->id;
                    $rePricing->min_price=0;
                    $rePricing->max_price=0;
                    if($rePricing->save())
                    {
                      
                    }
         
                }
            }
        }
        
    }
    public function updateOfferDetails($input)
    {
        Validator::make($input, [
            'minPrice'  => 'required|min:1|max:9999999',             
            'maxPrice'  => 'required|min:1|max:9999999',             
            'my_soh'    => 'required|min:1|max:9999999',             
            'updateFor' => 'required',             
        ])->validate();
        
        if(empty($input['my_soh']))
        {
            $input['my_soh']=0;
        }
        
        if(($input['minPrice']>0 && $input['maxPrice']>0) && $input['minPrice']<$input['maxPrice'])
        {
            $updatePrice=TakealotRePricing::with('offer','takealotShopDetails')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $oldOfferData=offerMap($updatePrice->offer);
                $updatePrice->min_price=$input['minPrice'];
                $updatePrice->max_price=$input['maxPrice'];
                $updatePrice->ip=getUserIp();
                $updatePrice->updated_through=1;
                $updatePrice->status=1;                
                if($updatePrice->save())
                {
                   
                    $newData=repricingMap($updatePrice,1);                    
                    if($updatePrice->status==1)
                    {                  
                       
                            $leadtime_decode=json_decode($updatePrice->offer->leadtime_stock);                          
                            if(!empty($leadtime_decode[0]) && ($input['my_soh']>$leadtime_decode[0]->quantity_available || $input['my_soh']<$leadtime_decode[0]->quantity_available))
                            {
                                
                                $leadtime_decode[0]->quantity_available=$input['my_soh'];
                                $leadtime_encode=json_encode($leadtime_decode);
                                $updatePrice->offer->leadtime_stock=$leadtime_encode;
                                $updatePrice->offer->ip=getUserIp();
                                if($updatePrice->offer->save())
                                { 
                                    $newOfferData=offerMap($updatePrice->offer,1);                                  
                                }
                                
                               
                            }
                        
                        if($updatePrice->offer->selling_price<$updatePrice->min_price)
                        {
                            //start update new price
                              $updatePrice->offer->selling_price=$updatePrice->min_price;                              
                              if($updatePrice->offer->save())
                              {
                                $updateOfferPrice=new ManualRePricingController();
                                $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->min_price,$updatePrice->takealotShopDetails,$updatePrice); 
                                // $updateOfferPrice=new OffersController();
                                // $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->min_price,$updatePrice->takealotShopDetails,$updatePrice); 
                             
                              }
                            //end update new price 
                        }
                        else
                        {
                            //start update new price
                            $updatePrice->offer->selling_price= $updatePrice->offer->selling_price;                              
                            if($updatePrice->offer->save())
                            {
                                $updateOfferPrice=new ManualRePricingController();
                                $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->min_price,$updatePrice->takealotShopDetails,$updatePrice); 
                                      
                            //   $updateOfferPrice=new OffersController();
                            //   $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->offer->selling_price,$updatePrice->takealotShopDetails,$updatePrice); 
                           
                            }
                          //end update new price 
                        }
                       
                        

                        $repricing=new RePricingController();
                        $repricing->repriceVariantParams(decrypt($input['updateFor']),'manual');
                        $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Price Updated Successfully','move'=>0]);

                    }  
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);
                    }
                }
                else
                {
                    $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                }
                
            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

            }
            
        }
        else       
        {
            
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Min and max price must be greater than 0','move'=>0]);

        }
         
        
    }
    public function updatetakealotOffer($offer_id)
    {
      
        $offControl=new OffersController();
        $reftechOffer=TakealotOffers::with('rePricing','takealotShops')->where([['id','=',decrypt($offer_id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->orderBy('id','asc')->first();
        if(!empty($reftechOffer))
        {

            $flag=$offControl->reFetchOffer($reftechOffer);    
            $this->data=TakealotRePricing::with('offer','takealotOfferDetails')->where([['id','=',$reftechOffer->rePricing->id],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Offer Updated Successfully','move'=>0]);

        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Offer Not Updated','move'=>0]);
        }
      
        
    }
    public function updateOfferStatus($id)
    {            
        $updateStatus=TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
        if(!empty($updateStatus))
        {
          
            $updateStatus->status=( $updateStatus->status==1) ? 0 : 1;
            if($updateStatus->save())
            {              
                $this->rePricingDetails->status=$updateStatus->status;
                $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Status updated successfully','move'=>0]);
            }
            else
            {
              $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);
            }
        }
    }
    public function render()  
    {
        $activeShopID=$this->active_shop_id;
        $activeSellerID=$this->sellerID;
        $takealotRePricing = $this->readyToLoad ? TakealotRePricing::select('takealot_repricing.*')->with('offer','offer.takealotPlidHistory','offer.takealotCompetitionHistory','takealotOfferDetails')
        ->join('takealot_offers', function($query)  use ($activeShopID,$activeSellerID)
        {
            $query->on('takealot_offers.id', '=', 'takealot_repricing.takealot_offers_id')
            ->when($this->byPricingFilter,function($query, $byPricingFilter)  use ($activeShopID,$activeSellerID){   
                if($byPricingFilter=='min')
                {
                    $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereColumn('takealot_offers.selling_price', 'takealot_repricing.min_price');
                }  
                if($byPricingFilter=='max')
                {
                    $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereColumn('takealot_offers.selling_price', 'takealot_repricing.max_price');
                } 
                    
            }); 
        })
        ->when($this->buybox,function($query, $buybox)  use ($activeShopID,$activeSellerID){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->where([['takealot_repricing.takealot_shops_id','=',$this->active_shop_id],['takealot_repricing.seller_id','=',$this->sellerID]])->where('takealot_repricing.winner', $buybox);
           })
        ->when($this->rePricingStatus,function($query, $rePricingStatus)  use ($activeShopID,$activeSellerID){
            $rePricingStatus=($rePricingStatus=='enabled') ? 1 : 0;            
            $query->where([['takealot_repricing.takealot_shops_id','=',$this->active_shop_id],['takealot_repricing.seller_id','=',$this->sellerID]])->where('takealot_repricing.status', $rePricingStatus);
           })
        ->when($this->search,function($query, $search) use ($activeShopID,$activeSellerID){
            $search=trim($search);
            $query->whereHas('offer',fn($query) => $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%"));
           })
        ->when($this->status,function($query, $status) use ($activeShopID,$activeSellerID){
            $query->whereHas('offer',fn($query) => $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereIn('takealot_offers.status',$status));
           })
        ->when($this->stockFilter,function($query, $stockFilter) use ($activeShopID,$activeSellerID){

            if($stockFilter=='leadtime')
            {
                $query->whereHas('offer',fn($query) => $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) > 0"));
            }
            if($stockFilter=='instock')
            {
                $query->whereHas('offer',fn($query) => $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->where('takealot_offers.stock_at_takealot_total','>',0));
            }
            if($stockFilter=='outofstock')
            {
                $query->whereHas('offer',fn($query) => $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereRaw("json_unquote(json_extract(`leadtime_stock`, '$[0].quantity_available')) <= 0")->where('takealot_offers.stock_at_takealot_total','<=',0));
            }

        
         })
        ->where([['takealot_repricing.takealot_shops_id','=',$this->active_shop_id],['takealot_repricing.seller_id','=',$this->sellerID]]) : [];
       
        if(!empty($takealotRePricing))
        {
        if(!empty($this->priceSort))
        {
           if($this->priceSort=='min')
           {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.min_price', $this->orderBy)->paginate($this->entries);
           }
           else if($this->priceSort=='max')
           {

            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.max_price', 'asc')->paginate($this->entries);

           }
           else
           {

             $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.id', 'desc')->paginate($this->entries);

           }

        }
        else
        {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.price_updated_at', $this->orderBy)->paginate($this->entries);
        }
        }
        if(!empty($this->plidHistory))
        {
            
            $this->dispatchBrowserEvent('load-modal',['modelID'=> 'exampleModalScrollable']);
        }
        else
        {
            $this->dispatchBrowserEvent('close-modal',['modelID'=> 'exampleModalScrollable']);
        }
       
        return view('livewire.seller.re-pricing.index',['takealotRePricing'=>$takealotRePricing,'plidHistory'=>$this->plidHistory]);
    }
}
