<?php

namespace App\Http\Livewire\Seller\RePricingHistory;
use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotCompetitionHistory;
use Livewire\WithPagination;
use Auth;
use App\Http\Controllers\Seller\RePricingController;
use App\Http\Controllers\Seller\OffersController;
use App\Jobs\TakealotInstantRepricingJob;
use Illuminate\Support\Facades\Validator;
class RepricingHistory extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $plidHistory=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $priceSort=null;
    public $search=null;
    public $status=[];
    public $buybox=null;
    public $rePricingStatus=null;
    
    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 30;
         $this->orderBy  = 'desc';
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         }
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    } 
    public function removeFromRePricing($id)
    {
        if($rePricing=TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->delete())
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Successfully Removed From Re-Pricing List','move'=>0]);

        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
       
    }
    public function updateStatus($id)
    {
      
        $updateStatus=TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
        if(!empty($updateStatus))
        {
            $updateStatus->status=( $updateStatus->status==1) ? 0 : 1;
            if($updateStatus->save())
            {
                 $status='Disabled';
                 if($updateStatus->status==1)
                 {
                    $status='Enabled';
                    if(!empty($updateStatus->min_price) && !empty($updateStatus->max_price))
                    {
                        $repricing=new RePricingController();
                        $repricing->repriceVariantParams($updateStatus->id,'manual');
                    }
                    
                 }
                $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Successfully '.$status,'move'=>0]);

            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

            }
        }
    }
    public function updateMinPrice($price,$id)
    {
        if($price>0)
        {
          
            $updatePrice=TakealotRePricing::with('offer','takealotShopDetails')->where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->min_price=$price;
                if($updatePrice->save())
                {
                    if(($updatePrice->min_price>0 && $updatePrice->max_price>0) && $updatePrice->min_price<$updatePrice->max_price)
                    {

                        $updatePrice->status=1;
                        $updatePrice->save();

                        if($updatePrice->status==1)
                        {
                            $repricing=new RePricingController();
                            $repricing->repriceVariantParams(decrypt($id),'manual');
                            //TakealotInstantRepricingJob::dispatch($id)->onQueue('high')->beforeCommit();
                        }
                      
                          
                       
                    }
                   
                    $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Min Price Updated Successfully','move'=>0]);
                }
            }
        }
        
    }
    public function updateMaxPrice($price,$id)
    {
        if($price>0)
        {
           
            $updatePrice=TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->max_price=$price;
                if($updatePrice->save())
                {
                    if(($updatePrice->min_price>0 && $updatePrice->max_price>0) && $updatePrice->min_price<$updatePrice->max_price)
                    {
                        $updatePrice->status=1;
                        $updatePrice->save();
                        if($updatePrice->status==1)
                        {
                            $repricing=new RePricingController();
                            $repricing->repriceVariantParams(decrypt($id),'manual');
                            //TakealotInstantRepricingJob::dispatch($id)->onQueue('high')->beforeCommit();
                        }
                       
                    }
                    $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Max Price Updated Successfully','move'=>0]);
                }
            }
        }
    }
    public function updateOfferPrice($input)
    {
        Validator::make($input, [
            'minPrice' => 'required|min:1|max:12',             
            'maxPrice' => 'required|min:1|max:12',             
            'updateFor' => 'required',             
        ])->validate();
        
        if(($input['minPrice']>0 && $input['maxPrice']>0) && $input['minPrice']<$input['maxPrice'])
        {
            $updatePrice=TakealotRePricing::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->min_price=$input['minPrice'];
                $updatePrice->max_price=$input['maxPrice'];
                $updatePrice->status=1;
                if($updatePrice->save())
                {
                    if($updatePrice->status==1)
                    {
                        $repricing=new RePricingController();
                        $repricing->repriceVariantParams(decrypt($input['updateFor']),'manual');
                        $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Price Updated Successfully','move'=>0]);

                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                    }
                }
                else
                {
                    $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                }
                
            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

            }
            
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Min and max price must be greater than 0','move'=>0]);

        }
         
        
    }
    public function updatedSearch()
    {
        // $this->clearFilter(true);
        $this->resetPage();
    }
    public function updatedStatus()
    {
        $this->resetPage();
    }
    public function updatedBuybox()
    {
        $this->resetPage();
    }
    public function updatedPriceSort()
    {
        $this->resetPage();
    }
    public function updatedRePricingStatus()
    {
        $this->resetPage();
    }
    public function clearFilter($searchCheck=false)
    {
        
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->priceSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingStatus=null;

    }
    public function getPlidHistory($tsin)
    {
        $this->plidHistory=TakealotCompetitionHistory::where([['tsin','=',decrypt($tsin)]])->orderBy('id', 'desc')->get();
       
    }
    public function closeModel()
    {
        $this->plidHistory=[];
    }
    public function render()  
    {
        $takealotRePricing =TakealotRePricing::with('offer','offer.takealotPlidHistory','offer.takealotCompetitionHistory','takealotOfferDetails')
        ->when($this->buybox,function($query, $buybox){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->where('takealot_repricing.winner', $buybox);
           })
        ->when($this->rePricingStatus,function($query, $rePricingStatus){
            $rePricingStatus=($rePricingStatus=='enabled') ? 1 : 0;
            $query->where('takealot_repricing.status', $rePricingStatus);
           })
        ->when($this->search,function($query, $search){
            $search=trim($search);
            $query->whereHas('offer',fn($query) => $query->where('takealot_offers.title', 'LIKE', "%{$search}%")->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%"));
           })
        ->when($this->status,function($query, $status){
            $query->whereHas('offer',fn($query) => $query->whereIn('takealot_offers.status',$status));
           })
        ->where([['takealot_shops_id','=',$this->active_shop_id],['seller_id','=',$this->sellerID]]);
       
        if(!empty($this->priceSort))
        {
           if($this->priceSort=='min')
           {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.min_price', $this->orderBy)->paginate($this->entries);
           }
           else if($this->priceSort=='max')
           {

            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.max_price', 'asc')->paginate($this->entries);

           }
           else
           {

             $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.created_at', 'desc')->paginate($this->entries);

           }

        }
        else
        {
            $takealotRePricing=$takealotRePricing->orderBy('takealot_repricing.created_at', $this->orderBy)->paginate($this->entries);
        }
        
        if(!empty($this->plidHistory))
        {
            $this->dispatchBrowserEvent('load-modal',['modelID'=> 'exampleModalScrollable']);
        }
        else
        {
            $this->dispatchBrowserEvent('close-modal',['modelID'=> 'exampleModalScrollable']);
        }
        return view('livewire.seller.re-pricing-history.repricing-history',['takealotRePricing'=>$takealotRePricing]);
    }
}
