<?php

namespace App\Http\Livewire\Seller\Replenishment;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotOrders;
use App\Models\TakealotRePricing;
use App\Models\TakealotShops;
use App\Http\Controllers\Seller\OrdersController;
use App\Http\Controllers\Seller\OffersController;
use App\Http\Controllers\Seller\RePricingController;
use App\Http\Controllers\Seller\ManualRePricingController;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Auth;
use Livewire\WithPagination;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use PDF;
// use Maatwebsite\Excel\Concerns\WithMultipleSheets;
class AllList extends Component
{
    use WithPagination;
    protected $paginationTheme = 'bootstrap';
    public $readyToLoad    = false;
    public $sellerID=null;
    public $active_shop_id=null;
    public $view='list';

    public $entries;    
    public $orderBy='desc';
    public $orderSort=null;
    public $search=null;
    public $status=['Buyable'];
    public $buybox=null;
    public $rePricingFilter=null;
    public $rePricingDetails;
    public $other_status;
    public $editSelling=null;
    public $editofferCost=null;
    public $editTakealotCommission=null;
    public $editOtherCost=null;
    public $editTotalCost=null;
    public $pricesDetails=null;
    public $pricingdataID=null;
    protected $listeners = ['getRepricePopup','reCheckWinner','updatetakealotOffer','pricingdata'];
    
    public function mount()
    {
         $this->sellerID         = Auth::user()->id;
         $this->entries  = 30;
         $this->other_status  = 0;
         $this->orderBy  = 'desc';
         $this->rePricingDetails=[];
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         }
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
        $this->resetVariables();
    }
    public function readyToLoad()
    {
        $this->readyToLoad = true;
    }
    public function pricingdata($repriceID)
    {      
       $this->pricingdataID=$repriceID;
       $this->pricesDetails=TakealotOffers::with('rePricing')->where([['id','=',decrypt($repriceID)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
       $this->dispatchBrowserEvent('popupnew',['id'=>'updatePricesRow']);
      
    }
    public function closeModel()
    {
        $this->rePricingDetails=[];
        $this->pricesDetails=[];
        $this->pricingdataID=[];
        $this->resetVariables();
    }
        
    public function resetVariables()
    {
        
        $this->editSelling=null;
        $this->editofferCost=null;
        $this->editTakealotCommission=null;
        $this->editOtherCost=null;
        $this->editTotalCost=null;
        $this->OfferStatus=null;
        $this->pricingdataID=null;
    }
    public function offerCostUpdate($input)
    {
        Validator::make($input, [
            'offer_cost'  => 'required|min:1|max:99999999',             
            'updateFor'  => 'required',             
        ])->validate();
        //update min price

        if($input['offer_cost']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->offer_cost=$input['offer_cost'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->offer_cost=$updatePrice->offer_cost;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Product cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function sellingPriceUpdate($input)
    {
          
        Validator::make($input, [
            'selling_price'  => 'required|min:1|max:99999999',  
            'updateFor'  => 'required',              
        ])->validate();
        if($input['selling_price']>0)
        {
          
            $updatePrice=TakealotOffers::with('takealotShops')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->selling_price=$input['selling_price'];
                if($updatePrice->save())
                {
                    $this->pricesDetails->selling_price=$updatePrice->selling_price;
                    $updateOfferPrice=new OffersController();
                    $result=$updateOfferPrice->updateSellingPrice($updatePrice->takealotShops,$updatePrice->offer_id,$updatePrice->selling_price);
                    if($result==true)
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Selling price updated successfully','move'=>0]);
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong'.$result,'move'=>0]);
                    }
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function takealotCommission($input)
    {
        Validator::make($input, [
            'takealot_commission'  => 'required|min:1|max:99999999', 
            'updateFor'  => 'required',               
        ])->validate();
        //update min price

        if($input['takealot_commission']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->takealot_commission=$input['takealot_commission'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->takealot_commission=$updatePrice->takealot_commission;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Takealot commission updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function otherCost($input)
    {
        Validator::make($input, [
            'other_costs'  => 'required|min:1|max:99999999',     
            'updateFor'  => 'required',           
        ])->validate();
        //update min price

        if($input['other_costs']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->other_costs=$input['other_costs'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->other_costs=$updatePrice->other_costs;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Other cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function  updatedother_status($value)
    {
        if($value<=3)
        {
            $this->other_status=$value;
        }
        else
        {
            $this->other_status=0;
        }
        
    }
    public function getRepricePopup($id)
    {
        $this->rePricingDetails=TakealotRePricing::with('offer')->where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
       
    }
    public function updateStatus($id)
    {
      
        $updateStatus=TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
        if(!empty($updateStatus))
        {
            $oldData=repricingMap($updateStatus);
            $updateStatus->status=( $updateStatus->status==1) ? 0 : 1;
            if($updateStatus->save())
            {
                 $status='Disabled';
                 $this->data->status=$updateStatus->status;
                 if($updateStatus->status==1)
                 {
                    $status='Enabled';
                    if(!empty($updateStatus->min_price) && !empty($updateStatus->max_price))
                    {
                        // $repricing=new RePricingController();
                        // $repricing->repriceVariantParams($updateStatus->id,'manual');
                        $repricing=new ManualRePricingController();        
                        $response=$repricing->repriceVariantParams($updateStatus->id);        
                        if(!empty($response))
                        {
                            $responseData = json_decode($response->getContent(), true);
                            // Access specific data from the array
                            if(!empty($responseData) && $responseData['status'] && $responseData['message'])
                            {
                                $status = $responseData['status'];
                                $message = $responseData['message'];
                                $this->dispatchBrowserEvent('notify', ['error' => $status,'msg'=> $message,'move'=>0]);               
                
                            }
                            else
                            {
                                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                            }
                            
                        }
                        else
                        {
                            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
                        }
                    }
                    
                 }
                 $newData=repricingMap($updateStatus,1);
                 $diff2=array_diff($oldData,$newData);
                 if((count($diff2)==1) && !empty($diff2['updated_at']))
                 {
                     $diff2=[];
                 }
                 if(!empty($diff2))
                 {
                    //  TakealotTsinUpdateHistoryJob::dispatch($oldData,$newData,'repricing')->onQueue('default'); 
                    $repricing=new OffersController();
                    $repricing->tsinUpdateHistory($oldData,$newData,'repricing');     
                 }


                // $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Successfully '.$status,'move'=>0]);

            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

            }
        }
    }
    public function updateOfferPrice($input)
    {
        $checkUserPackageOffers=checkUserPackageOffers($this->sellerID);
        if($checkUserPackageOffers==true)
        {
                Validator::make($input, [
                    'minPrice'  => 'required|min:1|max:12',             
                    'maxPrice'  => 'required|min:1|max:12',             
                    'my_soh'    => 'required|min:1|max:12',             
                    'updateFor' => 'required',             
                ])->validate();
                
                if(empty($input['my_soh']))
                {
                    $input['my_soh']=0;
                }
                if(($input['minPrice']>0 && $input['maxPrice']>0) && $input['minPrice']<$input['maxPrice'])
                {
                    $updatePrice=TakealotRePricing::with('offer','takealotShopDetails')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
                    if(empty($updatePrice) && !empty($input['minPrice']) && !empty($input['maxPrice']))
                    {
                        $addOffer= new TakealotRePricing();
                        $addOffer->seller_id=$this->sellerID;
                        $addOffer->takealot_shops_id=$this->active_shop_id;
                        $addOffer->takealot_offers_id= decrypt($input['updateFor']);
                        $addOffer->min_price=$input['minPrice'];
                        $addOffer->max_price=$input['maxPrice'];
                        $addOffer->updated_through=1;
                        $addOffer->status=1;
                        $addOffer->save();
                        $updatePrice=TakealotRePricing::with('offer','takealotShopDetails')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();

                    }
                    if(!empty($updatePrice))
                    {
                        // $oldData=repricingMap($updatePrice);
                        $oldOfferData=offerMap($updatePrice->offer);
                        $updatePrice->min_price=$input['minPrice'];
                        $updatePrice->max_price=$input['maxPrice'];
                        $updatePrice->ip=getUserIp();
                        $updatePrice->updated_through=1;
                        $updatePrice->status=1;
                        if($updatePrice->save())
                        {
                        
                        
                            $newData=repricingMap($updatePrice,1);                    
                            if($updatePrice->status==1)
                            {                 
                            
                                    $leadtime_decode=json_decode($updatePrice->offer->leadtime_stock);                          
                                    if(!empty($leadtime_decode[0]) && ($input['my_soh']>$leadtime_decode[0]->quantity_available || $input['my_soh']<$leadtime_decode[0]->quantity_available))
                                    {                                
                                        $leadtime_decode[0]->quantity_available=$input['my_soh'];
                                        $leadtime_encode=json_encode($leadtime_decode);
                                        $updatePrice->offer->leadtime_stock=$leadtime_encode;
                                        $updatePrice->offer->ip=getUserIp();
                                        if($updatePrice->offer->save())
                                        { 
                                            $newOfferData=offerMap($updatePrice->offer,1);
                                        }
                                        
                                    
                                    }
                                
                                

                                if($updatePrice->offer->selling_price<$updatePrice->min_price)
                                {
                                    //start update new price
                                    $updatePrice->offer->selling_price=$updatePrice->min_price;                              
                                    if($updatePrice->offer->save())
                                    {

                                        // $updateOfferPrice=new OffersController();
                                        $updateOfferPrice=new ManualRePricingController();
                                        $updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->min_price,$updatePrice->takealotShopDetails,$updatePrice); 
                                    
                                    }
                                    //end update new price 
                                }
                                else
                                {
                                    //start update new price
                                    $updatePrice->offer->selling_price= $updatePrice->offer->selling_price;                              
                                    if($updatePrice->offer->save())
                                    {
                                        
                                    $updateOfferPrice=new OffersController();
                                    $updatedOffer=$updateOfferPrice->updateTakealotOffer($updatePrice->offer->offer_id,$updatePrice->offer->selling_price,$updatePrice->takealotShopDetails,$updatePrice); 
                                    
                                    }
                                //end update new price 
                                }                   
                            

                                $repricing=new RePricingController();
                                $repricing->repriceVariantParams(decrypt($input['updateFor']),'manual');    
                                $this->rePricingDetails=[];                   
                                $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Price Updated Successfully','move'=>0]);

                            }  
                            else
                            {

                                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                            }
                        }
                        else
                        {
                            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                        }
                        
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Price Not Updated','move'=>0]);

                    }
                    
                }
                else       
                {
                    
                    $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Min and max price must be greater than 0','move'=>0]);

                }
        }
        else       
        {
            
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>"You are not able to add more offers in re-pricing. Please upgrade your package",'move'=>0]);

        }
         
        
    }
    public function updatetakealotOffer($offer_id)
    {      
        $reFetchOffer=new OffersController();
        $reftechOffer=TakealotOffers::with('rePricing','takealotShops')->where([['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id],['id','=',decrypt($offer_id)]])->orderBy('id','asc')->first();
        if(!empty($reftechOffer))
        {
            $flag=$reFetchOffer->reFetchOffer($reftechOffer);    
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Offer Updated Successfully','move'=>0]);
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Offer Not Updated','move'=>0]);
        }      
        
    }
    public function reCheckWinner($id)
    {        
        // $repricing=new RePricingController();
        // $repricing->repriceVariantParams(decrypt($id),'manual');
        // $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Winner Re-Checked Successfully','move'=>0]);
        $repricing=new ManualRePricingController();        
        $response=$repricing->repriceVariantParams(decrypt($id));
       
        if(!empty($response))
        {
            $responseData = json_decode($response->getContent(), true);
            // Access specific data from the array
            if(!empty($responseData) && $responseData['status'] && $responseData['message'])
            {
                $status = $responseData['status'];
                $message = $responseData['message'];
                $this->data=TakealotRePricing::with('offer','takealotOfferDetails')->where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
                $this->dispatchBrowserEvent('notify', ['error' => $status,'msg'=> $message,'move'=>0]);               

            }
            else
            {
                $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
            }
            
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=> 'Sorry! Something went wrong.','move'=>0]);
        }

    }
    public function removeFromRePricing($id)
    {
        if(TakealotRePricing::where([['id','=',decrypt($id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->delete())
        {
            $this->data=[];
            $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Successfully Removed From Re-Pricing List','move'=>0]);

        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
       
    }      
    public function updatedSearch()
    {
        $this->clearFilter(true);
        $this->resetPage();
    }
    public function exportPdf()
    {
        ini_set('max_execution_time', -1);
        $takealotOffers = $this->TakealotOffersQuery->limit(5)->get(); 
        $data = [           
            'model' => $takealotOffers
        ];
        $pdf = PDF::loadView('components.replenishment-pdf',$data)->setOptions(['defaultFont' => 'sans-serif']);;  
        return $pdf->download('pdfview.pdf');
        
       
    }
    public function allrefetchoncurrentPage()
    {
       
        foreach($this->TakealotOffers as $item)
        {
            $this->reFtechData(encrypt($item->tsin_id),encrypt($item->offer_id));
        }
    }
    public function reFtechData($tsin,$offer_id)
    {
        
        $shops=TakealotShops::with('checkShopSubscription')->where([['seller_id','=',$this->sellerID],['id','=',$this->active_shop_id]])->orderBy('created_at','asc')->first();
        if(!empty($shops))
        {
            $orders=new OrdersController();
            $orders->updateSingleOrder($shops,decrypt($tsin));
            
                $reFetchOffer=new OffersController();
                $reftechOffer=TakealotOffers::with('rePricing','takealotShops')->where([['offer_id','=',decrypt($offer_id)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->orderBy('id','asc')->first();
               
                if(!empty($reftechOffer))
                {
        
                    $flag=$reFetchOffer->reFetchOffer($reftechOffer);    
                  
                }
               
        }
    }
    public function clearFilter($searchCheck=false)
    {
       
         $this->search=($searchCheck==false) ? null : $this->search;
         $this->orderSort='';
         $this->status=[];
         $this->buybox=null;
         $this->rePricingFilter=null;

    }
    public function getTakealotOffersQueryProperty()
    {
        $activeShopID=$this->active_shop_id;
        $activeSellerID=$this->sellerID;
        $thirtyDays=Carbon::now()->subDays(30)->toDateString();
        $takealotOffers=TakealotOffers::distinct('takealot_offers.id')->select('takealot_offers.id','takealot_offers.offer_id','takealot_offers.offer_cost','takealot_offers.other_status','takealot_offers.seller_id','takealot_offers.leadtime_stock','takealot_offers.stock_at_takealot_total','takealot_offers.leadtime_days','takealot_offers.takealot_shops_id','takealot_offers.stock_cover','takealot_offers.stock_on_way','takealot_offers.offer_url','takealot_offers.tsin_id','takealot_offers.sku','takealot_offers.stock_at_takealot','takealot_offers.status','takealot_offers.title','takealot_offers.image_url','takealot_offers.selling_price','takealot_offers.total_stock_on_way','takealot_offers.date_created','takealot_offers.sales_units', DB::raw('(SELECT SUM(quantity) FROM takealot_orders WHERE offer_id = takealot_offers.offer_id AND takealot_orders.sale_status NOT IN (\'Cancelled\', \'Refunded\', \'Cancelled by Customer\', \'Cancelled - Late\', \'Cancelled by Takealot - DC Stock Inquiry\', \'Cancelled - Inbound Exception (Short Supplied)\')  AND takealot_orders.order_date >= DATE_SUB(NOW(), INTERVAL 30 DAY)) as thirty_day_sales_quantity'))->with('takealotShops','takealotOrders','rePricing','takealotOfferDetails') 
         
        ->leftJoin('takealot_orders', function($join) use ($thirtyDays,$activeShopID,$activeSellerID) {
            $join->on('takealot_offers.id', '=', 'takealot_orders.offer_id')
                ->where('takealot_orders.order_date', '>=',$thirtyDays)
                ->where([['takealot_orders.seller_id','=',$activeSellerID],['takealot_orders.takealot_shops_id','=',$activeShopID]]);
        })
        ->when($this->search,function($query, $search) use ($activeShopID,$activeSellerID)
        {
            $search=trim($search);
            $query->whereIn('takealot_offers.other_status', [0,1, 2, 3])->where(function($query) use ($search,$activeShopID,$activeSellerID) {
                $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])
                ->where('takealot_offers.title', 'LIKE', "%{$search}%")
                      ->orWhere('takealot_offers.sku', 'LIKE', "%{$search}%")
                      ->orWhere('takealot_offers.tsin_id', 'LIKE', "%{$search}%");
            });
        })
           
        ->when($this->status,function($query, $status) use ($activeShopID,$activeSellerID){

             $query->where([['takealot_offers.takealot_shops_id','=',$activeShopID],['takealot_offers.seller_id','=',$activeSellerID]])->whereIn('takealot_offers.status',$status);

            })
        ->when($this->buybox,function($query, $buybox)  use ($activeShopID,$activeSellerID){
            $buybox=($buybox=='winner') ? 1 : 0;
            $query->whereHas('rePricing',fn($query) => $query->where([['takealot_repricing.takealot_shops_id','=',$activeShopID],['takealot_repricing.seller_id','=',$activeSellerID]])->where('takealot_repricing.winner', $buybox)
           );
           })
        ->when($this->rePricingFilter,function($query, $rePricingFilter)  use ($activeShopID,$activeSellerID){   
              if($rePricingFilter=='yes')
              {
                $query->whereHas('rePricing',fn($query) => $query->where([['takealot_repricing.takealot_shops_id','=',$activeShopID],['takealot_repricing.seller_id','=',$activeSellerID]])->where('takealot_repricing.winner','=',1)->orWhere('takealot_repricing.winner','=',0));

              }
              else
              {
                $query->doesntHave('rePricing');
              }
     
           });
           if(empty($this->search))
           {
            $takealotOffers=$takealotOffers->where([['takealot_offers.other_status','=',$this->other_status],['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID]]);           
           }  
           else
           {
            $takealotOffers=$takealotOffers->where([['takealot_offers.takealot_shops_id','=',$this->active_shop_id],['takealot_offers.seller_id','=',$this->sellerID]]);          
           }
           $takealotOffers=$takealotOffers->groupBy('takealot_offers.id')->orderBy('thirty_day_sales_quantity', $this->orderBy);   
            
           return $takealotOffers;
    }
    public function getTakealotOffersProperty()
    {
         return $this->TakealotOffersQuery->paginate($this->entries);
    }
    public function render()
    {
        $takealotOffers =$this->readyToLoad ? $this->TakealotOffers : [];

        if(!empty($this->rePricingDetails))
        {
            
            $this->dispatchBrowserEvent('load-modal',['modelID'=> 'exampleModalScrollable']);
        }
        else
        {
            $this->dispatchBrowserEvent('close-modal',['modelID'=> 'exampleModalScrollable']);
        }

        if(!empty($this->pricesDetails) && !empty($this->pricingdataID))
        {
         $this->pricingdata($this->pricingdataID);
        }
        return view('livewire.seller.replenishment.all-list',['takealotOffers'=>$takealotOffers])->extends('seller.layouts.app');
    }
}
