<?php

namespace App\Http\Livewire\Seller\Replenishment;

use Livewire\Component;
use App\Models\TakealotOffers;
use App\Models\TakealotRePricing;
use App\Models\TakealotShops;
use App\Http\Controllers\Seller\OffersController;
use App\Http\Controllers\Seller\RePricingController;
use App\Jobs\TakealotInstantRepricingJob;
use App\Jobs\TakealotTsinUpdateHistoryJob;
use Illuminate\Support\Facades\Validator;
use Auth;
use Carbon\Carbon;
class PriceEditModel extends Component
{
    public $editSelling=null;
    public $editofferCost=null;
    public $editTakealotCommission=null;
    public $editOtherCost=null;
    public $editTotalCost=null;
    public $pricesDetails=null;
    public $pricingdataID=null;
    public $popupkey=null;
    protected $listeners = ['singlepricingdata'];
    
    public function mount($repriceID,$newpopupkey)
    {
        
         $this->sellerID         = Auth::user()->id;
         $this->popupkey         = $newpopupkey;
         $this->rePricingDetails=[];
         //start check active shop
         if(empty(session('active_shop_id')))
         {
            $activeShop=Auth::user()->AllTakealotShops->first()->id;
            session(['active_shop_id' => (!empty($activeShop) ? $activeShop : 0)]);
         }
         $this->active_shop_id=session('active_shop_id');
        //end active shop id
        $this->resetVariables();
        $this->singlepricingdata($repriceID);
    }
    public function singlepricingdata($repriceID)
    {  
        $this->pricingdataID=$repriceID;
        
        $this->pricesDetails=TakealotOffers::with('rePricing')->where([['id','=',decrypt($repriceID)],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();      
        $this->dispatchBrowserEvent('singlepopupnew',['id'=>'updateSinglePricesRow'.$this->popupkey]);
      
    }
    public function closeModel()
    {
        $this->rePricingDetails=[];
        $this->pricesDetails=[];
        $this->pricingdataID=[];
        $this->emitUp('resetEdit');
        $this->resetVariables();
    }
        
    public function resetVariables()
    {
        
        $this->editSelling=null;
        $this->editofferCost=null;
        $this->editTakealotCommission=null;
        $this->editOtherCost=null;
        $this->editTotalCost=null;
        $this->OfferStatus=null;
        $this->pricingdataID=null;
    }
    public function offerCostUpdate($input)
    {
        Validator::make($input, [
            'offer_cost'  => 'required|min:1|max:99999999',             
            'updateFor'  => 'required',             
        ])->validate();
        //update min price

        if($input['offer_cost']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->offer_cost=$input['offer_cost'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->offer_cost=$updatePrice->offer_cost;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Product cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function sellingPriceUpdate($input)
    {
          
        Validator::make($input, [
            'selling_price'  => 'required|min:1|max:99999999',  
            'updateFor'  => 'required',              
        ])->validate();
        if($input['selling_price']>0)
        {
          
            $updatePrice=TakealotOffers::with('takealotShops')->where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->selling_price=$input['selling_price'];
                if($updatePrice->save())
                {
                    $this->pricesDetails->selling_price=$updatePrice->selling_price;
                    $updateOfferPrice=new OffersController();
                    $result=$updateOfferPrice->updateSellingPrice($updatePrice->takealotShops,$updatePrice->offer_id,$updatePrice->selling_price);
                    if($result==true)
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Selling price updated successfully','move'=>0]);
                    }
                    else
                    {
                        $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong'.$result,'move'=>0]);
                    }
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function takealotCommission($input)
    {
        Validator::make($input, [
            'takealot_commission'  => 'required|min:1|max:99999999', 
            'updateFor'  => 'required',               
        ])->validate();
        //update min price

        if($input['takealot_commission']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->takealot_commission=$input['takealot_commission'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->takealot_commission=$updatePrice->takealot_commission;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Takealot commission updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function otherCost($input)
    {
        Validator::make($input, [
            'other_costs'  => 'required|min:1|max:99999999',     
            'updateFor'  => 'required',           
        ])->validate();
        //update min price

        if($input['other_costs']>0)
        {
          
            $updatePrice=TakealotOffers::where([['id','=',decrypt($input['updateFor'])],['seller_id','=',$this->sellerID],['takealot_shops_id','=',$this->active_shop_id]])->first();
            if(!empty($updatePrice))
            {
                $updatePrice->other_costs=$input['other_costs'];
                $updatePrice->total_cost=$updatePrice->takealot_commission+ $updatePrice->other_costs+$updatePrice->offer_cost;
                if($updatePrice->save())
                {
                    $this->pricesDetails->other_costs=$updatePrice->other_costs;
                    $this->pricesDetails->total_cost=$updatePrice->total_cost;
                   $this->dispatchBrowserEvent('notify', ['error' => 'success','msg'=>'Other cost updated successfully','move'=>0]);
                }
            }
            
          
        }
        else
        {
            $this->dispatchBrowserEvent('notify', ['error' => 'error','msg'=>'Sorry! Somthing Went Wrong','move'=>0]);

        }
        $this->resetVariables();
    }
    public function render()
    {
        
        $this->dispatchBrowserEvent('singlepopupnew',['id'=>'updateSinglePricesRow'.$this->popupkey]);
        return view('livewire.seller.replenishment.price-edit-model');
    }
}
