<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use App\Mail\PackageExpiryMail;
use Mail;
use Illuminate\Queue\MaxAttemptsExceededException; // Import the exception class
class PackageExpiryMailJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    
    protected $expiryDate;
    protected $emailTo;
    public function __construct($expiryDate,$mailto)
    {

        $this->expiryDate      = $expiryDate;
        $this->emailTo         = $mailto;
        $this->onQueue('high');
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        Mail::to($this->emailTo)->send(new PackageExpiryMail($this->expiryDate));
    }
    public function failed(\Exception $exception)
    {
        // Release the job back into the queue with a delay of 1 hour
        $this->release(600); // 3600 seconds = 1 hour
    }
}
